@@ -88,10 +88,12 @@
 #endif
 
 #define PORTALINIT	0		// Portal states
-#define BOUND		1
-#define COPYINPROGRESS	2
-#define CLOSING		3
-#define CLOSED		4
+#define PARSING	  	1
+#define BOUND		2
+#define COMMITTED	3
+#define COPYINPROGRESS	4
+#define CLOSING		5
+#define CLOSED		6
 
 #define KEEP		0		// Sendcmd subcommands
 #define SENDOUT		1
@@ -102,3 +104,7 @@
 #define NOERROR			0	// Error states networkparser
 #define PROTOCOLERROR		1
 #define PROTOCOLUNSUPPORTED	2
+
+#define NOTRANS         0
+#define TRANSBEGIN      1
+#define TRANSEND        2
