@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 0282cd5c2503938fa0a5eb558532de84ce4057e7 $
 */
 
 #ifndef MACHINE_H
@@ -41,6 +40,9 @@
 /* Define this if you want pike to interact with valgrind. */
 #undef USE_VALGRIND
 
+/* Define this to embed DTrace probes */
+#undef USE_DTRACE
+
 /* Define this if you are going to use a memory access checker (like Purify) */
 #undef __CHECKER__
 
@@ -68,21 +70,15 @@
 /* Define this to enable the internal Pike security system */
 #undef PIKE_SECURITY
 
-/* Define this to enable experimental facets support */
-#undef WITH_FACETS
+/* Define this to simulate dynamic module loading with static modules. */
+#undef USE_SEMIDYNAMIC_MODULES
 
 /* Define this to enable the internal bignum conversion */
 #undef AUTO_BIGNUM
 
-/* Define this to enable experimental code for multicpu machines */
-#undef PIKE_RUN_UNLOCKED
-
 /* Define this if you want to enable the shared nodes mode of the optimizer. */
 #undef SHARED_NODES
 
-/* Define this if you want to enable the new type checker code. */
-#undef NEW_ARG_CHECK
-
 /* Define this to use the new keypair loop. */
 #undef PIKE_MAPPING_KEYPAIR_LOOP
 
@@ -251,12 +247,12 @@
 /* Define if you have _isnan */
 #undef HAVE__ISNAN
 
+/* Define if you have isfinite */
+#undef HAVE_ISFINITE
+
 /* Define if you have fork */
 #undef HAVE_FORK
 
-/* Define if you have isspace */
-#undef HAVE_ISSPACE
-
 /* Define if you have fpsetmask */
 #undef HAVE_FPSETMASK
 
@@ -302,44 +298,18 @@
 /* ... by using the RDTSC instruction? */
 #undef OWN_GETHRTIME_RDTSC
 
-/* Define if you have a working, 8-bit-clean memcmp */
-#undef HAVE_MEMCMP
-
 /* Define if it is possible to allocate PROT_EXEC memory with mmap */
 #undef MEXEC_USES_MMAP
 
 /* Define if you have gethostname */
 #undef HAVE_GETHOSTNAME
 
-/* Define if you have memmove.  */
-#ifndef __CHECKER__
-#undef HAVE_MEMMOVE
-#endif
-
-/* Define if you have memmem.  */
-#undef HAVE_MEMMEM
-
-/* Define if you have memset.  */
-#undef HAVE_MEMSET
-
-/* Define if you have memcpy.  */
-#undef HAVE_MEMCPY
-
-/* Define if you have strcoll */
-#undef HAVE_STRCOLL
-
 /* Define this if you have dlopen */
 #undef HAVE_DLOPEN
 
-/* Define if you have ldexp.  */
-#undef HAVE_LDEXP
-
 /* Define if you have rint.  */
 #undef HAVE_RINT
 
-/* Define if you have frexp.  */
-#undef HAVE_FREXP
-
 /* Define if your signals are one-shot */
 #undef SIGNAL_ONESHOT
 
@@ -352,6 +322,9 @@
 /* Define this to use machine code */
 #undef PIKE_USE_MACHINE_CODE
 
+/* Define if you have the RDTSC instruction */
+#undef HAVE_RDTSC
+
 /* Define this to one of the available bytecode methods. */
 #undef PIKE_BYTECODE_METHOD
 
@@ -386,9 +359,6 @@
 #undef DOUBLE_IS_IEEE_BIG
 #undef DOUBLE_IS_IEEE_LITTLE
 
-/* Define this if strtol exists, and doesn't cut at 0x7fffffff */
-#undef HAVE_WORKING_STRTOL
-
 /* The rest of this file is just to eliminate warnings */
 
 /* define if declaration of strchr is missing */
@@ -424,9 +394,6 @@
 /* Assembler prefix for general purpose registers */
 #undef PIKE_CPU_REG_PREFIX
 
-/* Number of possible filedesriptors */
-#define MAX_OPEN_FILEDESCRIPTORS 1024
-
 /* define this if #include <time.h> provides an external int timezone */
 #undef HAVE_EXTERNAL_TIMEZONE
 
@@ -514,9 +481,6 @@
 /* Define if your OS has the union wait. */
 #undef HAVE_UNION_WAIT
 
-/* Define if you have isgraph */
-#undef HAVE_ISGRAPH
-
 /* Define if your cpp supports the ANSI concatenation operator ## */
 #undef HAVE_ANSI_CONCAT
 
@@ -562,9 +526,6 @@
 /* The last argument to accept() is an ACCEPT_SIZE_T * */
 #define ACCEPT_SIZE_T	int
 
-/* Can we compile in MMX support? */
-#undef TRY_USE_MMX
-
 /* Define if you have the <sys/resource.h> header file.  */
 #undef HAVE_SYS_RESOURCE_H
 
@@ -580,9 +541,12 @@
 /* set this to the modifier type string to print INT64 if that type exists */
 #undef PRINTINT64
 
-/* Define if the compiler understand union initializations. */
+/* Define if the compiler understands union initializations. */
 #undef HAVE_UNION_INIT
 
+/* Define if the compiler has problems with union aliasing. */
+#undef NO_COMBINED_TYPE_SUBTYPE
+
 /* Define when binary --disable-binary is used. */
 #undef DISABLE_BINARY
 
