@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: c8a4d9ea70506d8281d60d2908ad78d48fdfcbbc $
 */
 
 #include "global.h"
@@ -28,8 +27,8 @@
 
 static MUTEX_T fd_mutex;
 
-HANDLE da_handle[MAX_OPEN_FILEDESCRIPTORS];
-int fd_type[MAX_OPEN_FILEDESCRIPTORS];
+HANDLE da_handle[FD_SETSIZE];
+int fd_type[FD_SETSIZE];
 int first_free_handle;
 
 /* #define FD_DEBUG */
@@ -48,7 +47,8 @@ int first_free_handle;
 #endif
 
 /* _dosmaperr is internal but still exported in the dll interface. */
-__declspec(dllimport) void __cdecl _dosmaperr(unsigned long);
+/*__declspec(dllimport)*/
+ void __cdecl _dosmaperr(unsigned long);
 
 PMOD_EXPORT void set_errno_from_win32_error (unsigned long err)
 {
@@ -66,6 +66,9 @@ PMOD_EXPORT void set_errno_from_win32_error (unsigned long err)
       case ERROR_INVALID_HANDLE: /* 124 */
       case ERROR_NEGATIVE_SEEK:	/* 131 */
 	return;
+      case ERROR_DIRECTORY: /* 267 */
+	errno = ENOTDIR; /* [Bug 7271] */
+	return;
     }
 
     /* FIXME: This lets most winsock codes through as-is, e.g.
@@ -121,7 +124,7 @@ PMOD_EXPORT char *debug_fd_info(int fd)
   if(fd<0)
     return "BAD";
 
-  if(fd > MAX_OPEN_FILEDESCRIPTORS)
+  if(fd > FD_SETSIZE)
     return "OUT OF RANGE";
 
   switch(fd_type[fd])
@@ -136,6 +139,7 @@ PMOD_EXPORT char *debug_fd_info(int fd)
 
 PMOD_EXPORT int debug_fd_query_properties(int fd, int guess)
 {
+  if ((fd < 0) || (fd >= FD_SETSIZE)) return 0;
   switch(fd_type[fd])
   {
     case FD_SOCKET:
@@ -153,7 +157,7 @@ PMOD_EXPORT int debug_fd_query_properties(int fd, int guess)
   }
 }
 
-void fd_init()
+void fd_init(void)
 {
   int e;
   WSADATA wsadata;
@@ -174,13 +178,13 @@ void fd_init()
   da_handle[2] = GetStdHandle(STD_ERROR_HANDLE);
 
   first_free_handle=3;
-  for(e=3;e<MAX_OPEN_FILEDESCRIPTORS-1;e++)
+  for(e=3;e<FD_SETSIZE-1;e++)
     fd_type[e]=e+1;
   fd_type[e]=FD_NO_MORE_FREE;
   mt_unlock(&fd_mutex);
 }
 
-void fd_exit()
+void fd_exit(void)
 {
   WSACleanup();
   mt_destroy(&fd_mutex);
@@ -442,7 +446,7 @@ int debug_fd_stat(const char *file, PIKE_STAT_T *buf)
       errno=EINVAL;
       return -1;
     }
-    MEMCPY(fname, file, l);
+    memcpy(fname, file, l);
     fname[l]=0;
     file=fname;
   }
@@ -652,7 +656,7 @@ PMOD_EXPORT FD debug_fd_open(const char *file, int open_mode, int create_mode)
       errno=EINVAL;
       return -1;
     }
-    MEMCPY(fname, file, l);
+    memcpy(fname, file, l);
     fname[l]=0;
     file=fname;
   }
@@ -777,38 +781,56 @@ PMOD_EXPORT FD debug_fd_socket(int domain, int type, int proto)
 
 PMOD_EXPORT int debug_fd_pipe(int fds[2] DMALLOC_LINE_ARGS)
 {
+  int tmp_fds[2];
   HANDLE files[2];
   mt_lock(&fd_mutex);
-  if(first_free_handle == FD_NO_MORE_FREE)
+  if((first_free_handle == FD_NO_MORE_FREE) ||
+     (fd_type[first_free_handle] == FD_NO_MORE_FREE))
   {
     mt_unlock(&fd_mutex);
     errno=EMFILE;
     return -1;
   }
+  /* Allocate the fds. */
+  tmp_fds[0] = first_free_handle;
+  first_free_handle = fd_type[tmp_fds[0]];
+  fd_type[tmp_fds[0]] = FD_PIPE;
+  da_handle[tmp_fds[0]] = INVALID_HANDLE_VALUE;
+  tmp_fds[1] = first_free_handle;
+  first_free_handle = fd_type[tmp_fds[1]];
+  fd_type[tmp_fds[1]] = FD_PIPE;
+  da_handle[tmp_fds[1]] = INVALID_HANDLE_VALUE;
   mt_unlock(&fd_mutex);
+
   if(!CreatePipe(&files[0], &files[1], NULL, 0))
   {
     set_errno_from_win32_error (GetLastError());
+    mt_lock(&fd_mutex);
+    fd_type[tmp_fds[1]] = first_free_handle;
+    first_free_handle = tmp_fds[1];
+    fd_type[tmp_fds[0]] = first_free_handle;
+    first_free_handle = tmp_fds[0];
+    mt_unlock(&fd_mutex);
     return -1;
   }
   
-  FDDEBUG(fprintf(stderr,"ReadHANDLE=%d WriteHANDLE=%d\n",files[0],files[1]));
+  FDDEBUG(fprintf(stderr, "ReadHANDLE=%d WriteHANDLE=%d\n",
+		  files[0], files[1]));
   
-  SetHandleInformation(files[0],HANDLE_FLAG_INHERIT|HANDLE_FLAG_PROTECT_FROM_CLOSE,0);
-  SetHandleInformation(files[1],HANDLE_FLAG_INHERIT|HANDLE_FLAG_PROTECT_FROM_CLOSE,0);
+  SetHandleInformation(files[0],
+		       HANDLE_FLAG_INHERIT|HANDLE_FLAG_PROTECT_FROM_CLOSE, 0);
+  SetHandleInformation(files[1],
+		       HANDLE_FLAG_INHERIT|HANDLE_FLAG_PROTECT_FROM_CLOSE, 0);
+
   mt_lock(&fd_mutex);
-  fds[0]=first_free_handle;
-  first_free_handle=fd_type[fds[0]];
-  fd_type[fds[0]]=FD_PIPE;
+  fds[0] = tmp_fds[0];
   da_handle[fds[0]] = files[0];
-
-  fds[1]=first_free_handle;
-  first_free_handle=fd_type[fds[1]];
-  fd_type[fds[1]]=FD_PIPE;
+  fds[1] = tmp_fds[1];
   da_handle[fds[1]] = files[1];
-
   mt_unlock(&fd_mutex);
-  FDDEBUG(fprintf(stderr,"New pipe: %d (%d) -> %d (%d)\n",fds[0],files[0], fds[1], fds[1]));;
+
+  FDDEBUG(fprintf(stderr,"New pipe: %d (%d) -> %d (%d)\n",
+		  fds[0], files[0], fds[1], files[1]));
 
 #ifdef DEBUG_MALLOC
   debug_malloc_register_fd( fds[0], DMALLOC_LOCATION());
@@ -826,6 +848,11 @@ PMOD_EXPORT FD debug_fd_accept(FD fd, struct sockaddr *addr,
   mt_lock(&fd_mutex);
   FDDEBUG(fprintf(stderr,"Accept on %d (%ld)..\n",
 		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd])));
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
   if(first_free_handle == FD_NO_MORE_FREE)
   {
     mt_unlock(&fd_mutex);
@@ -872,6 +899,10 @@ PMOD_EXPORT int PIKE_CONCAT(debug_fd_,NAME) X1 { \
   int ret; \
   FDDEBUG(fprintf(stderr, #NAME " on %d (%ld)\n", \
 		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd]))); \
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {	\
+    errno = EBADF; \
+    return -1; \
+  } \
   mt_lock(&fd_mutex); \
   if(fd_type[fd] != FD_SOCKET) { \
      mt_unlock(&fd_mutex); \
@@ -927,6 +958,11 @@ PMOD_EXPORT int debug_fd_connect (FD fd, struct sockaddr *a, int len)
 	  fprintf(stderr," %02x",((unsigned char *)a)[ret]);
 	  fprintf(stderr,"\n");
   )
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
   if(fd_type[fd] != FD_SOCKET)
   {
     mt_unlock(&fd_mutex);
@@ -946,6 +982,11 @@ PMOD_EXPORT int debug_fd_close(FD fd)
   HANDLE h;
   int type;
   mt_lock(&fd_mutex);
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
   h = da_handle[fd];
   FDDEBUG(fprintf(stderr,"Closing %d (%ld)\n",
 		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd])));
@@ -992,6 +1033,11 @@ PMOD_EXPORT ptrdiff_t debug_fd_write(FD fd, void *buf, ptrdiff_t len)
   mt_lock(&fd_mutex);
   FDDEBUG(fprintf(stderr, "Writing %d bytes to %d (%d)\n",
 		  len, fd, da_handle[fd]));
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
   kind = fd_type[fd];
   handle = da_handle[fd];
   mt_unlock(&fd_mutex);
@@ -1050,6 +1096,11 @@ PMOD_EXPORT ptrdiff_t debug_fd_read(FD fd, void *to, ptrdiff_t len)
   FDDEBUG(fprintf(stderr,"Reading %d bytes from %d (%d) to %lx\n",
 		  len, fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd]),
 		  PTRDIFF_T_TO_LONG((ptrdiff_t)to)));
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
   ret=fd_type[fd];
   handle=da_handle[fd];
   mt_unlock(&fd_mutex);
@@ -1073,16 +1124,45 @@ PMOD_EXPORT ptrdiff_t debug_fd_read(FD fd, void *to, ptrdiff_t len)
     case FD_FILE:
     case FD_PIPE:
       ret=0;
-      if(len && !ReadFile(handle, to,
-			  DO_NOT_WARN((DWORD)len),
-			  &ret,0) && ret<=0)
-      {
+      while(len && !ReadFile(handle, to,
+			     DO_NOT_WARN((DWORD)len),
+			     &ret,0) && ret<=0) {
 	unsigned int err = GetLastError();
 	set_errno_from_win32_error (err);
 	switch(err)
 	{
-	  /* Pretend we reached the end of the file */
+	  case ERROR_NOT_ENOUGH_MEMORY:
+	    /* This can happen when reading from stdin, and can be due
+	     * to running out of OS io-buffers. cf ReadConsole():
+	     *
+	     * | lpBuffer [out]
+	     * |   A pointer to a buffer that receives the data read from
+	     * |   the console input buffer.
+	     * |
+	     * |   The storage for this buffer is allocated from a shared
+	     * |   heap for the process that is 64 KB in size. The maximum
+	     * |   size of the buffer will depend on heap usage.
+	     *
+	     * The limit seems to be 26608 bytes on Windows Server 2003,
+	     * and 31004 bytes on some versions of Windows XP.
+	     *
+	     * The gdb people ran into the same bug in 2006.
+	     * cf http://permalink.gmane.org/gmane.comp.gdb.patches/29669
+	     *
+	     * FIXME: We ought to attempt to fill the remainder of
+	     *        the buffer on success and full read, but as
+	     *        this failure mode usually only happens with
+	     *        the console, we would risk blocking, so let
+	     *        the higher levels of code deal with the
+	     *        resulting short reads.
+	     */
+	    /* Halve the size of the request and retry. */
+	    len = len >> 1;
+	    if (len) continue;
+	    /* Total failure. Fall out to the generic error return. */
+	    break;
 	  case ERROR_BROKEN_PIPE:
+	    /* Pretend we reached the end of the file */
 	    return 0;
 	}
 	FDDEBUG(fprintf(stderr,"Read failed %d\n",errno));
@@ -1103,6 +1183,11 @@ PMOD_EXPORT PIKE_OFF_T debug_fd_lseek(FD fd, PIKE_OFF_T pos, int where)
   HANDLE h;
 
   mt_lock(&fd_mutex);
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
   if(fd_type[fd]!=FD_FILE)
   {
     mt_unlock(&fd_mutex);
@@ -1166,6 +1251,11 @@ PMOD_EXPORT int debug_fd_ftruncate(FD fd, PIKE_OFF_T len)
   DWORD err;
 
   mt_lock(&fd_mutex);
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
   if(fd_type[fd]!=FD_FILE)
   {
     mt_unlock(&fd_mutex);
@@ -1222,6 +1312,10 @@ PMOD_EXPORT int debug_fd_flock(FD fd, int oper)
 {
   long ret;
   HANDLE h;
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {
+    errno = EBADF;
+    return -1;
+  }
   mt_lock(&fd_mutex);
   if(fd_type[fd]!=FD_FILE)
   {
@@ -1242,7 +1336,7 @@ PMOD_EXPORT int debug_fd_flock(FD fd, int oper)
   }else{
     DWORD flags = 0;
     OVERLAPPED tmp;
-    MEMSET(&tmp, 0, sizeof(tmp));
+    memset(&tmp, 0, sizeof(tmp));
     tmp.Offset=0;
     tmp.OffsetHigh=0;
 
@@ -1279,6 +1373,11 @@ PMOD_EXPORT int debug_fd_fstat(FD fd, PIKE_STAT_T *s)
   mt_lock(&fd_mutex);
   FDDEBUG(fprintf(stderr, "fstat on %d (%ld)\n",
 		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd])));
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
   if(fd_type[fd]!=FD_FILE)
   {
     errno=ENOTSUPP;
@@ -1286,7 +1385,7 @@ PMOD_EXPORT int debug_fd_fstat(FD fd, PIKE_STAT_T *s)
     return -1;
   }
 
-  MEMSET(s, 0, sizeof(PIKE_STAT_T));
+  memset(s, 0, sizeof(PIKE_STAT_T));
   s->st_nlink=1;
 
   switch(fd_type[fd])
@@ -1407,6 +1506,10 @@ PMOD_EXPORT int debug_fd_ioctl(FD fd, int cmd, void *data)
   int ret;
   FDDEBUG(fprintf(stderr,"ioctl(%d (%ld,%d,%p)\n",
 		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd]), cmd, data));
+  if ((fd < 0) || (fd >= FD_SETSIZE)) {
+    errno = EBADF;
+    return -1;
+  }
   switch(fd_type[fd])
   {
     case FD_SOCKET:
@@ -1432,10 +1535,16 @@ PMOD_EXPORT FD debug_fd_dup(FD from)
   HANDLE x,p=GetCurrentProcess();
 
   mt_lock(&fd_mutex);
-#ifdef DEBUG
-  if(fd_type[from]>=FD_NO_MORE_FREE)
-    Pike_fatal("fd_dup() on file which is not open!\n");
-#endif
+  if ((from < 0) || (from >= FD_SETSIZE)) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
+  if(fd_type[from] >= FD_NO_MORE_FREE) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
   if(!DuplicateHandle(p,da_handle[from],p,&x,0,0,DUPLICATE_SAME_ACCESS))
   {
     set_errno_from_win32_error (GetLastError());
@@ -1458,7 +1567,16 @@ PMOD_EXPORT FD debug_fd_dup2(FD from, FD to)
 {
   HANDLE x,p=GetCurrentProcess();
 
+  if ((from < 0) || (from >= FD_SETSIZE)) {
+    errno = EBADF;
+    return -1;
+  }
   mt_lock(&fd_mutex);
+  if(fd_type[from] >= FD_NO_MORE_FREE) {
+    mt_unlock(&fd_mutex);
+    errno = EBADF;
+    return -1;
+  }
   if(!DuplicateHandle(p,da_handle[from],p,&x,0,0,DUPLICATE_SAME_ACCESS))
   {
     set_errno_from_win32_error (GetLastError());
@@ -1498,7 +1616,7 @@ PMOD_EXPORT FD debug_fd_dup2(FD from, FD to)
 PMOD_EXPORT const char *debug_fd_inet_ntop(int af, const void *addr,
 					   char *cp, size_t sz)
 {
-  static char *(*inet_ntop_funp)(int, void*, char *, size_t);
+  static char *(*inet_ntop_funp)(int, const void*, char *, size_t);
   static int tried;
   static HINSTANCE ws2_32lib;
 
@@ -1508,7 +1626,7 @@ PMOD_EXPORT const char *debug_fd_inet_ntop(int af, const void *addr,
       if ((ws2_32lib = LoadLibrary("Ws2_32"))) {
 	FARPROC proc;
 	if ((proc = GetProcAddress(ws2_32lib, "InetNtopA"))) {
-	  inet_ntop_funp = (char *(*)(int, void *, char *, size_t))proc;
+	  inet_ntop_funp = (char *(*)(int, const void *, char *, size_t))proc;
 	}
       }
     }
@@ -1561,14 +1679,14 @@ PMOD_EXPORT DIR *opendir(char *dir)
 {
   ptrdiff_t len=strlen(dir);
   char *foo;
-  DIR *ret=(DIR *)malloc(sizeof(DIR) + len+5);
+  DIR *ret=malloc(sizeof(DIR) + len+5);
   if(!ret)
   {
     errno=ENOMEM;
     return 0;
   }
   foo=sizeof(DIR) + (char *)ret;
-  MEMCPY(foo, dir, len);
+  memcpy(foo, dir, len);
 
   if(len && foo[len-1]!='/') foo[len++]='/';
   foo[len++]='*';
@@ -1580,7 +1698,7 @@ PMOD_EXPORT DIR *opendir(char *dir)
   if(ret->h == DO_NOT_WARN(INVALID_HANDLE_VALUE))
   {
     errno=ENOENT;
-    free((char *)ret);
+    free(ret);
     return 0;
   }
   ret->first=1;
@@ -1608,408 +1726,6 @@ PMOD_EXPORT int readdir_r(DIR *dir, struct direct *tmp ,struct direct **d)
 PMOD_EXPORT void closedir(DIR *dir)
 {
   FindClose(dir->h);
-  free((char *)dir);
+  free(dir);
 }
 #endif
-
-#if 0
-
-#ifdef FD_LINEAR
-struct fd_mapper
-{
-  int size;
-  void **data;
-};
-
-void init_fd_mapper(struct fd_mapper *x)
-{
-  x->size=64;
-  x->data=(void **)xalloc(x->size*sizeof(void *));
-}
-
-void exit_fd_mapper(struct fd_mapper *x)
-{
-  free((char *)x->data);
-}
-
-void fd_mapper_set(struct fd_mapper *x, FD fd, void *data)
-{
-  while(fd>=x->size)
-  {
-    x->size*=2;
-    x->data=(void **)realloc((char *)x->data, x->size*sizeof(void *));
-    if(!x->data)
-      Pike_fatal("Out of memory.\n");
-    x->data=nd;
-  }
-  x->data[fd]=data;
-  
-}
-
-void *fd_mapper_get(struct fd_mapper *x, FD fd)
-{
-  return x->data[fd];
-}
-#else /* FD_LINEAR */
-struct fd_mapper_data
-{
-  FD x;
-  void *data;
-};
-struct fd_mapper
-{
-  int num;
-  int hsize;
-  struct fd_mapper_data *data;
-};
-
-void init_fd_mapper(struct fd_mapper *x)
-{
-  int i;
-  x->num=0;
-  x->hsize=127;
-  x->data=(struct fd_mapper_data *)xalloc(x->hsize*sizeof(struct fd_mapper_data));
-  for(i=0;i<x->hsize;i++) x->data[i].fd=-1;
-}
-
-void exit_fd_mapper(struct fd_mapper *x)
-{
-  free((char *)x->data);
-}
-
-void fd_mapper_set(struct fd_mapper *x, FD fd, void *data)
-{
-  int hval;
-  x->num++;
-  if(x->num*2 > x->hsize)
-  {
-    struct fd_mapper_data *old=x->data;
-    int i,old_size=x->hsize;
-    x->hsize*=3;
-    x->num=0;
-    x->data=(struct fd_mapper_data *)xalloc(x->size*sizeof(struct fd_mapper_data *));
-    for(i=0;i<x->size;i++) x->data[i].fd=-1;
-    for(i=0;i<old_size;i++)
-      if(old[i].fd!=-1)
-	fd_mapper_set(x, old[i].fd, old[i].data);
-  }
-
-  hval=fd % x->hsize;
-  while(x->data[hval].fd != -1)
-  {
-    hval++;
-    if(hval==x->hsize) hval=0;
-  }
-  x->data[hval].fd=fd;
-  x->data[hval].data=data;
-}
-
-void *fd_mapper_get(struct fd_mapper *x, FD fd)
-{
-  int hval=fd % x->hsize;
-  while(x->data[hval].fd != fd)
-  {
-    hval++;
-    if(hval==x->hsize) hval=0;
-  }
-  return x->data[hval].data;
-}
-#endif /* FD_LINEAR */
-
-
-struct fd_data_hash
-{
-  FD fd;
-  int key;
-  struct fd_data_hash *next;
-  void *data;
-};
-
-#define FD_DATA_PER_BLOCK 255
-
-struct fd_data_hash_block
-{
-  struct fd_data_hash_block *next;
-  struct fd_data_hash data[FD_DATA_PER_BLOCk];
-};
-
-static int keynum=0;
-static unsigned int num_stored_keys=0;
-static unsigned int hash_size=0;
-static struct fd_data_hash *free_blocks=0;
-static struct fd_data_hash **htable=0;
-static fd_data_hash_block *hash_blocks=0;
-
-int get_fd_data_key(void)
-{
-  return ++keynum;
-}
-
-void store_fd_data(FD fd, int key, void *data)
-{
-  struct fd_data_hash *p,**last;
-  unsigned int hval=(fd + key * 53) % hash_size;
-
-  for(last=htable[e];p=*last;last=&p->next)
-  {
-    if(p->fd == fd && p->key == key)
-    {
-      if(data)
-      {
-	p->data=data;
-      }else{
-	*last=p->next;
-	p->next=free_blocks;
-	free_blocks=p;
-	num_stored_keys--;
-      }
-      return;
-    }
-  }
-  if(!data) return;
-
-  num_stored_keys++;
-
-  if(num_stored_keys * 2 >= hash_size)
-  {
-    /* time to rehash */
-    unsigned int h;
-    unsigned int old_hsize=hash_size;
-    unsigned fd_data_hash **old_htable=htable;
-    if(!hash_size)
-      hash_size=127;
-    else
-      hash_size*=3;
-
-    htable=(struct fd_data_hash **)xalloc(hash_size * sizeof(struct fd_data_hash *));
-    
-    for(h=0;h<old_hsize;h++)
-    {
-      for(last=old_htable+e;p=*last;last=&p->next)
-	store_fd_data(p->fd, p->key, p->data);
-      *last=free_blocks;
-      free_blocks=old_htable[h];
-    }
-    if(old_htable)
-      free((char *)old_htable);
-  }
-
-
-  if(!free_blocks)
-  {
-    struct fd_data_hash_block *n;
-    int e;
-    n=ALLOC_STRUCT(fd_data_hash_block);
-    n->next=hash_blocks;
-    hash_blocks=n;
-    for(e=0;e<FD_DATA_PER_BLOCK;e++)
-    {
-      n->data[e].next=free_blocks;
-      free_blocks=n->data+e;
-    }
-  }
-  
-  p=free_blocks;
-  free_blocks=p->next;
-  p->fd=fd;
-  p->key=key;
-  p->data=data;
-  p->next=htable[hval];
-  htable[hval]=p;
-}
-
-void *get_fd_data(FD fd, int key)
-{
-  struct fd_data_hash *p,**last;
-  unsigned int hval=(fd + key * 53) % hash_size;
-
-  for(p=htable[hval];p;p=p->next)
-    if(p->fd == fd && p->key == key)
-      return p->data;
-
-  return 0;
-}
-
-
-#define FD_EVENT_READ 1
-#define FD_EVENT_WRITE 2
-#define FD_EVENT_OOB 4
-
-struct event
-{
-  int fd;
-  int events;
-};
-
-#ifdef FDLIB_USE_SELECT
-
-struct fd_waitor
-{
-  fd_FDSET rcustomers,wcustomers,xcustomers;
-  fd_FDSET rtmp,wtmp,xtmp;
-  FD last;
-  int numleft;
-  int max;
-};
-
-#define init_waitor(X) do { (X)->numleft=0; (X)->max=0; \
- fd_FDZERO(&X->rcustomers); \
- fd_FDZERO(&X->wcustomers); \
- fd_FDZERO(&X->xcustomers); \
- } while(0)
-
-void fd_waitor_set_customer(struct fd_waitor *x, FD customer, int flags)
-{
-  if(flags & FD_EVENT_READ)
-  {
-    fd_FD_SET(& x->rcustomer, customer);
-  }else{
-    fd_FD_CLR(& x->rcustomer, customer);
-  }
-
-  if(flags & FD_EVENT_WRITE)
-  {
-    fd_FD_SET(& x->wcustomer, customer);
-  }else{
-    fd_FD_CLR(& x->wcustomer, customer);
-  }
-
-  if(flags & FD_EVENT_OOB)
-  {
-    fd_FD_SET(& x->xcustomer, customer);
-  }else{
-    fd_FD_CLR(& x->xcustomer, customer);
-  }
-
-  if(flags)
-    if(customer>x->max) x->max=customer;
-  else
-    if(customer == x->max)
-    {
-      x->max--;
-      while(
-	    !fd_ISSET(& x->rcustomers,x->max) &&
-	    !fd_ISSET(& x->wcustomers,x->max) &&
-	    !fd_ISSET(& x->xcustomers,x->max)
-	    )
-	x->max--;
-    }
-}
-
-int fd_waitor_idle(fd_waitor *x,
-		  struct timeval *t,
-		  struct event *e)
-{
-  int tmp;
-  if(!x->numleft)
-  {
-    x->rtmp=x->rcustomers;
-    x->wtmp=x->wcustomers;
-    x->xtmp=x->xcustomers;
-
-    tmp=select(x->max, & x->rtmp, & x->wtmp, & x->xtmp, &t);
-    if(tmp<0) return 0;
-
-    x->last=0;
-    x->numleft=tmp;
-  }
-  while(x->numleft)
-  {
-    while(x->last<x->max)
-    {
-      int flags=
-	(fd_FD_ISSET(& x->rtmp, x->last) ? FD_EVENT_READ : 0) |
-	(fd_FD_ISSET(& x->wtmp, x->last) ? FD_EVENT_WRITE : 0) |
-	(fd_FD_ISSET(& x->xtmp, x->last) ? FD_EVENT_OOB: 0);
-
-      if(flags)
-      {
-	numleft--;
-
-	e->fd=x->last
-	e->event=flags;
-	return 1;
-      }
-    }
-  }
-  return 0;
-}
-
-#endif /* FDLIB_USE_SELECT */
-
-#ifdef FDLIB_USE_WAITFORMULTIPLEOBJECTS
-
-#define FD_MAX 16384
-
-struct fd_waitor
-{
-  int occupied;
-  HANDLE customers[FD_MAX];
-  FD pos_to_fd[FD_MAX];
-  int fd_to_pos_key;
-  int last_swap;
-};
-
-void fd_waitor_set_customer(fd_waitor *x, FD customer, int flags)
-{
-  HANDLE h=CreateEvent();
-  x->customers[x->occupied]=h;
-  x->pos_to_fd[x->occupied]=customer;
-  fd_mapper_store(customer, x->fd_to_pos_key, x->occupied);
-  x->occupied++;
-}
-
-void fd_waitor_remove_customer(fd_waitor *x, FD customer)
-{
-  int pos=(int)fd_mapper_get(customer, x->fd_to_pos_key);
-
-  CloseHandle(x->customers[pos]);
-  
-  fd_mapper_store(customer, x->fd_to_pos_key, NULL);
-  x->occupied--;
-  if(x->occupied != pos)
-  {
-    x->customer[pos]=x->customer[x->occupied];
-    x->pos_to_fd[pos]=x->pos_to_fd[x->occupied];
-    fd_mapper_store(x->pos_to_fd[pos], x->fd_to_pos_key, (void *)pos);
-  }
-}
-
-FD fd_waitor_idle(fd_waitor *x, struct timeval delay)
-{
-  DWORD ret,d=delay.tv_usec/1000;
-  d+=MINIMUM(100000,delay.tv_sec) *1000;
-
-  ret=WaitForMultipleObjects(x->occupied,
-			     x->customers,
-			     0,
-			     delay);
-
-  if(ret>= WAIT_OBJECT_0 && ret< WAIT_OBJECT_0 + x->occupied)
-  {
-    long tmp;
-    ret-=WAIT_OBJECT_0;
-    if(-- (x->last_swap) <= ret)
-    {
-      x->last_swap=x->occupied;
-      if(x->occupied == ret) return ret;
-    }
-    tmp=customers[ret];
-    customers[ret]=customers[x->last_swap];
-    customers[x->last_swap]=tmp;
-
-    tmp=pos_to_fd[ret];
-    pos_to_fd[ret]=pos_to_fd[x->last_swap];
-    pos_to_fd[x->last_swap]=tmp;
-
-    fd_mapper_store(x->pos_to_fd[ret], x->fd_to_pos_key, ret);
-    fd_mapper_store(x->pos_to_fd[x->last_swap], x->fd_to_pos_key, x->last_swap);
-    return x->pos_to_fd[ret];
-  }else{
-    return -1;
-  }
-}
-
-#endif /* FDLIB_USE_WAITFORMULTIPLEOBJECTS */
-
-#endif /* 0 */
