@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: e0c3a56166e44fee11907b0b8007bc3468936021 $
 */
 
 #include "global.h"
@@ -21,10 +20,11 @@
 #include "program.h"
 #include "bignum.h"
 #include "backend.h"
+#include "pike_types.h"
 
 #ifdef HAVE_LIBFUSE
-/* Attempt to use FUSE API version 2.6 (if possible). */
-#define FUSE_USE_VERSION 26
+/* Attempt to use FUSE API version 2.9 (if possible). */
+#define FUSE_USE_VERSION 29
 #include <fuse.h>
 #include <stdio.h>
 #include <string.h>
@@ -47,7 +47,7 @@ struct fuse_cmd_storage
     struct fuse_cmd *cmd;
 };
 
-static void f_fuse_cmd_process( INT32 args )
+static void f_fuse_cmd_process( INT32 UNUSED(args) )
 {
     struct svalue *sp = Pike_sp;
     fuse_process_cmd(THISC->f, THISC->cmd);
@@ -55,17 +55,9 @@ static void f_fuse_cmd_process( INT32 args )
     push_int(0);
 }
 
-static void push_fuse_cmd(struct fuse_cmd *cmd, struct fuse *f)
-{
-    struct object *o = clone_object( fuse_cmd_program, 0);
-    ((struct fuse_cmd_storage *)o->storage)->cmd = cmd;
-    ((struct fuse_cmd_storage *)o->storage)->f = f;
-    push_object( o );
-}
-
 static struct object *global_fuse_obj; // There Can Be Only One
 
-#define DEFAULT_ERRNO() do{if(Pike_sp[-1].u.integer) return -Pike_sp[-1].u.integer;return -ENOENT;}while(0)
+#define DEFAULT_ERRNO() do{if((TYPEOF(Pike_sp[-1]) == T_INT) && Pike_sp[-1].u.integer) return -Pike_sp[-1].u.integer;return -ENOENT;}while(0)
 
 static int pf_getattr(const char *path, struct stat *stbuf)
 {
@@ -73,8 +65,8 @@ static int pf_getattr(const char *path, struct stat *stbuf)
     struct stat *st;
     push_text( path );
     apply( global_fuse_obj, "getattr", 1 );
-    if( Pike_sp[-1].type != PIKE_T_OBJECT ||
-	!(st = (struct stat *)get_storage( Pike_sp[-1].u.object, stat_program)) )
+    if( TYPEOF(Pike_sp[-1]) != PIKE_T_OBJECT ||
+	!(st = get_storage( Pike_sp[-1].u.object, stat_program)) )
 	DEFAULT_ERRNO();
     *stbuf = *st;
     return 0;
@@ -85,9 +77,7 @@ static int pf_readlink(const char *path, char *buf, size_t size)
     int res;
     push_text( path );
     apply( global_fuse_obj, "readlink", 1 );
-    if(Pike_sp[-1].type == T_INT)
-        return -Pike_sp[-1].u.integer;
-    if( (Pike_sp[-1].type != PIKE_T_STRING) ||
+    if( (TYPEOF(Pike_sp[-1]) != PIKE_T_STRING) ||
 	(Pike_sp[-1].u.string->size_shift) )
 	DEFAULT_ERRNO();
     if( Pike_sp[-1].u.string->len >= (int)size )
@@ -123,7 +113,7 @@ static int pf_getdir(const char *path, fuse_dirh_t h, fuse_dirfil_t filler)
     push_text( path );
     push_getdir_callback( h, filler );
     apply( global_fuse_obj, "readdir", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -134,7 +124,7 @@ static int pf_mknod(const char *path, mode_t mode, dev_t rdev)
     push_int( mode );
     push_int( rdev );
     apply( global_fuse_obj, "mknod", 3 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -144,7 +134,7 @@ static int pf_mkdir(const char *path, mode_t mode)
     push_text( path );
     push_int( mode );
     apply( global_fuse_obj, "mkdir", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -153,7 +143,7 @@ static int pf_unlink(const char *path)
 {
     push_text( path );
     apply( global_fuse_obj, "unlink", 1 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -162,7 +152,7 @@ static int pf_rmdir(const char *path)
 {
     push_text( path );
     apply( global_fuse_obj, "rmdir", 1 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -172,7 +162,7 @@ static int pf_symlink(const char *from, const char *to)
     push_text( from );
     push_text( to );
     apply( global_fuse_obj, "symlink", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -182,7 +172,7 @@ static int pf_rename(const char *from, const char *to)
     push_text( from );
     push_text( to );
     apply( global_fuse_obj, "rename", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -192,7 +182,7 @@ static int pf_link(const char *from, const char *to)
     push_text( from );
     push_text( to );
     apply( global_fuse_obj, "link", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -202,7 +192,7 @@ static int pf_chmod(const char *path, mode_t mode)
     push_text( path );
     push_int( mode );
     apply( global_fuse_obj, "chmod", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -213,7 +203,7 @@ static int pf_chown(const char *path, uid_t uid, gid_t gid)
     push_int( uid );
     push_int( gid );
     apply( global_fuse_obj, "chown", 3 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -223,42 +213,54 @@ static int pf_truncate(const char *path, off_t size)
     push_text( path );
     push_int64( size );
     apply( global_fuse_obj, "truncate", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
 
+#if FUSE_VERSION < 26
 static int pf_utime(const char *path, struct utimbuf *buf)
 {
     push_text( path );
     push_int( buf->actime );
     push_int( buf->modtime );
     apply( global_fuse_obj, "utime", 3 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
-
+#else
+static int pf_utimens(const char *path, const struct timespec tv[2])
+{
+    push_text( path );
+    push_int64( (INT64)tv[0].tv_sec*(INT64)1000000000 + (INT64)tv[0].tv_nsec );
+    push_int64( (INT64)tv[1].tv_sec*(INT64)1000000000 + (INT64)tv[1].tv_nsec );
+    apply( global_fuse_obj, "utimens", 3 );
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
+	DEFAULT_ERRNO();
+    return -Pike_sp[-1].u.integer;
+}
+#endif
 
 static int pf_open(const char *path, struct fuse_file_info *fi)
 {
     push_text( path );
     push_int( fi->flags );
     apply( global_fuse_obj, "open", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
 
 static int pf_read(const char *path, char *buf, size_t size, off_t offset,
-                    struct fuse_file_info *fi)
+                    struct fuse_file_info *UNUSED(fi))
 {
     push_text( path );
     push_int( size );
     push_int64( offset );
     apply( global_fuse_obj, "read", 3 );
-    
-    if( (Pike_sp[-1].type != PIKE_T_STRING) ||
+
+    if( (TYPEOF(Pike_sp[-1]) != PIKE_T_STRING) ||
 	(Pike_sp[-1].u.string->size_shift) )
 	DEFAULT_ERRNO();
     if (((size_t)Pike_sp[-1].u.string->len) > size) {
@@ -269,13 +271,13 @@ static int pf_read(const char *path, char *buf, size_t size, off_t offset,
 }
 
 static int pf_write(const char *path, const char *buf, size_t size,
-		    off_t offset, struct fuse_file_info *fi)
+		    off_t offset, struct fuse_file_info *UNUSED(fi))
 {
     push_text( path );
     push_string( make_shared_binary_string(buf, size ) );
     push_int64( offset );
     apply( global_fuse_obj, "write", 3 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -287,7 +289,7 @@ static int pf_statfs(const char *path, struct statvfs *stbuf)
 
     push_text( path );
     apply( global_fuse_obj, "statfs", 1 );
-    if( Pike_sp[-1].type != PIKE_T_MAPPING )
+    if( TYPEOF(Pike_sp[-1]) != PIKE_T_MAPPING )
 	DEFAULT_ERRNO();
     m = Pike_sp[-1].u.mapping;
     memset( stbuf, 0, sizeof(*stbuf) );
@@ -295,7 +297,7 @@ static int pf_statfs(const char *path, struct statvfs *stbuf)
     stbuf->f_bsize = 1024;
 #define STSET(X)    do {				\
       if ((val=simple_mapping_string_lookup( m,#X )) &&	\
-	  (val->type == T_INT))				\
+	  (TYPEOF(*val) == T_INT))			\
 	stbuf->f_##X=val->u.integer;			\
     } while(0)
     STSET(bsize);
@@ -305,32 +307,32 @@ static int pf_statfs(const char *path, struct statvfs *stbuf)
     STSET(files);
     STSET(ffree);
     if (((val=simple_mapping_string_lookup( m,"namemax" )) &&
-	 (val->type == T_INT)) ||
+	 (TYPEOF(*val) == T_INT)) ||
 	/* namelen is compat. */
 	((val=simple_mapping_string_lookup( m,"namelen" )) &&
-	 (val->type == T_INT))) {
+	 (TYPEOF(*val) == T_INT))) {
       stbuf->f_namemax = val->u.integer;
     }
 #undef STSET
     return 0;
 }
 
-static int pf_release(const char *path, struct fuse_file_info *fi)
+static int pf_release(const char *path, struct fuse_file_info *UNUSED(fi))
 {
     push_text( path );
     apply( global_fuse_obj, "release", 1 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
 
 static int pf_fsync(const char *path, int isdatasync,
-                     struct fuse_file_info *fi)
+                     struct fuse_file_info *UNUSED(fi))
 {
     push_text( path );
     push_int( isdatasync );
     apply( global_fuse_obj, "fsync", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -343,7 +345,7 @@ static int pf_setxattr(const char *path, const char *name, const char *value,
     push_string( make_shared_binary_string( value, size ) );
     push_int( flags );
     apply( global_fuse_obj, "setxattr", 4 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -355,7 +357,7 @@ static int pf_getxattr(const char *path, const char *name, char *value,
     push_text( path );
     push_text( name );
     apply( global_fuse_obj, "getxattr", 2 );
-    if( Pike_sp[-1].type != PIKE_T_STRING ||
+    if( TYPEOF(Pike_sp[-1]) != PIKE_T_STRING ||
 	(Pike_sp[-1].u.string->size_shift) )
 	DEFAULT_ERRNO();
     ds = Pike_sp[-1].u.string->len <<Pike_sp[-1].u.string->size_shift;
@@ -373,14 +375,15 @@ static int pf_listxattr(const char *path, char *list, size_t size)
 
     push_text( path );
     apply( global_fuse_obj, "listxattr", 1 );
-    if( Pike_sp[-1].type != PIKE_T_ARRAY )
+    if( TYPEOF(Pike_sp[-1]) != PIKE_T_ARRAY )
 	DEFAULT_ERRNO();
     push_string( make_shared_binary_string( "\0", 1 ) );
     o_multiply();
-    if( Pike_sp[-1].type != PIKE_T_STRING ||
+    if( TYPEOF(Pike_sp[-1]) != PIKE_T_STRING ||
 	(Pike_sp[-1].u.string->size_shift) )
 	DEFAULT_ERRNO();
-    ds = Pike_sp[-1].u.string->len <<Pike_sp[-1].u.string->size_shift;
+    /* We need to account for the terminating NUL. */
+    ds = (Pike_sp[-1].u.string->len + 1) <<Pike_sp[-1].u.string->size_shift;
     if( !size )
 	return ds;
     if( size < ds )
@@ -394,27 +397,7 @@ static int pf_removexattr(const char *path, const char *name)
     push_text( path );
     push_text( name );
     apply( global_fuse_obj, "removexattr", 2 );
-    if (Pike_sp[-1].type != T_INT)
-	DEFAULT_ERRNO();
-    return -Pike_sp[-1].u.integer;
-}
-
-static int pf_flush( const char *path, struct fuse_file_info *fi)
-{
-    push_text( path );
-    push_int( fi->flags );
-    apply( global_fuse_obj, "flush", 2 );
-    if (Pike_sp[-1].type != T_INT)
-	DEFAULT_ERRNO();
-    return -Pike_sp[-1].u.integer;
-}
-
-static int pf_opendir( const char *path, struct fuse_file_info *fi)
-{
-    push_text( path );
-    push_int( fi->flags );
-    apply( global_fuse_obj, "opendir", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -425,7 +408,7 @@ static int pf_creat( const char *path, mode_t mode, struct fuse_file_info *fi)
     push_int( mode );
     push_int( fi->flags );
     apply( global_fuse_obj, "creat", 3 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
@@ -435,38 +418,44 @@ static int pf_access( const char *path, int mode)
     push_text( path );
     push_int( mode );
     apply( global_fuse_obj, "access", 2 );
-    if (Pike_sp[-1].type != T_INT)
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
 	DEFAULT_ERRNO();
     return -Pike_sp[-1].u.integer;
 }
 
-static int pf_releasedir( const char *path, struct fuse_file_info *fi)
+static int pf_lock( const char *path, struct fuse_file_info *fi, int cmd, struct flock *lck )
 {
-    push_text( path );
-    apply( global_fuse_obj, "releasedir", 1 );
-    if (Pike_sp[-1].type != T_INT)
-	DEFAULT_ERRNO();
-    return -Pike_sp[-1].u.integer;
-}
+  push_text( path );
+  push_int( cmd );
 
-static int pf_fsyncdir( const char *path, int nometa, struct fuse_file_info *fi)
-{
-    push_text( path );
-    push_int( nometa );
-    apply( global_fuse_obj, "fsyncdir", 2 );
-    if (Pike_sp[-1].type != T_INT)
-	DEFAULT_ERRNO();
-    return -Pike_sp[-1].u.integer;
-}
+  push_text("owner"); push_int( fi->lock_owner );
+  ref_push_string(literal_type_string);  push_int( lck->l_type );
+  push_text("whence");push_int( lck->l_whence );
+  push_text("start");   push_int( lck->l_start );
+  push_text("len");   push_int( lck->l_len );
+  push_text("pid");   push_int( lck->l_pid );
+  f_aggregate_mapping( 6 * 2 );
+  apply( global_fuse_obj, "lock", 3 );
 
-/* static int pf_readdir( const char *path,  */
-/* 		       struct fuse_fill_dir_t fill, off_t off,  */
-/* 		       struct fuse_file_info *info ) */
-/* { */
-/*     push_text( path ); */
-/*     push_int( off ); */
-    
-/* } */
+  if( TYPEOF(Pike_sp[-1]) == PIKE_T_MAPPING )
+  {
+    struct svalue *tmp;
+#define COUT(X)                                                         \
+    if((tmp=simple_mapping_string_lookup(Pike_sp[-1].u.mapping,#X))     \
+       && TYPEOF(*tmp)==PIKE_T_INT) lck->l_##X = tmp->u.integer;
+    COUT(type);  COUT(whence);
+    COUT(start);  COUT(len);
+    COUT(pid);
+#undef COUT
+    return 0;
+  }
+  else
+  {
+    if (TYPEOF(Pike_sp[-1]) != T_INT)
+      DEFAULT_ERRNO();
+    return -Pike_sp[-1].u.integer;
+  }
+}
 
 static struct fuse_operations pike_fuse_oper = {
     .getattr	= pf_getattr,
@@ -482,22 +471,24 @@ static struct fuse_operations pike_fuse_oper = {
     .chmod	= pf_chmod,
     .chown	= pf_chown,
     .truncate	= pf_truncate,
+#if FUSE_VERSION >= 26
+    .utimens	= pf_utimens,
+#else
     .utime	= pf_utime,
+#endif
+#if FUSE_VERSION >= 26
+    .lock	= pf_lock,
+#endif
     .open	= pf_open,
     .read	= pf_read,
     .write	= pf_write,
     .statfs	= pf_statfs,
-/*     .flush	= pf_flush, */
     .release	= pf_release,
     .fsync	= pf_fsync,
     .setxattr	= pf_setxattr,
     .getxattr	= pf_getxattr,
     .listxattr	= pf_listxattr,
     .removexattr= pf_removexattr,
-/*     .opendir    = pf_opendir, */
-/*     .readdir    = pf_readdir, */
-/*     .releasedir = pf_releasedir, */
-/*     .fsyncdir   = pf_fsyncdir, */
     .access     = pf_access,
     .create     = pf_creat, 
 };
@@ -515,32 +506,12 @@ static void low_dispatch_fuse_command( void *ptr )
     pop_n_elems( Pike_sp - old );
 }
 
-static void dispatch_fuse_command( struct fuse *f, struct fuse_cmd *cmd, void *a )
+static void dispatch_fuse_command( struct fuse *f, struct fuse_cmd *cmd, void *UNUSED(a) )
 {
     struct passon x = {
 	.f = f,
 	.cmd = cmd
     };
-    struct thread_state *state;
-
-    if((state = thread_state_for_id(th_self()))==NULL) 
-    {
-	struct object *thread_obj;
-	fprintf( stderr, "Creating a new pike-thread\n");
-
-	mt_lock_interpreter();
-	init_interpreter();
-	Pike_interpreter.stack_top=((char *)&state)+ (thread_stack_size-16384) * STACK_DIRECTION;
-	Pike_interpreter.recoveries = NULL;
-	thread_obj = fast_clone_object(thread_id_prog);
-	INIT_THREAD_STATE((struct thread_state *)(thread_obj->storage +
-						  thread_storage_offset));
-	num_threads++;
-	thread_table_insert(Pike_interpreter.thread_state);
-	state = Pike_interpreter.thread_state;
-	SWAP_OUT_THREAD(state);
-	mt_unlock_interpreter();
-    }
 
     call_with_interpreter(low_dispatch_fuse_command, &x );
 }
@@ -576,7 +547,7 @@ static void f_fuse_run( INT32 nargs )
     argv = malloc( sizeof(char *) * args->size );
     for( i = 0; i<args->size; i++ )
     {
-	if( args->item[i].type != PIKE_T_STRING ||
+        if( TYPEOF(args->item[i]) != PIKE_T_STRING ||
 	    string_has_null(args->item[i].u.string) )
 	{
 	    free( argv );
@@ -619,6 +590,12 @@ PIKE_MODULE_EXIT
   free_program( fuse_cmd_program );
 }
 
+/*! @decl constant FUSE_MAJOR_VERSION
+ *! @decl constant FUSE_MINOR_VERSION
+ *!
+ *!  The version of FUSE
+ */
+
 PIKE_MODULE_INIT
 {
     ADD_FUNCTION( "run", f_fuse_run, tFunc(tObj tArr(tStr),tVoid ), 0 );
@@ -637,6 +614,12 @@ PIKE_MODULE_INIT
     }
     fuse_cmd_program = end_program();
 
+    add_integer_constant("F_RDLCK", F_GETLK, 0 );
+    add_integer_constant("F_WRLCK", F_WRLCK, 0 );
+    add_integer_constant("F_UNLCK", F_UNLCK, 0 );
+    add_integer_constant("F_GETLK", F_GETLK, 0 );
+    add_integer_constant("F_SETLK", F_SETLK, 0 );
+    add_integer_constant("F_SETLKW", F_SETLKW, 0 );
     add_integer_constant("FUSE_MAJOR_VERSION", FUSE_MAJOR_VERSION, 0);
     add_integer_constant("FUSE_MINOR_VERSION", FUSE_MINOR_VERSION, 0);
 }
