@@ -663,6 +663,9 @@ static void mpzmod_create(INT32 args)
 }
 
 /*! @decl int cast_to_int()
+ *! Casts the object to an integer.
+ *! @deprecated
+ *!   Use @expr{(int)@} instead.
  */
 static void mpzmod_get_int(INT32 args)
 {
@@ -732,6 +735,9 @@ static void mpzmod___hash(INT32 args)
 }
 
 /*! @decl float cast_to_float()
+ *! Casts the object to a float.
+ *! @deprecated
+ *!   Use @expr{(float)@} instead.
  */
 static void mpzmod_get_float(INT32 args)
 {
@@ -793,6 +799,9 @@ struct pike_string *low_get_mpz_digits(MP_INT *mpz, int base)
 }
 
 /*! @decl string cast_to_string()
+ *! Casts the object to a string.
+ *! @deprecated
+ *!   Use @expr{(string)@} instead.
  */
 static void mpzmod_get_string(INT32 args)
 {
@@ -2158,7 +2167,7 @@ static void gmp_fac(INT32 args)
   if (TYPEOF(sp[-1]) != T_INT)
     SIMPLE_ARG_TYPE_ERROR ("fac", 1, "int");
   if (sp[-1].u.integer < 0)
-    SIMPLE_ARG_ERROR ("fac", 1, "Got negative exponent.");
+    SIMPLE_ARG_ERROR ("fac", 1, "Got negative factorial.");
   res = fast_clone_object(mpzmod_program);
   mpz_fac_ui(OBTOMPZ(res), sp[-1].u.integer);
   pop_n_elems(args);
