@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 9242e24ddff1af585f4fb9683c876d4c6d92f6ca $
 */
 
 #include "config.h"
@@ -42,17 +41,17 @@
 
 #define sp Pike_sp
 
-int num_log_entries;
-void free_log_entry( struct log_entry *le )
+static int num_log_entries;
+static void free_log_entry( struct log_entry *le )
 {
   num_log_entries--;
-  aap_free( le );
+  free( le );
 }
 
-struct log_entry *new_log_entry(ptrdiff_t extra)
+static struct log_entry *new_log_entry(ptrdiff_t extra)
 {
   num_log_entries++;
-  return aap_malloc( sizeof( struct log_entry )+extra );
+  return malloc( sizeof( struct log_entry )+extra );
 }
 
 
@@ -111,7 +110,7 @@ void f_aap_log_as_array(INT32 args)
   }
 }
 
-void f_aap_log_exists(INT32 args)
+void f_aap_log_exists(INT32 UNUSED(args))
 {
   if(LTHIS->log->log_head) 
     push_int(1);
@@ -119,7 +118,7 @@ void f_aap_log_exists(INT32 args)
     push_int(0);
 }
 
-void f_aap_log_size(INT32 args)
+void f_aap_log_size(INT32 UNUSED(args))
 {
   int n=1;
   struct log *l = LTHIS->log;
@@ -141,7 +140,7 @@ void f_aap_log_as_commonlog_to_file(INT32 args)
   struct log_entry *le;
   struct log *l = LTHIS->log;
   int n = 0;
-  int mfd, ot=0;
+  int mfd, ot = INT_MIN;
   struct object *f;
   struct tm tm;
   FILE *foo;
@@ -170,10 +169,11 @@ void f_aap_log_as_commonlog_to_file(INT32 args)
   l->log_head = l->log_tail = 0;
   mt_unlock( &l->log_lock );
 
+  memset(&tm, 0, sizeof(tm));
+
   while(le)
   {
     int i;
-    struct tm *tm_p;
     struct log_entry *l = le->next;
     /* remotehost rfc931 authuser [date] "request" status bytes */
     if(le->t != ot)
@@ -182,16 +182,9 @@ void f_aap_log_as_commonlog_to_file(INT32 args)
 #ifdef HAVE_GMTIME_R
       gmtime_r( &t, &tm );
 #else
-#ifdef HAVE_GMTIME
+      struct tm *tm_p;
       tm_p = gmtime( &t ); /* This will break if two threads run
 			    gmtime() at once. */
-
-#else
-#ifdef HAVE_LOCALTIME
-      tm_p = localtime( &t ); /* This will break if two threads run
-			       localtime() at once. */
-#endif
-#endif
       if (tm_p) tm = *tm_p;
 #endif
       ot = le->t;
@@ -256,7 +249,7 @@ void aap_log_append(int sent, struct args *arg, int reply)
   le->sent_bytes = sent;
   le->reply = reply;
   le->received_bytes = arg->res.body_start + arg->res.content_len;
-  MEMCPY(data_to, arg->res.data, arg->res.body_start-4);
+  memcpy(data_to, arg->res.data, arg->res.body_start-4);
   le->raw.str = data_to;
   le->raw.len = arg->res.body_start-4;
   le->url.str = (data_to + (size_t)(arg->res.url-arg->res.data));
