@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 341adaa6f1e4e4cfe2b1272db736e82cf9eeb155 $
 */
 
 /*
@@ -141,8 +140,10 @@
 **!    
 **!    array grey=Array.filter(cs,lambda(object c) { return c->s==0; });
 **!    array colored=cs-grey;
-**! 
+**!
+**!    sort(grey->name(), grey);
 **!    sort(grey->v,grey);
+**!    sort(colored->name(), colored);
 **!    sort(Array.map(colored,lambda(object c)
 **! 			      {
 **! 				 return (c->h*50-c->s)*10+c->v;
@@ -208,6 +209,7 @@
 #include "module_support.h"
 #include "sscanf.h"
 #include "program_id.h"
+#include "pike_types.h"
 
 #include "image.h"
 #include "colortable.h"
@@ -222,8 +224,6 @@ static struct array *colornames=NULL;
 struct program *image_color_program=NULL;
 extern struct program *image_colortable_program;
 
-static struct pike_string *str_array;
-static struct pike_string *str_string;
 static struct pike_string *str_r;
 static struct pike_string *str_g;
 static struct pike_string *str_b;
@@ -282,8 +282,7 @@ static void make_colors(void)
       copy_shared_string(c[i].pname,sp[-1].u.string);
 
       push_object(clone_object(image_color_program,0)); 
-      cs=(struct color_struct*)
-	 get_storage(sp[-1].u.object,image_color_program);
+      cs=get_storage(sp[-1].u.object,image_color_program);
       cs->rgb.r=(COLORTYPE)c[i].r;
       cs->rgb.g=(COLORTYPE)c[i].g;
       cs->rgb.b=(COLORTYPE)c[i].b;
@@ -329,13 +328,13 @@ static void make_colors(void)
 #define THIS ((struct color_struct*)(Pike_fp->current_storage))
 #define THISOBJ (Pike_fp->current_object)
 
-static void init_color_struct(struct object *dummy)
+static void init_color_struct(struct object *UNUSED(dummy))
 {
    THIS->rgb.r=THIS->rgb.g=THIS->rgb.b=0;
    THIS->name=NULL;
 }
 
-static void exit_color_struct(struct object *dummy)
+static void exit_color_struct(struct object *UNUSED(dummy))
 {
    if (THIS->name) 
    {
@@ -459,7 +458,7 @@ static void image_color_greylevel(INT32 args)
    }
    else
    {
-      get_all_args("Image.Color.Color->greylevel()",args,"%i%i%i",&r,&g,&b);
+      get_all_args("greylevel",args,"%i%i%i",&r,&g,&b);
    }
    pop_n_elems(args);
    if (r+g+b==0) r=g=b=1;
@@ -658,12 +657,12 @@ static void image_color_hex(INT32 args)
    INT_TYPE i=sizeof(COLORTYPE)*2;
 
    if (args)
-      get_all_args("Image.Color.Color->hex()",args,"%i",&i);
+      get_all_args("hex",args,"%i",&i);
 
    pop_n_elems(args);
    if (i<1)
    {
-      push_constant_text("#");  /* stupid */
+      push_text("#");  /* stupid */
       return;
    }
    else if (i!=sizeof(COLORTYPE)*2)
@@ -758,21 +757,28 @@ static void image_color_name(INT32 args)
 static void image_color_cast(INT32 args)
 {
    if (args!=1 ||
-       sp[-1].type!=T_STRING)
-      bad_arg_error("Image.Color.Color->cast",sp-args,args,0,"",sp-args,
-		"Bad arguments to Image.Color.Color->cast()\n");
+       TYPEOF(sp[-1]) != T_STRING)
+     bad_arg_error("cast",sp-args,args,0,"",sp-args,
+                   "Bad arguments to cast.\n");
    
-   if (sp[-1].u.string==str_array)
+   if (sp[-1].u.string==literal_array_string)
    {
       image_color_rgb(args);
       return;
    }
-   if (sp[-1].u.string==str_string)
+   if (sp[-1].u.string==literal_string_string)
    {
       image_color_name(args);
       return;
    }
-   Pike_error("Image.Color.Color->cast(): Can't cast to that\n");
+   if (sp[-1].u.string==literal_int_string)
+   {
+     pop_stack();
+     push_int( (THIS->rgb.r << 8 | THIS->rgb.g)  << 8 | THIS->rgb.b );
+     return;
+   }
+   pop_stack();
+   push_undefined();
 }
 
 static void image_color__sprintf(INT32 args)
@@ -782,16 +788,16 @@ static void image_color__sprintf(INT32 args)
    if (args<2)
       SIMPLE_TOO_FEW_ARGS_ERROR("_sprintf",2);
 
-   if (sp[-args].type!=T_INT)
-      SIMPLE_BAD_ARG_ERROR("_sprintf",0,"integer");
-   if (sp[1-args].type!=T_MAPPING)
+   if (TYPEOF(sp[-args]) != T_INT)
+      SIMPLE_BAD_ARG_ERROR("_sprintf",0,"int");
+   if (TYPEOF(sp[1-args]) != T_MAPPING)
       SIMPLE_BAD_ARG_ERROR("_sprintf",1,"mapping");
 
    pop_n_elems(args-2);
 
    push_text("precision");
    f_index(2);
-   if (sp[-1].type!=T_INT)
+   if (TYPEOF(sp[-1]) != T_INT)
       SIMPLE_BAD_ARG_ERROR("_sprintf",1,"mapping(\"precision\":int)");
    prec=sp[-1].u.integer;
    x=sp[-2].u.integer;
@@ -802,13 +808,13 @@ static void image_color__sprintf(INT32 args)
 /*       case 'c': */
 /*       case 'd': */
       case 't':
-	 push_constant_text("Image.Color.Color");
+	 push_text("Image.Color.Color");
 	 return;
       case 'O':
 	 if (!THIS->name) try_find_name(THIS);
 	 if (THIS->name==no_name)
 	 {
-	    push_constant_text("Image.Color(\"");
+	    push_text("Image.Color(\"");
 	    if (prec)
 	    {
 	       push_int(prec);
@@ -816,13 +822,13 @@ static void image_color__sprintf(INT32 args)
 	    }
 	    else
 	       image_color_hex(0);
-	    push_constant_text("\")");
+	    push_text("\")");
 	    f_add(3);
 	    return;
 	 }
 	 else
 	 {
-	    push_constant_text("Image.Color.");
+	    push_text("Image.Color.");
 	    ref_push_string(THIS->name);
 	    f_add(2);
 	    return;
@@ -864,7 +870,7 @@ static void image_color_index(INT32 args)
       Pike_error("Image.Color[]: illegal number of arguments\n");
 
    object_index_no_free2(&s, THISOBJ, 0, sp-1);
-   if (s.type==T_INT && sp[-1].type==T_STRING)
+   if (TYPEOF(s) == T_INT && TYPEOF(sp[-1]) == T_STRING)
    {
       if (sp[-1].u.string==str_r)
       {
@@ -943,11 +949,10 @@ static void image_color_equal(INT32 args)
    if (args!=1) 
       Pike_error("Image.Color.Color->`==: illegal number of arguments\n");
 
-   if (sp[-1].type==T_OBJECT)
+   if (TYPEOF(sp[-1]) == T_OBJECT)
    {
       struct color_struct *other;
-      other=(struct color_struct*)
-	 get_storage(sp[-1].u.object,image_color_program);
+      other=get_storage(sp[-1].u.object,image_color_program);
       if (other&&
 	  other->rgbl.r==THIS->rgbl.r &&
 	  other->rgbl.g==THIS->rgbl.g &&
@@ -958,12 +963,12 @@ static void image_color_equal(INT32 args)
 	 return;
       }
    }
-   else if (sp[-1].type==T_ARRAY)
+   else if (TYPEOF(sp[-1]) == T_ARRAY)
    {
       if (sp[-1].u.array->size==3 &&
-	  sp[-1].u.array->item[0].type==T_INT &&
-	  sp[-1].u.array->item[1].type==T_INT &&
-	  sp[-1].u.array->item[2].type==T_INT &&
+	  TYPEOF(sp[-1].u.array->item[0]) == T_INT &&
+	  TYPEOF(sp[-1].u.array->item[1]) == T_INT &&
+	  TYPEOF(sp[-1].u.array->item[2]) == T_INT &&
 	  sp[-1].u.array->item[0].u.integer == THIS->rgb.r &&
 	  sp[-1].u.array->item[1].u.integer == THIS->rgb.g &&
 	  sp[-1].u.array->item[2].u.integer == THIS->rgb.b)
@@ -973,7 +978,7 @@ static void image_color_equal(INT32 args)
 	 return;
       }
    }
-/*    else if (sp[-1].type==T_INT) */
+/*    else if (TYPEOF(sp[-1]) == T_INT) */
 /*    { */
 /*       if (sp[-1].u.integer == THIS->rgb.r && */
 /* 	  THIS->rgb.r==THIS->rgb.g && */
@@ -984,7 +989,7 @@ static void image_color_equal(INT32 args)
 /* 	 return; */
 /*       } */
 /*    } */
-   else if (sp[-1].type==T_STRING)
+   else if (TYPEOF(sp[-1]) == T_STRING)
    {
       if (!THIS->name)
 	 try_find_name(THIS);
@@ -1160,7 +1165,7 @@ static void image_color_bright(INT32 args)
 static void image_color_mult(INT32 args)
 {
    FLOAT_TYPE x=0.0;
-   get_all_args("Image.Color.Color->`*",args,"%f",&x);
+   get_all_args("`*",args,"%f",&x);
    pop_n_elems(args);
    _image_make_rgb_color(DOUBLE_TO_INT(THIS->rgb.r*x),
 			 DOUBLE_TO_INT(THIS->rgb.g*x),
@@ -1169,10 +1174,9 @@ static void image_color_mult(INT32 args)
 
 int image_color_svalue(struct svalue *v,rgb_group *rgb)
 {
-   if (v->type==T_OBJECT)
+   if (TYPEOF(*v) == T_OBJECT)
    {
-      struct color_struct *cs=(struct color_struct*)
-	 get_storage(v->u.object,image_color_program);
+      struct color_struct *cs=get_storage(v->u.object,image_color_program);
 
       if (cs) 
       {
@@ -1180,12 +1184,12 @@ int image_color_svalue(struct svalue *v,rgb_group *rgb)
 	 return 1;
       }
    }
-   else if (v->type==T_ARRAY)
+   else if (TYPEOF(*v) == T_ARRAY)
    {
       if (v->u.array->size==3 &&
-	  v->u.array->item[0].type==T_INT &&
-	  v->u.array->item[1].type==T_INT &&
-	  v->u.array->item[2].type==T_INT)
+	  TYPEOF(v->u.array->item[0]) == T_INT &&
+	  TYPEOF(v->u.array->item[1]) == T_INT &&
+	  TYPEOF(v->u.array->item[2]) == T_INT)
       {
 	 rgb->r=(COLORTYPE)(v->u.array->item[0].u.integer);
 	 rgb->g=(COLORTYPE)(v->u.array->item[1].u.integer);
@@ -1193,14 +1197,13 @@ int image_color_svalue(struct svalue *v,rgb_group *rgb)
 	 return 1;
       }
    }
-   else if (v->type==T_STRING)
+   else if (TYPEOF(*v) == T_STRING)
    {
       push_svalue(v);
       image_make_color(1);
-      if (sp[-1].type==T_OBJECT)
+      if (TYPEOF(sp[-1]) == T_OBJECT)
       {
-	 struct color_struct *cs=(struct color_struct*)
-	    get_storage(sp[-1].u.object,image_color_program);
+	 struct color_struct *cs=get_storage(sp[-1].u.object,image_color_program);
 	 *rgb=cs->rgb;
 	 pop_stack();
 	 return 1;
@@ -1222,7 +1225,7 @@ static void image_color_add(INT32 args)
    rgb_group rgb;
 
    if (!image_color_arg(-args,&rgb))
-      SIMPLE_BAD_ARG_ERROR("Image.Color.Color->`+",1,"Color");
+      SIMPLE_BAD_ARG_ERROR("`+",1,"Image.Color");
 
    pop_n_elems(args);
    _image_make_rgb_color((int)(THIS->rgb.r+rgb.r),
@@ -1249,10 +1252,10 @@ static void image_get_color(INT32 args)
    if (!colors)
       make_colors();
 
-   if (sp[-1].type==T_STRING)
+   if (TYPEOF(sp[-1]) == T_STRING)
    {
       mapping_index_no_free(&s,colors,sp-1);
-      if (s.type==T_OBJECT)
+      if (TYPEOF(s) == T_OBJECT)
       {
 	 pop_stack();
 	 *(sp++)=s;
@@ -1262,7 +1265,7 @@ static void image_get_color(INT32 args)
 	 free_svalue(&s);
    }
 
-   if (sp[-1].type==T_STRING &&
+   if (TYPEOF(sp[-1]) == T_STRING &&
        sp[-1].u.string->size_shift==0)
    {
       if (sp[-1].u.string->len>=4 &&
@@ -1284,8 +1287,7 @@ static void image_get_color(INT32 args)
 		  if (HEXTONUM(*src)==-1)
 		  {
 		     pop_stack();
-		     push_int(0);
-		     sp[-1].subtype=NUMBER_UNDEFINED;
+		     push_undefined();
 		     return;
 		  }
 		  z=z*16+HEXTONUM(*src),src++;
@@ -1323,7 +1325,7 @@ static void image_get_color(INT32 args)
 	 stack_dup();
 	 push_text("@%f,%f,%f\n");
 	 f_sscanf(2);
-	 if (sp[-1].type==T_ARRAY &&
+	 if (TYPEOF(sp[-1]) == T_ARRAY &&
 	     sp[-1].u.array->size==3)
 	 {
             FLOAT_TYPE h,s,v;
@@ -1332,7 +1334,7 @@ static void image_get_color(INT32 args)
 	    sp--;
 	    dmalloc_touch_svalue(sp);
 	    push_array_items(sp->u.array);
-	    get_all_args("Image.Color()",3,"%f%f%f",&h,&s,&v);
+	    get_all_args("create",3,"%f%f%f",&h,&s,&v);
 	    pop_n_elems(3);
 	    push_int(DOUBLE_TO_INT(h/360.0*256.0));
 	    push_int(DOUBLE_TO_INT(s/100.0*255.4));
@@ -1349,7 +1351,7 @@ static void image_get_color(INT32 args)
 	 stack_dup();
 	 push_text("%%%f,%f,%f,%f\n");
 	 f_sscanf(2);
-	 if (sp[-1].type==T_ARRAY &&
+	 if (TYPEOF(sp[-1]) == T_ARRAY &&
 	     sp[-1].u.array->size==4)
 	 {
 	    stack_swap();
@@ -1370,7 +1372,7 @@ static void image_get_color(INT32 args)
 	    push_int(1000000);
 	    f_index(3);
 	    image_get_color(1);
-	    if (sp[-1].type!=T_OBJECT) return; /* no way */
+	    if (TYPEOF(sp[-1]) != T_OBJECT) return; /* no way */
 	    safe_apply(sp[-1].u.object,callables[n],0);
 	    stack_swap();
 	    pop_stack();
@@ -1383,7 +1385,7 @@ static void image_get_color(INT32 args)
 	 stack_dup();
 	 push_text("gr%*[ea]y%f\n");
 	 f_sscanf(2);
-	 if (sp[-1].type==T_ARRAY &&
+	 if (TYPEOF(sp[-1]) == T_ARRAY &&
 	     sp[-1].u.array->size==1)
 	 {
 	    double f;	
@@ -1405,31 +1407,30 @@ static void image_get_color(INT32 args)
    /* try other stuff here */
 
    pop_stack();
-   push_int(0);
-   sp[-1].subtype=NUMBER_UNDEFINED;
+   push_undefined();
    return;
 }
 
 static void image_guess_color(INT32 args)
 {
-   if (args!=1 && sp[-args].type!=T_STRING) 
-      bad_arg_error("Image.Color->guess",sp-args,args,0,"",sp-args,
-		"Bad arguments to Image.Color->guess()\n");
+   if (args!=1 && TYPEOF(sp[-args]) != T_STRING) 
+     bad_arg_error("guess",sp-args,args,0,"",sp-args,
+                   "Bad arguments to guess.\n");
    
    f_lower_case(1);
-   push_constant_text(" ");
+   push_text(" ");
    o_subtract();
 
    stack_dup();
    image_get_color(1);
-   if (sp[-1].type==T_OBJECT)
+   if (TYPEOF(sp[-1]) == T_OBJECT)
    {
       stack_swap();
       pop_stack();
       return;
    }
    pop_stack();
-   push_constant_text("#");
+   push_text("#");
    stack_swap();
    f_add(2);
 
@@ -1440,7 +1441,7 @@ static void image_colors_index(INT32 args)
 {
    struct svalue s;
    object_index_no_free2(&s, THISOBJ, 0, sp-1);
-   if (s.type!=T_INT)
+   if (TYPEOF(s) != T_INT)
    {
       pop_stack();
       *(sp++)=s;
@@ -1451,7 +1452,7 @@ static void image_colors_index(INT32 args)
 
 static void image_make_color(INT32 args)
 {
-   if (args==1 && sp[-args].type==T_STRING) 
+   if (args==1 && TYPEOF(sp[-args]) == T_STRING) 
    {
       image_get_color(args);
       return;
@@ -1499,8 +1500,7 @@ static void _image_make_rgbl_color(INT32 r,INT32 g,INT32 b)
 
    push_object(clone_object(image_color_program,0));
 
-   cs=(struct color_struct*)
-      get_storage(sp[-1].u.object,image_color_program);
+   cs=get_storage(sp[-1].u.object,image_color_program);
 
    cs->rgbl.r=(INT32)r;
    cs->rgbl.g=(INT32)g;
@@ -1508,7 +1508,7 @@ static void _image_make_rgbl_color(INT32 r,INT32 g,INT32 b)
    RGBL_TO_RGB(cs->rgb,cs->rgbl);
 }
 
-static void image_color__encode( INT32 args )
+static void image_color__encode( INT32 UNUSED(args) )
 {
     push_int( THIS->rgbl.r );
     push_int( THIS->rgbl.g );
@@ -1516,10 +1516,10 @@ static void image_color__encode( INT32 args )
     f_aggregate( 3 );
 }
 
-static void image_color__decode( INT32 args )
+static void image_color__decode( INT32 UNUSED(args) )
 {
     struct svalue *a;
-    if( Pike_sp[-1].type != PIKE_T_ARRAY || Pike_sp[-1].u.array->size != 3 )
+    if( TYPEOF(Pike_sp[-1]) != PIKE_T_ARRAY || Pike_sp[-1].u.array->size != 3 )
 	Pike_error("Illegal argument to _decode\n");
     a=Pike_sp[-1].u.array->item;
     THIS->rgbl.r = a[0].u.integer;
@@ -1546,8 +1546,7 @@ void _image_make_rgb_color(INT32 r,INT32 g,INT32 b)
 
    push_object(clone_object(image_color_program,0));
 
-   cs=(struct color_struct*)
-      get_storage(sp[-1].u.object,image_color_program);
+   cs=get_storage(sp[-1].u.object,image_color_program);
 
    cs->rgb.r=(COLORTYPE)r;
    cs->rgb.g=(COLORTYPE)g;
@@ -1559,7 +1558,17 @@ static void image_make_rgb_color(INT32 args)
 {
    INT_TYPE r=0,g=0,b=0;
 
-   get_all_args("Image.Color.rgb()",args,"%i%i%i",&r,&g,&b);
+   if( args==1 && TYPEOF(sp[-1]) == T_INT )
+   {
+     r = sp[-1].u.integer;
+     b = r & 0xff;
+     r >>= 8;
+     g = r & 0xff;
+     r >>= 8;
+     r &= 0xff;
+   }
+   else
+     get_all_args("rgb",args,"%i%i%i",&r,&g,&b);
 
    _image_make_rgb_color(r,g,b);
 }
@@ -1569,11 +1578,10 @@ static void image_make_hsv_color(INT32 args)
    FLOAT_TYPE h,s,v;
    FLOAT_TYPE r=0,g=0,b=0; /* to avoid warning */
 
-   if (args && sp[-args].type==T_INT)
+   if (args && TYPEOF(sp[-args]) == T_INT)
    {
       INT_TYPE hi,si,vi;
-      get_all_args("Image.Color.hsv()",args,"%i%i%i",
-		   &hi,&si,&vi);
+      get_all_args("hsv",args,"%i%i%i",&hi,&si,&vi);
       pop_n_elems(args);
 
       if (hi<0) hi=(hi%COLORMAX)+COLORMAX; 
@@ -1587,8 +1595,7 @@ static void image_make_hsv_color(INT32 args)
    }
    else
    {
-      get_all_args("Image.Color.hsv()",args,"%f%f%f",
-		   &h,&s,&v);
+      get_all_args("hsv",args,"%f%f%f",&h,&s,&v);
       pop_n_elems(args);
       if (h<0) h = 360 + h - ((DOUBLE_TO_INT(h)/360)*360);
       if (h>360.0) h -= ((DOUBLE_TO_INT(h)/360)*360);
@@ -1630,7 +1637,7 @@ static void image_make_hsv_color(INT32 args)
 static void image_make_cmyk_color(INT32 args)
 {
    FLOAT_TYPE c,m,y,k,r,g,b;
-   get_all_args("Image.Color.cmyk()",args,"%F%F%F%F",&c,&m,&y,&k);
+   get_all_args("cmyk",args,"%F%F%F%F",&c,&m,&y,&k);
    pop_n_elems(args);
 
    r=100-(c+k);
@@ -1644,7 +1651,7 @@ static void image_make_greylevel_color(INT32 args)
 {
    INT_TYPE i;
 
-   get_all_args("Image.Color.greylevel()",args,"%i",&i);
+   get_all_args("greylevel",args,"%i",&i);
    pop_n_elems(args);
 
    _image_make_rgb_color(i,i,i);
@@ -1655,10 +1662,10 @@ static void image_make_html_color(INT32 args)
    int i;
 
    if (args!=1 ||
-       sp[-1].type!=T_STRING) 
+       TYPEOF(sp[-1]) != T_STRING) 
    {
-      bad_arg_error("Image.Color.html",sp-args,args,0,"",sp-args,
-		"Bad arguments to Image.Color.html()\n");
+     bad_arg_error("html",sp-args,args,0,"",sp-args,
+                   "Bad arguments to html.\n");
       return;
    }
    
@@ -1677,7 +1684,7 @@ static void image_make_html_color(INT32 args)
       image_get_color(1);
    else
    {
-      push_constant_text("#");
+      push_text("#");
       stack_swap();
       f_add(2);
       image_get_color(1);
@@ -1711,8 +1718,6 @@ static void image_colors_values(INT32 args)
 
 void init_image_colors(void)
 {
-   str_array=make_shared_string("array");
-   str_string=make_shared_string("string");
    str_r=make_shared_string("r");
    str_g=make_shared_string("g");
    str_b=make_shared_string("b");
@@ -1732,12 +1737,12 @@ void init_image_colors(void)
 
    /* color info methods */
 
-   ADD_FUNCTION("cast",image_color_cast,tFunc(tStr,tOr(tArray,tStr)),0);
+   ADD_FUNCTION("cast",image_color_cast,tFunc(tStr,tOr(tArray,tStr)),ID_PROTECTED);
    ADD_FUNCTION("_sprintf",image_color__sprintf,
 		tFunc(tInt tMap(tStr,tMix),tStr),0);
    ADD_FUNCTION("`[]",image_color_index,tFunc(tOr(tStr,tInt),tOr(tInt,tFunction)),0);
    ADD_FUNCTION("`->",image_color_index,tFunc(tOr(tStr,tInt),tOr(tInt,tFunction)),0);
-   ADD_FUNCTION("`==",image_color_equal,tFunc(tOr(tObjImpl_IMAGE_COLOR_COLOR_ID,tInt),
+   ADD_FUNCTION("`==",image_color_equal,tFunc(tOr(tObjImpl_IMAGE_COLOR_COLOR,tInt),
 					      tInt),0);
    ADD_FUNCTION("__hash",image_color___hash,tFunc(tNone,tInt),0);
 
@@ -1760,18 +1765,18 @@ void init_image_colors(void)
    /* color conversion methods */
 
    ADD_FUNCTION("grey",image_color_grey,
-		tOr(tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR_ID),
-		    tFunc(tInt tInt tInt,tObjIs_IMAGE_COLOR_COLOR_ID)),0);
+		tOr(tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR),
+		    tFunc(tInt tInt tInt,tObjIs_IMAGE_COLOR_COLOR)),0);
 
-   ADD_FUNCTION("light",image_color_light,tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR_ID),0);
-   ADD_FUNCTION("dark",image_color_dark,tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR_ID),0);
-   ADD_FUNCTION("neon",image_color_neon,tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR_ID),0);
-   ADD_FUNCTION("bright",image_color_bright,tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR_ID),0);
-   ADD_FUNCTION("dull",image_color_dull,tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR_ID),0);
+   ADD_FUNCTION("light",image_color_light,tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR),0);
+   ADD_FUNCTION("dark",image_color_dark,tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR),0);
+   ADD_FUNCTION("neon",image_color_neon,tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR),0);
+   ADD_FUNCTION("bright",image_color_bright,tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR),0);
+   ADD_FUNCTION("dull",image_color_dull,tFunc(tNone,tObjIs_IMAGE_COLOR_COLOR),0);
 
-   ADD_FUNCTION("`*",image_color_mult,tFunc(tFlt,tObjIs_IMAGE_COLOR_COLOR_ID),0);
-   ADD_FUNCTION("`+",image_color_add,tFunc(tObjImpl_IMAGE_COLOR_COLOR_ID,
-					   tObjIs_IMAGE_COLOR_COLOR_ID),0);
+   ADD_FUNCTION("`*",image_color_mult,tFunc(tFlt,tObjIs_IMAGE_COLOR_COLOR),0);
+   ADD_FUNCTION("`+",image_color_add,tFunc(tObjImpl_IMAGE_COLOR_COLOR,
+					   tObjIs_IMAGE_COLOR_COLOR),0);
 
    image_color_program=end_program();
    image_color_program->flags |= 
@@ -1782,30 +1787,32 @@ void init_image_colors(void)
 
    /* this is the Image.Color stuff */
    
-   ADD_FUNCTION("`[]",image_colors_index,tFunc(tStr,tObjIs_IMAGE_COLOR_COLOR_ID),0);
+   ADD_FUNCTION("`[]",image_colors_index,tFunc(tStr,tObjIs_IMAGE_COLOR_COLOR),0);
    ADD_FUNCTION("`()",image_make_color,
-		tOr(tFunc(tStr,tObjIs_IMAGE_COLOR_COLOR_ID),
-		    tFunc(tInt tInt tInt,tObjIs_IMAGE_COLOR_COLOR_ID)),0);
+		tOr3(tFunc(tStr,tObjIs_IMAGE_COLOR_COLOR),
+                     tFunc(tInt,tObjIs_IMAGE_COLOR_COLOR),
+                     tFunc(tInt tInt tInt,tObjIs_IMAGE_COLOR_COLOR)),0);
    ADD_FUNCTION("rgb",image_make_rgb_color,
-		tFunc(tInt tInt tInt,tObjIs_IMAGE_COLOR_COLOR_ID),0);
+		tOr(tFunc(tInt,tObjIs_IMAGE_COLOR_COLOR),
+                    tFunc(tInt tInt tInt,tObjIs_IMAGE_COLOR_COLOR)),0);
    ADD_FUNCTION("hsv",image_make_hsv_color,
-		tOr(tFunc(tInt tInt tInt,tObjIs_IMAGE_COLOR_COLOR_ID),
-		    tFunc(tFlt tFlt tFlt,tObjIs_IMAGE_COLOR_COLOR_ID)) ,0);
+		tOr(tFunc(tInt tInt tInt,tObjIs_IMAGE_COLOR_COLOR),
+		    tFunc(tFlt tFlt tFlt,tObjIs_IMAGE_COLOR_COLOR)),0);
    ADD_FUNCTION("cmyk",image_make_cmyk_color,tFunc(tOr(tInt,tFlt) 
 						   tOr(tInt,tFlt) 
 						   tOr(tInt,tFlt) 
 						   tOr(tInt,tFlt),
-						   tObjIs_IMAGE_COLOR_COLOR_ID), 0);
+						   tObjIs_IMAGE_COLOR_COLOR), 0);
    ADD_FUNCTION("html",image_make_html_color,
-		tFunc(tStr,tObjIs_IMAGE_COLOR_COLOR_ID),0);
+		tFunc(tStr,tObjIs_IMAGE_COLOR_COLOR),0);
    ADD_FUNCTION("guess",image_guess_color,
-		tFunc(tStr,tObjIs_IMAGE_COLOR_COLOR_ID),0);
+		tFunc(tStr,tObjIs_IMAGE_COLOR_COLOR),0);
    ADD_FUNCTION("greylevel",image_make_greylevel_color,
-		tFunc(tInt,tObjIs_IMAGE_COLOR_COLOR_ID),0);
+		tFunc(tInt,tObjIs_IMAGE_COLOR_COLOR),0);
    ADD_FUNCTION("_indices",image_colors_indices,
 		tFunc(tNone,tArr(tStr)),0);
    ADD_FUNCTION("_values",image_colors_values,
-		tFunc(tNone,tArr(tObjIs_IMAGE_COLOR_COLOR_ID)),0);
+		tFunc(tNone,tArr(tObjIs_IMAGE_COLOR_COLOR)),0);
 
    image_color_program->id = PROG_IMAGE_COLOR_COLOR_ID;
    
@@ -1830,8 +1837,6 @@ void exit_image_colors(void)
       for (i=0; (size_t)i<sizeof(html_color)/sizeof(html_color[0]); i++)
 	 free_string(html_color[i].pname);
    }
-   free_string(str_array);
-   free_string(str_string);
    free_string(str_r);
    free_string(str_g);
    free_string(str_b);
