@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 0ec82d97c4ee85ca86f4a7ed0298f40e2dcf40e0 $
 */
 
 /*
@@ -32,6 +31,7 @@
 #include "mapping.h"
 #include "pike_error.h"
 #include "operators.h"
+#include "pike_types.h"
 
 #include "image.h"
 #include "colortable.h"
@@ -122,7 +122,7 @@ static int parameter_int(struct svalue *map,struct pike_string *what,INT32 *p)
    struct svalue *v;
    v=low_mapping_string_lookup(map->u.mapping,what);
 
-   if (!v || v->type!=T_INT) return 0;
+   if (!v || TYPEOF(*v) != T_INT) return 0;
 
    *p=v->u.integer;
    return 1;
@@ -184,21 +184,21 @@ void img_bmp_encode(INT32 args)
    int rle=0;
 
    if (!args)
-      SIMPLE_TOO_FEW_ARGS_ERROR("Image.BMP.encode",1);
+      SIMPLE_TOO_FEW_ARGS_ERROR("encode",1);
 
-   if (sp[-args].type!=T_OBJECT ||
-       !(img=(struct image*)get_storage(o=sp[-args].u.object,image_program)))
-      SIMPLE_BAD_ARG_ERROR("Image.BMP.encode",1,"image object");
+   if (TYPEOF(sp[-args]) != T_OBJECT ||
+       !(img=get_storage(o=sp[-args].u.object,image_program)))
+      SIMPLE_BAD_ARG_ERROR("encode",1,"Image.Image");
 
    if (args>1) {
-      if (sp[1-args].type==T_OBJECT)
+      if (TYPEOF(sp[1-args]) == T_OBJECT)
       {
-	 if (!(nct=(struct neo_colortable*)
+	 if (!(nct=
 	       get_storage(oc=sp[1-args].u.object,image_colortable_program)))
-	    SIMPLE_BAD_ARG_ERROR("Image.BMP.encode",2,"colortable object");
+	    SIMPLE_BAD_ARG_ERROR("encode",2,"Image.Colortable");
 	 add_ref(oc);
       }
-      else if (sp[1-args].type==T_MAPPING)
+      else if (TYPEOF(sp[1-args]) == T_MAPPING)
       {
 	 struct svalue *v;
 
@@ -209,19 +209,19 @@ void img_bmp_encode(INT32 args)
 
 	 if (parameter(sp+1-args, colortable_string, &v))
 	 {
-	    if (v->type!=T_OBJECT  ||
-		!(nct=(struct neo_colortable*)
+	    if (TYPEOF(*v) != T_OBJECT  ||
+		!(nct=
 		  get_storage(oc=v->u.object,image_colortable_program)))
-	       SIMPLE_BAD_ARG_ERROR("Image.BMP.encode",2,"colortable object at index \"colortable\"\n");
+	       SIMPLE_BAD_ARG_ERROR("encode",2,"colortable object at index \"colortable\"");
 	    add_ref(oc);
 	 }
       } 
-      else if (sp[1-args].type==T_INT)
+      else if (TYPEOF(sp[1-args]) == T_INT)
       {
 	 bpp=sp[1-args].u.integer;
       }
       else
-	 SIMPLE_BAD_ARG_ERROR("Image.BMP.encode",2,"mapping|object|int\n");
+	 SIMPLE_BAD_ARG_ERROR("encode",2,"mapping|object|int");
    }
 
    if (bpp==0) {
@@ -239,7 +239,7 @@ void img_bmp_encode(INT32 args)
    {
       case 1:
 	 if (rle) 
-	    bad_arg_error("Image.BMP.encode", sp-args, args, 2, "mapping",
+	    bad_arg_error("encode", sp-args, args, 2, "mapping",
 			  sp+2-1-args,
 			  "run-length encoding can only be done on a "
 			  "palette-based image with 4 or 8 bits per pixel");
@@ -251,14 +251,14 @@ void img_bmp_encode(INT32 args)
 	    ref_push_object(o);
 	    push_int(1<<bpp);
 	    oc = clone_object(image_colortable_program, 2);
-	    if (!(nct=(struct neo_colortable*)
+	    if (!(nct=
 		  get_storage(oc, image_colortable_program))) {
 	      free_object(oc);
 	      Pike_error("Unexpected result from clone_object().\n");
 	    }
 	 }
 	 else if (image_colortable_size(nct)>(1<<bpp))
-	    bad_arg_error("Image.BMP.encode", sp-args, args, 2, "mapping",
+	    bad_arg_error("encode", sp-args, args, 2, "mapping",
 			  sp+2-1-args,
 			  "colortable must have at most %d colors "
 			  "(has %ld colors)\n",
@@ -267,7 +267,7 @@ void img_bmp_encode(INT32 args)
       case 24:
 	 break;
       default:
-	 bad_arg_error("Image.BMP.encode", sp-args, args, 2, "mapping",
+	 bad_arg_error("encode", sp-args, args, 2, "mapping",
 		       sp+2-1-args,
 		       "illegal bits per pixel: %d "
 		       "(1, 4, 8 and 24 are valid)\n",
@@ -279,10 +279,10 @@ void img_bmp_encode(INT32 args)
 
    apply(o,"mirrory",0);
    free_object(o);
-   if (sp[-1].type!=T_OBJECT ||
-       !(img=(struct image*)get_storage(o=sp[-1].u.object,image_program))) {
+   if (TYPEOF(sp[-1]) != T_OBJECT ||
+       !(img=get_storage(o=sp[-1].u.object,image_program))) {
       free_object(oc);
-      Pike_error("Image.BMP.encode: weird result from ->mirrory()\n");
+      Pike_error("Weird result from ->mirrory()\n");
    }
    if (nct) push_object(oc);
 
@@ -311,7 +311,7 @@ void img_bmp_encode(INT32 args)
    {
       /* FIXME: What if we don't have a nct? */
       ps=begin_shared_string((1<<bpp)*4);
-      MEMSET(ps->str,0,(1<<bpp)*4);
+      memset(ps->str,0,(1<<bpp)*4);
       image_colortable_write_bgrz(nct,(unsigned char *)ps->str);
       push_string(end_shared_string(ps));
       n++;
@@ -467,6 +467,9 @@ void img_bmp_encode(INT32 args)
 	 s++;
 	 if (!l--)
 	 {
+#ifdef USE_VALGRIND
+	    memset(c, 0, skip);
+#endif
 	    c+=skip;
 	    l=img->xsize-1;
 	 }
@@ -547,7 +550,7 @@ void i_img_bmp__decode(INT32 args,int header_only)
    int xsize=0,ysize=0,bpp=0,comp=0;
    struct image *img=NULL;
    struct neo_colortable *nct=NULL;
-   struct object *o;
+   struct object *o=NULL;
    rgb_group *d;
    int n=0,y,skip;
    ptrdiff_t i, j=0;
@@ -555,16 +558,14 @@ void i_img_bmp__decode(INT32 args,int header_only)
    int quality=50; /* for JPEG decoding */
 
    if (args<1)
-      SIMPLE_TOO_FEW_ARGS_ERROR("Image.BMP.decode",1);
+      SIMPLE_TOO_FEW_ARGS_ERROR("decode",1);
 
-   if (sp[-args].type!=T_STRING)
-      SIMPLE_BAD_ARG_ERROR("Image.BMP.decode",1,"string");
-   if (sp[-args].u.string->size_shift)
-      SIMPLE_BAD_ARG_ERROR("Image.BMP.decode",1,"8 bit string");
+   if (TYPEOF(sp[-args]) != T_STRING || sp[-args].u.string->size_shift)
+      SIMPLE_BAD_ARG_ERROR("decode",1,"string(8bit)");
 
    if (args>1) {
-      if (sp[1-args].type!=T_MAPPING)
-	 SIMPLE_BAD_ARG_ERROR("Image.BMP.decode",2,"mapping");
+      if (TYPEOF(sp[1-args]) != T_MAPPING)
+	 SIMPLE_BAD_ARG_ERROR("decode",2,"mapping");
       else
       {
 	 struct pike_string *qs;
@@ -601,7 +602,7 @@ void i_img_bmp__decode(INT32 args,int header_only)
 	 n++;
 
 	 push_text("ysize");
-	 push_int(ysize=int_from_32bit(s+14+4*2));
+	 push_int(abs(ysize=int_from_32bit(s+14+4*2)));
 	 n++;
 
 	 push_text("target_planes");
@@ -656,7 +657,7 @@ void i_img_bmp__decode(INT32 args,int header_only)
 	 n++;
 
 	 push_text("ysize");
-	 push_int(ysize=int_from_16bit(s+14+6));
+	 push_int(abs(ysize=int_from_16bit(s+14+6)));
 	 n++;
 
 	 push_text("target_planes");
@@ -682,7 +683,7 @@ void i_img_bmp__decode(INT32 args,int header_only)
 	       int_from_32bit(s+14));
    }
 
-   push_text("type");
+   ref_push_string(literal_type_string);
    push_text("image/x-MS-bmp");
    n++;
 
@@ -698,11 +699,11 @@ void i_img_bmp__decode(INT32 args,int header_only)
    {
       int i=int_from_32bit(os+10);
 
-      push_text("image");
-
-      if (olen-i<0)
+      if (i < 0 || olen < i)
 	 Pike_error("Image.BMP.decode: unexpected EOF in JFIF data\n");
 
+      push_text("image");
+
       push_text("Image.JPEG.decode");
       SAFE_APPLY_MASTER("resolv_or_error",1);
 
@@ -737,7 +738,7 @@ void i_img_bmp__decode(INT32 args,int header_only)
 	 push_string(make_shared_binary_string((char *)s,(4<<bpp)));
 	 push_int(2);
 	 push_object(o=clone_object(image_colortable_program,2));
-	 nct=(struct neo_colortable*)get_storage(o,image_colortable_program);
+	 nct=get_storage(o,image_colortable_program);
 
 	 s+=(4<<bpp);
 	 len-=(4<<bpp);
@@ -750,7 +751,7 @@ void i_img_bmp__decode(INT32 args,int header_only)
 	 push_string(make_shared_binary_string((char *)s,(3<<bpp)));
 	 push_int(1);
 	 push_object(o=clone_object(image_colortable_program,2));
-	 nct=(struct neo_colortable*)get_storage(o,image_colortable_program);
+	 nct=get_storage(o,image_colortable_program);
 
 	 s+=(3<<bpp);
 	 len-=(3<<bpp);
@@ -762,19 +763,42 @@ void i_img_bmp__decode(INT32 args,int header_only)
    push_text("image");
 
    push_int(xsize);
-   push_int(ysize);
+   push_int(abs(ysize));
    push_object(o=clone_object(image_program,2));
-   img=(struct image*)get_storage(o,image_program);
+   img=get_storage(o,image_program);
    n++;
 
-   if (int_from_32bit(os+10)) 
-   {
-      s=os+int_from_32bit(os+10);
-      len=olen-int_from_32bit(os+10);
-   }
+   j = int_from_32bit(os+10);
+
+   if (j < 0 || j > olen)
+       Pike_error("Image.BMP.decode: unexpected EOF\n");
+
+   s=os+j;
+   len=olen-j;
 
    if (len>0) switch (bpp)
    {
+      case 32:
+	 if (comp)
+	    Pike_error("Image.BMP.decode: can't handle compressed 32bpp BMP\n");
+
+	 j=len;
+	 y=img->ysize;
+	 while (j>2 && y--)
+	 {
+	    d = img->img+img->xsize*y;
+	    i = img->xsize;
+	    if (i*4>j) i=j/4;
+	    j -= i*4;
+	    while (i--)
+	    {
+	       d->b=*(s++);
+	       d->g=*(s++);
+	       d->r=*(s++);
+	       d++;s++;
+	    }
+	 }
+	 break;
       case 24:
 	 if (comp)
 	    Pike_error("Image.BMP.decode: can't handle compressed 24bpp BMP\n");
@@ -1059,8 +1083,20 @@ void i_img_bmp__decode(INT32 args,int header_only)
    }
 
 final:
-
    f_aggregate_mapping(n*2);
+   if( (ysize < 0) && o )
+   {
+       /* This is more expensive than actually correctly decoding the
+          image in the first place, but for now keep it like this
+          since it minimizes the needed changes. */
+       push_text("image");
+       apply(o,"mirrory",0);
+       mapping_string_insert( Pike_sp[-3].u.mapping,
+                              Pike_sp[-2].u.string,
+                              Pike_sp-1 );
+       Pike_sp--;
+       pop_stack();
+   }
    stack_swap();
    pop_stack();
 }
@@ -1090,7 +1126,7 @@ void init_image_bmp(void)
    colortable_string = make_shared_string("colortable");
 
    ADD_FUNCTION("encode",img_bmp_encode,
-		tFunc(tObj tOr4(tVoid,tObj,tInt,tMapping),tStr), 0);
+		tFunc(tObj tOr4(tVoid,tObj,tInt,tMapping),tStr8), 0);
    ADD_FUNCTION("_decode",img_bmp__decode,
 		tFunc(tStr tOr(tVoid,tMapping),tMapping),0);
    ADD_FUNCTION("decode",img_bmp_decode,
