@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 3fa34a044dd2497301fc9c4983bc0707482d8f0c $
 */
 
 #include "global.h"
@@ -21,6 +20,7 @@
 #include "stralloc.h"
 #include "builtin_functions.h"
 #include "module_support.h"
+#include "pike_types.h"
 
 #include "image.h"
 
@@ -35,11 +35,11 @@ extern struct program *image_program;
 **! module Image
 **! submodule PVR
 **!
-**! 	Handle encoding and decoding of PVR images.
-**! 
-**! 	PVR is the texture format of the NEC PowerVR system
-**! 	It is a rather simple, uncompressed, truecolor
-**!     format. 
+**! Handle encoding and decoding of PVR images.
+**!
+**! PVR is the texture format of the NEC PowerVR system
+**! It is a rather simple, uncompressed, truecolor
+**! format.
 */
 
 /*
@@ -55,14 +55,14 @@ extern struct program *image_program;
 **!	<pre>
 **!        "image":object            - image object    \- not decode_header
 **!	   "alpha":object            - decoded alpha   /
-**!	   
+**!
 **!	   "type":"image/x-pvr"      - image type
 **!	   "xsize":int               - horisontal size in pixels
 **!	   "ysize":int               - vertical size in pixels
 **!	   "attr":int		     - texture attributes
 **!	   "global_index":int	     - global index (if present)
 **!	</pre>
-**!	  
+**!
 **!
 **! method string encode(object image)
 **! method string encode(object image,mapping options)
@@ -583,21 +583,21 @@ void image_pvr_f_encode(INT32 args)
   unsigned char *dst;
   struct gla_state *gla_st = NULL;
 
-  get_all_args("Image.PVR.encode", args, (args>1 && !UNSAFE_IS_ZERO(&sp[1-args])?
-					  "%o%m":"%o"), &imgo, &optm);
+  get_all_args("encode", args, (args>1 && !UNSAFE_IS_ZERO(&sp[1-args])?
+                                "%o%m":"%o"), &imgo, &optm);
 
-  if((img=(struct image*)get_storage(imgo, image_program))==NULL)
+  if((img=get_storage(imgo, image_program))==NULL)
     Pike_error("Image.PVR.encode: illegal argument 1\n");
 
   if(optm != NULL) {
     struct svalue *s;
     if((s = simple_mapping_string_lookup(optm, "alpha"))!=NULL && !UNSAFE_IS_ZERO(s))
-      if(s->type != T_OBJECT ||
-	 (alpha=(struct image*)get_storage(s->u.object, image_program))==NULL)
+      if(TYPEOF(*s) != T_OBJECT ||
+	 (alpha=get_storage(s->u.object, image_program))==NULL)
 	Pike_error("Image.PVR.encode: option (arg 2) \"alpha\" has illegal type\n");
     if((s = simple_mapping_string_lookup(optm, "global_index"))!=NULL &&
        !IS_UNDEFINED(s)) {
-      if(s->type == T_INT) {
+      if(TYPEOF(*s) == T_INT) {
 	gbix = s->u.integer;
 	has_gbix=1;
       }
@@ -727,7 +727,7 @@ void image_pvr_f_encode(INT32 args)
 }
 
 static void pvr_decode_rect(INT32 attr, unsigned char *src, rgb_group *dst,
-			    INT32 stride, unsigned int h, unsigned int w)
+			    INT32 UNUSED(stride), unsigned int h, unsigned int w)
 {
   INT32 cnt = h * w;
   switch(attr&0xff) {
@@ -909,7 +909,7 @@ static void pvr_decode_twiddled(INT32 attr, unsigned char *s, rgb_group *dst,
 }
 
 static void pvr_decode_alpha_rect(INT32 attr, unsigned char *src,
-				  rgb_group *dst, INT32 stride,
+				  rgb_group *dst, INT32 UNUSED(stride),
 				  unsigned int h, unsigned int w)
 {
   INT32 cnt = h * w;
@@ -1037,7 +1037,7 @@ void img_pvr_decode(INT32 args,int header_only)
    INT32 attr;
    unsigned int h, w, x;
 
-   get_all_args("Image.PVR._decode", args, "%S", &str);
+   get_all_args("_decode", args, "%S", &str);
    s = (unsigned char *)str->str;
    len = str->len;
    pop_n_elems(args-1);
@@ -1064,7 +1064,7 @@ void img_pvr_decode(INT32 args,int header_only)
      len = l+8;
    }
 
-   push_text("type");
+   ref_push_string(literal_type_string);
    push_text("image/x-pvr");
    n++;
 
@@ -1166,7 +1166,7 @@ void img_pvr_decode(INT32 args,int header_only)
      push_int(w);
      push_int(h);
      o=clone_object(image_program,2);
-     img=(struct image*)get_storage(o,image_program);
+     img=get_storage(o,image_program);
      push_object(o);
      n++;
 
@@ -1189,7 +1189,7 @@ void img_pvr_decode(INT32 args,int header_only)
        push_int(w);
        push_int(h);
        o=clone_object(image_program,2);
-       img=(struct image*)get_storage(o,image_program);
+       img=get_storage(o,image_program);
        push_object(o);
        n++;
        
