@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 3038cf862a5be2394555d9a47dcff491a5771ac7 $
 */
 
 /*
@@ -96,7 +95,6 @@ static void chrono(char *x)
 **!	experimental status; may not be exact the same
 **!	output in later versions
 */
-static INLINE int sq(int a) { return a*a; }
 static INLINE int my_abs(int a) { return (a<0)?-a:a; }
 
 static void _image_orient(struct image *source,
@@ -112,7 +110,7 @@ static void _image_orient(struct image *source,
       push_int(source->xsize);
       push_int(source->ysize);
       o[i]=clone_object(image_program,2);
-      img[i]=(struct image*)get_storage(o[i],image_program);
+      img[i]=get_storage(o[i],image_program);
       push_object(o[i]);
    }
 
@@ -136,11 +134,6 @@ CHRONO("start");
   (COLORTYPE) \
      my_abs( s[(x+xd)+(y+yd)*xz].CO - s[(x-xd)+(y-yd)*xz].CO )
 
-	    /*
-	      sqrt( ( sq( s[(x-xd)+(y-yd)*xz].CO - s[x+y*xz].CO ) + \
-	      sq( s[(x+xd)+(y+yd)*xz].CO - s[x+y*xz].CO ) ) / 2.0 )
-	    */
-
 	    FOOBAR(r);
 	    FOOBAR(g);
 	    FOOBAR(b);
@@ -167,13 +160,12 @@ void image_orient(INT32 args)
 
   if (args)
   {
-    if (sp[-args].type==T_INT) 
+    if (TYPEOF(sp[-args]) == T_INT)
       mag=sp[-args].u.integer;
-    else if (sp[-args].type==T_FLOAT)
+    else if (TYPEOF(sp[-args]) == T_FLOAT)
       mag=sp[-args].u.float_number;
     else {
-      bad_arg_error("image->orient\\n",sp-args,args,1,"",sp+1-1-args,
-		"Bad argument 1 to image->orient\n()\n");
+      SIMPLE_BAD_ARG_ERROR("orient",1,"int|float");
       /* Not reached, but keeps the compiler happy. */
       mag = 0.0;
     }
@@ -185,16 +177,15 @@ void image_orient(INT32 args)
 
   if (args>1)
   {
-    if (sp[1-args].type!=T_ARRAY) 
-      bad_arg_error("image->orient\\n",sp-args,args,2,"",sp+2-1-args,
-		"Bad argument 2 to image->orient\n()\n");
+    if (TYPEOF(sp[1-args]) != T_ARRAY)
+      SIMPLE_BAD_ARG_ERROR("orient",2,"array");
     if (sp[1-args].u.array->size!=4)
-      Pike_error("The array given as argument 2 to image->orient do not have size 4\n");
+      Pike_error("The array given as argument 2 to orient do not have size 4\n");
     for(i=0; i<4; i++)
-      if ((sp[1-args].u.array->item[i].type!=T_OBJECT) ||
+      if ((TYPEOF(sp[1-args].u.array->item[i]) != T_OBJECT) ||
 	  (!(sp[1-args].u.array->item[i].u.object)) ||
 	  (sp[1-args].u.array->item[i].u.object->prog!=image_program))
-	Pike_error("The array given as argument 2 to image->orient do not contain images\n");
+	Pike_error("The array given as argument 2 to orient do not contain images\n");
     img1=(struct image*)sp[1-args].u.array->item[0].u.object->storage;
 
     w=this->xsize;
@@ -205,15 +196,15 @@ void image_orient(INT32 args)
       img1=(struct image*)sp[1-args].u.array->item[i].u.object->storage;
       if ((img1->xsize!=w)||
 	  (img1->ysize!=h))
-	Pike_error("The images in the array given as argument 2 to image->orient have different sizes\n");
+	Pike_error("The images in the array given as argument 2 to orient have different sizes\n");
     }
     for(i=0; i<4; i++) 
-      img[i]=(struct image*)sp[1-args].u.array->item[i].u.object->storage;
+        img[i]=get_storage(sp[1-args].u.array->item[i].u.object,image_program);
     pop_n_elems(args);
     push_int(this->xsize);
     push_int(this->ysize);
     o[4]=clone_object(image_program,2);
-    img[4]=(struct image*)get_storage(o[4],image_program);
+    img[4]=get_storage(o[4],image_program);
     push_object(o[4]);
     w=1;
   }
@@ -249,8 +240,11 @@ CHRONO("begin hsv...");
 	}
 	else z=0,w=0;
      else {
-	z = -DOUBLE_TO_INT(-32*(h/j)+(j>0)*128+128);
-	w = my_abs(DOUBLE_TO_INT(j));
+	if (j) {
+	    z = -DOUBLE_TO_INT(-32*(h/j)+(j>0)*128+128);
+	    w = my_abs(DOUBLE_TO_INT(j));
+	}
+	else z=0,w=0;
      }
 
      d->r=(COLORTYPE)z;
