@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 2a438cfa5996355d77dd5b8446161cf2c23ea248 $
 */
 
 /*
@@ -68,7 +67,6 @@
 #include "object.h"
 #include "stralloc.h"
 #include "interpret.h"
-#include "port.h"
 #include "pike_error.h"
 #include "threads.h"
 #include "program.h"
@@ -77,6 +75,7 @@
 #include "fd_control.h"
 #include "mapping.h"
 #include "bignum.h"
+#include "signal_handler.h"
 #include "module_support.h"
 
 /* System includes */
@@ -105,9 +104,10 @@
 /*! @module Mysql
  *!
  *! This module enables access to the Mysql database from within Pike.
- *! Use @[Sql.Sql] for general database access.
  *!
- *! Mysql is available from @url{http://www.mysql.com@}.
+ *! @note
+ *!  You typically don't want to access this module directly, instead
+ *!  use @[Sql.Sql()] with an "mysql://" URL.
  *!
  *! @seealso
  *!  @[Mysql.mysql], @[Mysql.mysql_result], @[Sql.Sql]
@@ -115,18 +115,18 @@
 
 /*! @class mysql
  *!
- *! Interface to the Mysql database.
+ *! Low level interface to the Mysql database.
  *!
  *! This class enables access to the Mysql database from within Pike.
  *!
- *! Mysql is available from @url{http://www.mysql.com@}.
- *!
  *! @seealso
  *!   @[Mysql.mysql_result], @[Sql.Sql]
  */
 
+struct program * mysql_error_program = NULL;
 struct program *mysql_program = NULL;
 
+#ifndef HAVE_MYSQL_REAL_CONNECT
 #if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
 #ifdef HAVE_MYSQL_PORT
 extern unsigned int mysql_port;
@@ -147,6 +147,7 @@ static MUTEX_T stupid_port_lock;
 #define STUPID_PORT_DESTROY()
 #endif /* _REENTRANT */
 #endif /* HAVE_MYSQL_PORT */
+#endif /* !HAVE_MYSQL_REAL_CONNECT */
 
 #ifdef _REENTRANT
 #define MYSQL_LOCK		(&(PIKE_MYSQL->lock))
@@ -165,16 +166,16 @@ static MUTEX_T stupid_port_lock;
 #define PIKE_MYSQL_FLAG_TYPED_RESULT	2
 
 #define CHECK_8BIT_NONBINARY_STRING(FUNC, ARG) do {			\
-    if (sp[ARG-1-args].type != T_STRING ||				\
+    if (TYPEOF(sp[ARG-1-args]) != T_STRING ||				\
 	sp[ARG-1-args].u.string->size_shift ||				\
         string_has_null(sp[ARG-1-args].u.string))			\
-      SIMPLE_BAD_ARG_ERROR (FUNC, ARG, "string (nonbinary 8bit)");	\
+      SIMPLE_BAD_ARG_ERROR (FUNC, ARG, "string(1..255)");               \
   } while (0)
 
 #define CHECK_8BIT_STRING(FUNC, ARG) do {				\
-    if (sp[ARG-1-args].type != T_STRING ||				\
+    if (TYPEOF(sp[ARG-1-args]) != T_STRING ||				\
 	sp[ARG-1-args].u.string->size_shift)				\
-      SIMPLE_BAD_ARG_ERROR (FUNC, ARG, "string (8bit)");	\
+      SIMPLE_BAD_ARG_ERROR (FUNC, ARG, "string(8bit)");                 \
   } while (0)
 
 
@@ -186,16 +187,27 @@ static MUTEX_T stupid_port_lock;
  * State maintenance
  */
 
-static void init_mysql_struct(struct object *o)
+static void init_mysql_struct(struct object *UNUSED(o))
 {
   MEMSET(PIKE_MYSQL, 0, sizeof(struct precompiled_mysql));
   INIT_MYSQL_LOCK();
   PIKE_MYSQL->mysql = mysql_init(NULL);
   if (!PIKE_MYSQL->mysql)
     Pike_error ("Out of memory when initializing mysql connection.\n");
+#ifdef SIGPIPE
+  /* Restore lost signal handler.
+   *
+   * NB: mysql_init() in MariaDB zaps the SIGPIPE handler if
+   *     mysql->client_flag hasn't set CLIENT_IGNORE_SIGPIPE.
+   *     Unfortunately, it also calls bzero(3BSD) on the struct
+   *     before checking the flag, so it in effect unconditionally
+   *     always zaps the SIGPIPE handler.
+   */
+  restore_signal_handler(SIGPIPE);
+#endif
 }
 
-static void exit_mysql_struct(struct object *o)
+static void exit_mysql_struct(struct object *UNUSED(o))
 {
   MYSQL *mysql = PIKE_MYSQL->mysql;
 
@@ -252,27 +264,27 @@ void pike_mysql_set_ssl(struct mapping *options)
 
 #ifdef HAVE_MYSQL_SSL
     if ((val = simple_mapping_string_lookup(options, "ssl_key")) &&
-	(val->type == T_STRING) &&
+	(TYPEOF(*val) == T_STRING) &&
 	(!val->u.string->size_shift))
       ssl_key = val->u.string->str;
 
     if ((val = simple_mapping_string_lookup(options, "ssl_cert")) &&
-	(val->type == T_STRING) &&
+	(TYPEOF(*val) == T_STRING) &&
 	(!val->u.string->size_shift))
       ssl_cert = val->u.string->str;
 
     if ((val = simple_mapping_string_lookup(options, "ssl_ca")) &&
-	(val->type == T_STRING) &&
+	(TYPEOF(*val) == T_STRING) &&
 	(!val->u.string->size_shift))
       ssl_ca = val->u.string->str;
 
     if ((val = simple_mapping_string_lookup(options, "ssl_capath")) &&
-	(val->type == T_STRING) &&
+	(TYPEOF(*val) == T_STRING) &&
 	(!val->u.string->size_shift))
       ssl_capath = val->u.string->str;
 
     if ((val = simple_mapping_string_lookup(options, "ssl_cipher")) &&
-	(val->type == T_STRING) &&
+	(TYPEOF(*val) == T_STRING) &&
 	(!val->u.string->size_shift))
       ssl_cipher = val->u.string->str;
 
@@ -302,34 +314,34 @@ static void pike_mysql_set_options(struct mapping *options)
   }
 #ifdef HAVE_MYSQL_READ_DEFAULT_FILE
   if ((val = simple_mapping_string_lookup(options, "mysql_config_file")) &&
-      (val->type == T_STRING) && (!val->u.string->size_shift)) {
+      (TYPEOF(*val) == T_STRING) && (!val->u.string->size_shift)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_READ_DEFAULT_FILE,
 		  val->u.string->str);
   }
 #endif /* MYSQL_READ_DEFAULT_FILE */
 #ifdef HAVE_MYSQL_READ_DEFAULT_GROUP
   if ((val = simple_mapping_string_lookup(options, "mysql_group")) &&
-      (val->type == T_STRING) && (!val->u.string->size_shift)) {
+      (TYPEOF(*val) == T_STRING) && (!val->u.string->size_shift)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_READ_DEFAULT_GROUP,
 		  val->u.string->str);
   }
 #endif /* MYSQL_READ_DEFAULT_GROUP */
 #ifdef HAVE_MYSQL_INIT_COMMAND
   if ((val = simple_mapping_string_lookup(options, "init_command")) &&
-      (val->type == T_STRING) && (!val->u.string->size_shift)) {
+      (TYPEOF(*val) == T_STRING) && (!val->u.string->size_shift)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_INIT_COMMAND,
 		  val->u.string->str);
   }
 #endif /* MYSQL_INIT_COMMAND */
 #ifdef HAVE_MYSQL_OPT_NAMED_PIPE
   if ((val = simple_mapping_string_lookup(options, "mysql_named_pipe")) &&
-      (val->type == T_INT) && (val->u.integer)) {
+      (TYPEOF(*val) == T_INT) && (val->u.integer)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_OPT_NAMED_PIPE, NULL);
   }
 #endif /* MYSQL_OPT_NAMED_PIPE */
 #ifdef HAVE_MYSQL_OPT_CONNECT_TIMEOUT
   if ((val = simple_mapping_string_lookup(options, "timeout")) &&
-      (val->type == T_INT)) {
+      (TYPEOF(*val) == T_INT)) {
     unsigned int timeout = (unsigned int)val->u.integer;
     mysql_options(PIKE_MYSQL->mysql, MYSQL_OPT_CONNECT_TIMEOUT,
 		  (char *)&timeout);
@@ -337,13 +349,13 @@ static void pike_mysql_set_options(struct mapping *options)
 #endif /* MYSQL_OPT_CONNECT_TIMEOUT */
 #ifdef HAVE_MYSQL_OPT_COMPRESS
   if ((val = simple_mapping_string_lookup(options, "compress")) &&
-      (val->type == T_INT) && (val->u.integer)) {
+      (TYPEOF(*val) == T_INT) && (val->u.integer)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_OPT_COMPRESS, NULL);
   }
 #endif /* MYSQL_OPT_COMPRESS */
 #ifdef HAVE_MYSQL_OPT_LOCAL_INFILE
   if ((val = simple_mapping_string_lookup(options, "mysql_local_infile")) &&
-      (val->type == T_INT)) {
+      (TYPEOF(*val) == T_INT)) {
     unsigned int allowed = (unsigned int)val->u.integer;
     mysql_options(PIKE_MYSQL->mysql, MYSQL_OPT_LOCAL_INFILE,
 		  (char *)&allowed);
@@ -356,7 +368,7 @@ static void pike_mysql_set_options(struct mapping *options)
 #endif /* MYSQL_OPT_LOCAL_INFILE */
 #ifdef HAVE_MYSQL_SET_CHARSET_DIR
   if ((val = simple_mapping_string_lookup(options, "mysql_charset_dir")) &&
-      (val->type == T_STRING) && (!val->u.string->size_shift)) {
+      (TYPEOF(*val) == T_STRING) && (!val->u.string->size_shift)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_SET_CHARSET_DIR,
 		  val->u.string->str);
   }
@@ -364,7 +376,7 @@ static void pike_mysql_set_options(struct mapping *options)
 #endif /* HAVE_MYSQL_OPTIONS */
 
   if ((val = simple_mapping_string_lookup(options, "mysql_charset_name")) &&
-      (val->type == T_STRING) && (!val->u.string->size_shift) &&
+      (TYPEOF(*val) == T_STRING) && (!val->u.string->size_shift) &&
       !string_has_null (val->u.string)) {
 #if defined (HAVE_MYSQL_OPTIONS) && defined (HAVE_MYSQL_SET_CHARSET_NAME)
     mysql_options (PIKE_MYSQL->mysql, MYSQL_SET_CHARSET_NAME,
@@ -372,7 +384,7 @@ static void pike_mysql_set_options(struct mapping *options)
 #endif
 
     if (PIKE_MYSQL->conn_charset)
-      free (PIKE_MYSQL->conn_charset);
+      free_string (PIKE_MYSQL->conn_charset);
     copy_shared_string (PIKE_MYSQL->conn_charset, val->u.string);
   }
 }
@@ -407,9 +419,9 @@ static void connection_set_charset (struct pike_string *charset)
    * charset. We emulate it by setting the charset ourselves. Note
    * that this doesn't work with mysql_real_escape_string, but that
    * function isn't used. */
-  push_constant_text ("SET NAMES '");
+  push_text ("SET NAMES '");
   ref_push_string (charset);	/* FIXME: Quote? */
-  push_constant_text ("'");
+  push_text ("'");
   f_add (3);
   low_query (1, "set_charset", PIKE_MYSQL_FLAG_STORE_RESULT);
   pop_stack();
@@ -459,14 +471,17 @@ static void pike_mysql_reconnect (int reconnect)
   if (PIKE_MYSQL->options &&
       (val = simple_mapping_string_lookup(PIKE_MYSQL->options,
 					  "connect_options")) &&
-      (val->type == T_INT) && (val->u.integer)) {
+      (TYPEOF(*val) == T_INT) && (val->u.integer)) {
     options = (unsigned int)val->u.integer;
   }
 
 #if defined(HAVE_MYSQL_OPTIONS) && defined(HAVE_MYSQL_OPT_RECONNECT)
   {
-    /* Enable the automatic reconnect. */
-    my_bool reconnectp = 1;
+    /* Disable the automatic reconnect. */
+    my_bool reconnectp = 0;
+    if (PIKE_MYSQL->options &&
+      (val = simple_mapping_string_lookup(PIKE_MYSQL->options, "reconnect")))
+      if (!SAFE_IS_ZERO(val)) reconnectp = 1;
     mysql_options(mysql, MYSQL_OPT_RECONNECT, &reconnectp);
   }
 #endif
@@ -480,13 +495,13 @@ static void pike_mysql_reconnect (int reconnect)
 
   MYSQL_ALLOW();
 
-#if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
-  STUPID_PORT_LOCK();
-#endif /* HAVE_MYSQL_PORT || HAVE_MYSQL_UNIX_PORT */
 #ifdef HAVE_MYSQL_REAL_CONNECT
   socket = mysql_real_connect(mysql, host, user, password,
                               NULL, port, portptr, options);
 #else
+#if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
+  STUPID_PORT_LOCK();
+#endif /* HAVE_MYSQL_PORT || HAVE_MYSQL_UNIX_PORT */
 
 #ifdef HAVE_MYSQL_PORT
   if (port) {
@@ -514,10 +529,10 @@ static void pike_mysql_reconnect (int reconnect)
   }
 #endif /* HAVE_MYSQL_UNIX_PORT */
 
-#endif /* HAVE_MYSQL_REAL_CONNECT */
 #if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
   STUPID_PORT_UNLOCK();
 #endif /* HAVE_MYSQL_PORT || MAVE_MYSQL_UNIX_PORT*/
+#endif /* HAVE_MYSQL_REAL_CONNECT */
 
   MYSQL_DISALLOW();
 
@@ -525,6 +540,7 @@ static void pike_mysql_reconnect (int reconnect)
   /* Note: In Mysql 3.22 the reconnect flag is always set by
    *       mysql_real_connect(), so we need to reset it here.
    */
+  mysql->reconnect = 0;
   if (PIKE_MYSQL->options &&
       (val = simple_mapping_string_lookup(PIKE_MYSQL->options, "reconnect"))) {
     my_bool reconnectp = 0;
@@ -690,32 +706,32 @@ static void f_create(INT32 args)
 #endif /* HAVE_MYSQL_OPTIONS && HAVE_MYSQL_OPT_LOCAL_INFILE */
 
   if (args >= 1) {
-    CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->create", 1);
+    CHECK_8BIT_NONBINARY_STRING ("create", 1);
     if (sp[-args].u.string->len) {
       add_ref(PIKE_MYSQL->host = sp[-args].u.string);
     }
 
     if (args >= 2) {
-      CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->create", 2);
+      CHECK_8BIT_NONBINARY_STRING ("create", 2);
       if (sp[1-args].u.string->len) {
 	add_ref(PIKE_MYSQL->database = sp[1-args].u.string);
       }
       
       if (args >= 3) {
-	CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->create", 3);
+	CHECK_8BIT_NONBINARY_STRING ("create", 3);
 	if (sp[2-args].u.string->len) {
 	  add_ref(PIKE_MYSQL->user = sp[2-args].u.string);
 	}
 	
 	if (args >= 4) {
-	  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->create", 4);
+	  CHECK_8BIT_NONBINARY_STRING ("create", 4);
 	  if (sp[3-args].u.string->len) {
 	    add_ref(PIKE_MYSQL->password = sp[3-args].u.string);
 	  }
 	  if (args >= 5) {
-	    if (sp[4-args].type != T_MAPPING) {
+	    if (TYPEOF(sp[4-args]) != T_MAPPING){
 	      if (!UNSAFE_IS_ZERO(sp + 4 - args)) {
-		SIMPLE_BAD_ARG_ERROR ("Mysql.mysql->create", 5, "mapping(string:mixed)");
+		SIMPLE_BAD_ARG_ERROR ("create", 5, "mapping(string:mixed)");
 	      }
 	    } else {
 	      add_ref(PIKE_MYSQL->options = sp[4-args].u.mapping);
@@ -756,8 +772,8 @@ static void mysql__sprintf(INT32 args)
 
   if(args < 1)
     SIMPLE_TOO_FEW_ARGS_ERROR("_sprintf",2);
-  if(sp[-args].type!=T_INT)
-    SIMPLE_BAD_ARG_ERROR("_sprintf",0,"integer");
+  if(TYPEOF(sp[-args]) != T_INT)
+    SIMPLE_BAD_ARG_ERROR("_sprintf",0,"int");
 
   type = sp[-args].u.integer;
   pop_n_elems(args);
@@ -789,7 +805,7 @@ static void mysql__sprintf(INT32 args)
 	f_sprintf(3);
       }
       else
-	push_constant_text ("mysql()");
+	push_text ("mysql()");
 
       return;
     }
@@ -801,8 +817,7 @@ static void mysql__sprintf(INT32 args)
       return;
     }
   }
-  push_int( 0 );
-  Pike_sp[-1].subtype = 1;
+  push_undefined();
 }
 
 /*! @decl int is_open()
@@ -955,6 +970,57 @@ static void f_error(INT32 args)
   }
 }
 
+/*! @decl int errno()
+ *!
+ *! Returns an error code describing the last error from the Mysql-server.
+ *!
+ *! Returns @expr{0@} (zero) if there was no error.
+ */
+static void f_errno(INT32 args)
+{
+  MYSQL *mysql;
+  unsigned int errnum;
+
+  mysql = PIKE_MYSQL->mysql;
+
+  MYSQL_ALLOW();
+
+  errnum = mysql_errno(mysql);
+
+  MYSQL_DISALLOW();
+
+  pop_n_elems(args);
+
+  push_int(errnum);
+}
+
+/*! @decl string sqlstate()
+ *!
+ *! Returns the SQLSTATE error code describing the last error.
+ *!
+ *! The value @expr{"000000"@} means 'no error'. The SQLSTATE error codes are
+ *! described in ANSI SQL.
+ */
+#ifdef HAVE_MYSQL_SQLSTATE
+static void f_sqlstate(INT32 args)
+{
+  MYSQL *mysql;
+  const char *error_msg;
+
+  mysql = PIKE_MYSQL->mysql;
+
+  MYSQL_ALLOW();
+
+  error_msg = mysql_sqlstate(mysql);
+
+  MYSQL_DISALLOW();
+
+  pop_n_elems(args);
+
+  push_text(error_msg);
+}
+#endif
+
 /*! @decl void select_db(string database)
  *!
  *! Select database.
@@ -972,9 +1038,9 @@ static void f_select_db(INT32 args)
   int tmp = -1;
 
   if (!args) {
-    SIMPLE_TOO_FEW_ARGS_ERROR ("Mysql.mysql->select_db", 1);
+    SIMPLE_TOO_FEW_ARGS_ERROR ("select_db", 1);
   }
-  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->select_db", 1);
+  CHECK_8BIT_NONBINARY_STRING ("select_db", 1);
 
   database = sp[-args].u.string->str;
 
@@ -1110,8 +1176,8 @@ static void low_query(INT32 args, char *name, int flags)
     }
 
     /* Set the result. */
-    if ((!(res = (struct precompiled_mysql_result *)
-	   get_storage(o, mysql_result_program))) || res->result) {
+    if ((!(res = get_storage(o, mysql_result_program))) ||
+        res->result) {
       mysql_free_result(result);
       Pike_error("%s(): Bad mysql result object!\n", name);
     }
@@ -1159,9 +1225,9 @@ static void f_create_db(INT32 args)
   int tmp = -1;
 
   if (!args) {
-    SIMPLE_TOO_FEW_ARGS_ERROR ("Mysql.mysql->create_db", 1);
+    SIMPLE_TOO_FEW_ARGS_ERROR ("create_db", 1);
   }
-  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->create_db", 1);
+  CHECK_8BIT_NONBINARY_STRING ("create_db", 1);
   if (sp[-args].u.string->len > 127) {
     if (sp[-args].u.string->len < 1024) {
       Pike_error("Database name \"%s\" is too long (max 127 characters)\n",
@@ -1204,9 +1270,9 @@ static void f_drop_db(INT32 args)
   int tmp = -1;
 
   if (!args) {
-    SIMPLE_TOO_FEW_ARGS_ERROR ("Mysql.mysql->drop_db", 1);
+    SIMPLE_TOO_FEW_ARGS_ERROR ("drop_db", 1);
   }
-  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->drop_db", 1);
+  CHECK_8BIT_NONBINARY_STRING ("drop_db", 1);
   if (sp[-args].u.string->len > 127) {
     if (sp[-args].u.string->len < 1024) {
       Pike_error("Database name \"%s\" is too long (max 127 characters)\n",
@@ -1305,7 +1371,7 @@ static void f_reload(INT32 args)
  *! This function returns some server statistics.
  *!
  *! @seealso
- *!   @[server_info()], @[host_info()], @[protocol_info()]
+ *!   @[server_info()], @[host_info()], @[protocol_info()], @[info()]
  */
 static void f_statistics(INT32 args)
 {
@@ -1328,7 +1394,7 @@ static void f_statistics(INT32 args)
  *! Get the version number of the Mysql-server.
  *!
  *! @seealso
- *!   @[statistics()], @[host_info()], @[protocol_info()]
+ *!   @[statistics()], @[host_info()], @[protocol_info()], @[info()]
  */
 static void f_server_info(INT32 args)
 {
@@ -1354,7 +1420,7 @@ static void f_server_info(INT32 args)
  *! Get information about the Mysql-server connection
  *!
  *! @seealso
- *!   @[statistics()], @[server_info()], @[protocol_info()]
+ *!   @[statistics()], @[server_info()], @[protocol_info()], @[info()]
  */
 static void f_host_info(INT32 args)
 {
@@ -1374,6 +1440,37 @@ static void f_host_info(INT32 args)
   push_text(info);
 }
 
+/*! @decl string info()
+ *!
+ *! Get information about the most recently executed statement.
+ *!
+ *! @seealso
+ *!   @[statistics()], @[server_info()], @[protocol_info()], @[host_info()]
+ */
+static void f_info(INT32 args) {
+  MYSQL *mysql;
+  const char *info;
+
+  pop_n_elems(args);
+
+  if (!PIKE_MYSQL->mysql) {
+    pike_mysql_reconnect (1);
+    push_undefined();
+    return;
+  }
+
+  mysql = PIKE_MYSQL->mysql;
+
+  MYSQL_ALLOW();
+
+  info = mysql_info(mysql);
+
+  MYSQL_DISALLOW();
+
+  if (info) push_text(info);
+  else push_undefined();
+}
+
 /*! @decl int protocol_info()
  *!
  *! Give the Mysql protocol version
@@ -1465,8 +1562,8 @@ static void f_list_dbs(INT32 args)
     push_object(o = clone_object(mysql_result_program, 1));
 
     /* Set the result. */
-    if ((!(res = (struct precompiled_mysql_result *)
-	   get_storage(o, mysql_result_program))) || res->result) {
+    if ((!(res = get_storage(o, mysql_result_program))) ||
+        res->result) {
       mysql_free_result(result);
       Pike_error("Mysql.mysql->list_dbs(): Bad mysql result object!\n");
     }
@@ -1524,7 +1621,7 @@ static void f_list_tables(INT32 args)
 
     MYSQL_DISALLOW();
 
-    Pike_error("Mysql.mysql->list_tables(): Cannot list databases: %s\n", err);
+    Pike_error("Mysql.mysql->list_tables(): Cannot list tables: %s\n", err);
   }
 
   pop_n_elems(args);
@@ -1539,8 +1636,8 @@ static void f_list_tables(INT32 args)
     push_object(o = clone_object(mysql_result_program, 1));
 
     /* Set the result. */
-    if ((!(res = (struct precompiled_mysql_result *)
-	   get_storage(o, mysql_result_program))) || res->result) {
+    if ((!(res = get_storage(o, mysql_result_program))) ||
+        res->result) {
       mysql_free_result(result);
       Pike_error("Mysql.mysql->list_tables(): Bad mysql result object!\n");
     }
@@ -1590,10 +1687,28 @@ static void f_list_tables(INT32 args)
  *! @multiset
  *!   @index "primary_key"
  *!     This field is part of the primary key for this table.
+ *!   @index "unique"
+ *!     This field is part of a unique key for this table.
+ *!   @index "multiple_key"
+ *!     This field is part of a nonunique key for this table.
  *!   @index "not_null"
- *!     This field may not be NULL.
+ *!     This field cannot be NULL.
  *!   @index "blob"
- *!     This field is a blob field.
+ *!     This field is a BLOB or TEXT.
+ *!   @index "auto_increment"
+ *!     This field has the AUTO_INCREMENT attribute.
+ *!   @index "zerofill"
+ *!     This Field has the ZEROFILL attribute.
+ *!   @index "binary"
+ *!     This Field has the BINARY attribute.
+ *!   @index "enum"
+ *!     This Field is an ENUM.
+ *!   @index "set"
+ *!     This Field is a SET.
+ *!   @index "unsigned"
+ *!     This Field has the UNSIGNED attribute.
+ *!   @index "numeric"
+ *!     This Field is numeric.
  *! @endmultiset
  *!
  *! @note
@@ -1614,10 +1729,10 @@ static void f_list_fields(INT32 args)
   char *wild = NULL;
 
   if (!args) {
-    SIMPLE_TOO_FEW_ARGS_ERROR ("Mysql.mysql->list_fields", 1);
+    SIMPLE_TOO_FEW_ARGS_ERROR ("list_fields", 1);
   }
 
-  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->list_fields", 1);
+  CHECK_8BIT_NONBINARY_STRING ("list_fields", 1);
   if (sp[-args].u.string->len > 125) {
     if (sp[-args].u.string->len < 1024) {
       Pike_error("Table name \"%s\" is too long (max 125 characters)\n",
@@ -1663,7 +1778,7 @@ static void f_list_fields(INT32 args)
 
     MYSQL_DISALLOW();
 
-    Pike_error("Mysql.mysql->list_fields(): Cannot list databases: %s\n", err);
+    Pike_error("Mysql.mysql->list_fields(): Cannot list fields: %s\n", err);
   }
 
   pop_n_elems(args);
@@ -1711,7 +1826,7 @@ static void f_list_processes(INT32 args)
 
     MYSQL_DISALLOW();
 
-    Pike_error("Mysql.mysql->list_processes(): Cannot list databases: %s\n", err);
+    Pike_error("Mysql.mysql->list_processes(): Cannot list processes: %s\n", err);
   }
 
   {
@@ -1724,8 +1839,8 @@ static void f_list_processes(INT32 args)
     push_object(o = clone_object(mysql_result_program, 1));
 
     /* Set the result. */
-    if ((!(res = (struct precompiled_mysql_result *)
-	   get_storage(o, mysql_result_program))) || res->result) {
+    if ((!(res = get_storage(o, mysql_result_program))) ||
+        res->result) {
       mysql_free_result(result);
       Pike_error("Mysql.mysql->list_processes(): Bad mysql result object!\n");
     }
@@ -1775,7 +1890,7 @@ static void f_get_charset (INT32 args)
   if (PIKE_MYSQL->conn_charset)
     ref_push_string (PIKE_MYSQL->conn_charset);
   else
-    push_constant_text ("latin1");
+    push_text ("latin1");
 }
 
 static void f__can_send_as_latin1 (INT32 args)
@@ -1788,7 +1903,7 @@ static void f__can_send_as_latin1 (INT32 args)
 
   if (args != 1)
     SIMPLE_WRONG_NUM_ARGS_ERROR ("_can_send_as_latin1", 1);
-  if (Pike_sp[-1].type != T_STRING)
+  if (TYPEOF(Pike_sp[-1]) != T_STRING)
     SIMPLE_ARG_TYPE_ERROR ("_can_send_as_latin1", 0, "string");
   str = Pike_sp[-1].u.string;
 
@@ -1857,6 +1972,12 @@ PIKE_MODULE_INIT
 
   /* function(void:int|string) */
   ADD_FUNCTION("error", f_error,tFunc(tVoid,tOr(tInt,tStr)), ID_PUBLIC);
+  /* function(void:int) */
+  ADD_FUNCTION("errno", f_errno,tFunc(tVoid,tInt), ID_PUBLIC);
+#ifdef HAVE_MYSQL_SQLSTATE
+  /* function(void:string) */
+  ADD_FUNCTION("sqlstate", f_sqlstate,tFunc(tVoid,tStr), ID_PUBLIC);
+#endif
   /* function(string|void, string|void, string|void, string|void:void) */
   ADD_FUNCTION("create", f_create,tFunc(tOr(tStr,tVoid) tOr(tStr,tVoid) tOr(tStr,tVoid) tOr(tStr,tVoid) tOr(tMapping, tVoid),tVoid), ID_PUBLIC);
   /* function(int, void|mapping:string) */
@@ -1900,6 +2021,8 @@ PIKE_MODULE_INIT
   ADD_FUNCTION("server_info", f_server_info,tFunc(tVoid,tStr), ID_PUBLIC);
   /* function(void:string) */
   ADD_FUNCTION("host_info", f_host_info,tFunc(tVoid,tStr), ID_PUBLIC);
+  /* function(void:string) */
+  ADD_FUNCTION("info", f_info,tFunc(tVoid,tStr), ID_PUBLIC);
   /* function(void:int) */
   ADD_FUNCTION("protocol_info", f_protocol_info,tFunc(tVoid,tInt), ID_PUBLIC);
   /* function(void|string:object) */
@@ -1942,12 +2065,37 @@ PIKE_MODULE_INIT
   mysql_program = end_program();
   add_program_constant("mysql", mysql_program, 0);
 
+#ifdef HAVE_MYSQL_MYSQLD_ERNAME_H
+  {
+    struct mysqld_ername {
+      const char * msg;
+      unsigned int code;
+    };
+    unsigned int i;
+    static const struct mysqld_ername list[] = {
+#include <mysql/mysqld_ername.h>
+    };
+    const unsigned int n = sizeof(list)/sizeof(list[0]);
+
+    start_new_program();
+
+    for (i = 0; i < n; i++) {
+      add_integer_constant(list[i].msg, list[i].code, 0);
+    }
+
+    mysql_error_program = end_program();
+    add_program_constant("error", mysql_error_program, 0);
+  }
+#endif
+
   /* function(void:string) */
   ADD_FUNCTION("client_info", f_client_info,tFunc(tVoid,tStr), ID_PUBLIC);
 
+#ifndef HAVE_MYSQL_REAL_CONNECT
 #ifdef HAVE_MYSQL_PORT
   STUPID_PORT_INIT();
 #endif /* HAVE_MYSQL_PORT */
+#endif /* !HAVE_MYSQL_REAL_CONNECT */
 
   init_mysql_res_programs();
 #else
@@ -1961,13 +2109,22 @@ PIKE_MODULE_EXIT
 #ifdef HAVE_MYSQL
   exit_mysql_res();
 
+#ifndef HAVE_MYSQL_REAL_CONNECT
 #ifdef HAVE_MYSQL_PORT
   STUPID_PORT_DESTROY();
 #endif /* HAVE_MYSQL_PORT */
+#endif /* !HAVE_MYSQL_REAL_CONNECT */
 
   if (mysql_program) {
     free_program(mysql_program);
     mysql_program = NULL;
   }
+
+#ifdef HAVE_MYSQL_MYSQLD_ERNAME_H
+  if (mysql_error_program) {
+    free_program(mysql_error_program);
+    mysql_error_program = NULL;
+  }
+#endif
 #endif /* HAVE_MYSQL */
 }
