@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 9c58c14fdf7284828d452fa9fd94fc453b139e51 $
 */
 
 /*
@@ -48,6 +47,7 @@
 #include "array.h"
 #include "pike_error.h"
 #include "threads.h"
+#include "pike_types.h"
 
 #include "../Image/image.h"
 #include "../Image/colortable.h"
@@ -198,22 +198,21 @@ void image_gif_header_block(INT32 args)
 
    if (args<3)
       Pike_error("Image.GIF.header_block(): too few arguments\n");
-   if (sp[-args].type!=T_INT ||
-       sp[1-args].type!=T_INT)
+   if (TYPEOF(sp[-args]) != T_INT ||
+       TYPEOF(sp[1-args]) != T_INT)
       Pike_error("Image.GIF.header_block(): illegal argument(s) 1..2 (expected int)\n");
 
    xs=sp[-args].u.integer;
    ys=sp[1-args].u.integer;
 
-   if (sp[2-args].type==T_INT)
+   if (TYPEOF(sp[2-args]) == T_INT)
    {
       numcolors=sp[2-args].u.integer;
       if (numcolors<2) numcolors=2;
       globalpalette=0;
-   } 
-   else if (sp[2-args].type==T_OBJECT &&
-	    (nct=(struct neo_colortable*)
-	        get_storage(sp[2-args].u.object,image_colortable_program)))
+   }
+   else if (TYPEOF(sp[2-args]) == T_OBJECT &&
+	    (nct=get_storage(sp[2-args].u.object,image_colortable_program)))
    {
       numcolors=image_colortable_size(nct);
       globalpalette=1;
@@ -222,20 +221,20 @@ void image_gif_header_block(INT32 args)
       Pike_error("Image.GIF.header_block(): illegal argument 3 (expected int or colortable object)\n");
 
    if (args>=4) {
-      if (sp[3-args].type!=T_INT)
+      if (TYPEOF(sp[3-args]) != T_INT)
 	 Pike_error("Image.GIF.header_block(): illegal argument 4 (expected int)\n");
       else
 	 bkgi=sp[3-args].u.integer;
    }
    if (args>=5) {
-      if (sp[4-args].type!=T_INT)
+      if (TYPEOF(sp[4-args]) != T_INT)
 	 Pike_error("Image.GIF.header_block(): illegal argument 4 (expected int)\n");
       else
 	 gif87a=sp[4-args].u.integer;
    }
    if (args>=7) {
-      if (sp[5-args].type!=T_INT ||
-	  sp[6-args].type!=T_INT)
+      if (TYPEOF(sp[5-args]) != T_INT ||
+	  TYPEOF(sp[6-args]) != T_INT)
 	 Pike_error("Image.GIF.header_block(): illegal argument(s) 5..6 (expected int)\n");
       else
 	 if (sp[5-args].u.integer &&
@@ -247,9 +246,9 @@ void image_gif_header_block(INT32 args)
    }
    if (args>=10)
    {
-      if (sp[7-args].type!=T_INT ||
-	  sp[8-args].type!=T_INT ||
-	  sp[9-args].type!=T_INT)
+      if (TYPEOF(sp[7-args]) != T_INT ||
+	  TYPEOF(sp[8-args]) != T_INT ||
+	  TYPEOF(sp[9-args]) != T_INT)
 	 Pike_error("Image.GIF.header_block(): illegal argument 8..10 (expected int)\n");
       alphacolor.r=(unsigned char)(sp[7-args].u.integer);
       alphacolor.g=(unsigned char)(sp[8-args].u.integer);
@@ -281,7 +280,7 @@ void image_gif_header_block(INT32 args)
    {
       ps=begin_shared_string((1<<bpp)*3);
       image_colortable_write_rgb(nct,(unsigned char *)ps->str);
-      MEMSET(ps->str + (numcolors + alphaentry) * 3, 0,
+      memset(ps->str + (numcolors + alphaentry) * 3, 0,
 	     ((1 << bpp) - numcolors - alphaentry) * 3);
 
       if (alphaentry) 
@@ -322,7 +321,7 @@ void image_gif_header_block(INT32 args)
 void image_gif_end_block(INT32 args)
 {
    pop_n_elems(args);
-   push_constant_text("\x3b");
+   push_text("\x3b");
 }
 
 /*
@@ -369,11 +368,11 @@ static void image_gif__gce_block(INT32 args)
    char buf[20];
    if (args<5) 
       Pike_error("Image.GIF._gce_block(): too few arguments\n");
-   if (sp[-args].type!=T_INT ||
-       sp[1-args].type!=T_INT ||
-       sp[2-args].type!=T_INT ||
-       sp[3-args].type!=T_INT ||
-       sp[4-args].type!=T_INT)
+   if (TYPEOF(sp[-args]) != T_INT ||
+       TYPEOF(sp[1-args]) != T_INT ||
+       TYPEOF(sp[2-args]) != T_INT ||
+       TYPEOF(sp[3-args]) != T_INT ||
+       TYPEOF(sp[4-args]) != T_INT)
       Pike_error("Image.GIF._gce_block(): Illegal argument(s)\n");
    sprintf(buf,"%c%c%c%c%c%c%c%c",
 	   0x21, /* extension intruder */
@@ -440,13 +439,13 @@ CHRONO("gif _render_block begun");
    if (args<8)
       Pike_error("Image.GIF._render_block(): Too few arguments\n");
 
-   if (sp[-args].type!=T_INT ||
-       sp[1-args].type!=T_INT ||
-       sp[2-args].type!=T_INT ||
-       sp[3-args].type!=T_INT ||
-       sp[4-args].type!=T_INT ||
-       sp[5-args].type!=T_STRING ||
-       sp[7-args].type!=T_INT)
+   if (TYPEOF(sp[-args]) != T_INT ||
+       TYPEOF(sp[1-args]) != T_INT ||
+       TYPEOF(sp[2-args]) != T_INT ||
+       TYPEOF(sp[3-args]) != T_INT ||
+       TYPEOF(sp[4-args]) != T_INT ||
+       TYPEOF(sp[5-args]) != T_STRING ||
+       TYPEOF(sp[7-args]) != T_INT)
       Pike_error("Image.GIF._render_block(): Illegal argument(s)\n");
 
    xpos=sp[-args].u.integer;
@@ -460,11 +459,11 @@ CHRONO("gif _render_block begun");
    if (bpp<1) bpp=1;
    else if (bpp>8) bpp=8;
 
-   if (sp[6-args].type==T_INT)
+   if (TYPEOF(sp[6-args]) == T_INT)
    {
       localpalette=0;
    }
-   else if (sp[6-args].type==T_STRING)
+   else if (TYPEOF(sp[6-args]) == T_STRING)
    {
       cps=sp[6-args].u.string;
       localpalette=1;
@@ -551,7 +550,7 @@ CHRONO("gif _render_block push of packed data begin");
       {
 	 ps=begin_shared_string(256);
 	 *((unsigned char*)(ps->str))=255;
-	 MEMCPY(ps->str+1,lzw.out+i,255);
+	 memcpy(ps->str+1,lzw.out+i,255);
 	 push_string(end_shared_string(ps));
 	 numstrings++;
 	 if (numstrings>32) /* shrink stack */
@@ -565,7 +564,7 @@ CHRONO("gif _render_block push of packed data begin");
       {
 	 ps=begin_shared_string(lzw.outpos-i+2);
 	 ps->str[0] = DO_NOT_WARN((char)(lzw.outpos-i));
-	 MEMCPY(ps->str+1,lzw.out+i,lzw.outpos-i);
+	 memcpy(ps->str+1,lzw.out+i,lzw.outpos-i);
 	 ps->str[lzw.outpos-i+1]=0;
 	 push_string(end_shared_string(ps));
 	 numstrings++;
@@ -729,23 +728,24 @@ void image_gif_render_block(INT32 args)
 
 CHRONO("gif render_block begin");
 
+   alphacolor.r=alphacolor.g=alphacolor.b=0;
+
    if (args<2) 
       Pike_error("Image.GIF.render_block(): Too few arguments\n");
-   if (sp[-args].type!=T_OBJECT ||
-       !(img=(struct image*)get_storage(sp[-args].u.object,image_program)))
+   if (TYPEOF(sp[-args]) != T_OBJECT ||
+       !(img=get_storage(sp[-args].u.object,image_program)))
       Pike_error("Image.GIF.render_block(): Illegal argument 1 (expected image object)\n");
    else if (!img->img)
       Pike_error("Image.GIF.render_block(): given image has no image\n");
-   if (sp[1-args].type!=T_OBJECT ||
-       !(nct=(struct neo_colortable*)
-  	     get_storage(sp[1-args].u.object,image_colortable_program)))
+   if (TYPEOF(sp[1-args]) != T_OBJECT ||
+       !(nct=get_storage(sp[1-args].u.object,image_colortable_program)))
       Pike_error("Image.GIF.render_block(): Illegal argument 2 (expected colortable object)\n");
 
    if (args>=4)
    {
-      if (sp[2-args].type!=T_INT)
+      if (TYPEOF(sp[2-args]) != T_INT)
 	 Pike_error("Image:GIF.render_block(): Illegal argument 3 (expected int)\n");
-      if (sp[3-args].type!=T_INT)
+      if (TYPEOF(sp[3-args]) != T_INT)
 	 Pike_error("Image:GIF.render_block(): Illegal argument 4 (expected int)\n");
       xpos=sp[2-args].u.integer;
       ypos=sp[3-args].u.integer;
@@ -762,15 +762,15 @@ CHRONO("gif render_block begin");
 
    if (args>=5)
    {
-      if (sp[4-args].type!=T_INT)
+      if (TYPEOF(sp[4-args]) != T_INT)
 	 Pike_error("Image:GIF.render_block(): Illegal argument 5 (expected int)\n");
       localpalette=sp[4-args].u.integer;
    }
    else localpalette=0;
    if (args>=6)
    {
-      if (sp[5-args].type==T_OBJECT &&
-	  (alpha=(struct image*)get_storage(sp[5-args].u.object,image_program)))
+      if (TYPEOF(sp[5-args]) == T_OBJECT &&
+	  (alpha=get_storage(sp[5-args].u.object,image_program)))
       {
 	 if (!alpha->img)
 	    Pike_error("Image.GIF.render_block(): given alpha channel has no image\n");
@@ -780,12 +780,11 @@ CHRONO("gif render_block begin");
 	 alphaidx=numcolors;
 	 n=9;
 
-	 alphacolor.r=alphacolor.g=alphacolor.b=0;
 	 if (args>=9)
 	 {
-	    if (sp[6-args].type!=T_INT ||
-		sp[7-args].type!=T_INT ||
-		sp[8-args].type!=T_INT)
+	    if (TYPEOF(sp[6-args]) != T_INT ||
+		TYPEOF(sp[7-args]) != T_INT ||
+		TYPEOF(sp[8-args]) != T_INT)
 	       Pike_error("Image.GIF.render_block(): illegal argument 7..9 (expected int)\n");
 	    alphacolor.r=(unsigned char)(sp[6-args].u.integer);
 	    alphacolor.g=(unsigned char)(sp[7-args].u.integer);
@@ -798,7 +797,7 @@ CHRONO("gif render_block begin");
 	       Pike_error("Image.GIF.render_block(): too many colors in colortable (255 is max, need one for transparency)\n");
 	 }
       }
-      else if (sp[5-args].type==T_INT)
+      else if (TYPEOF(sp[5-args]) == T_INT)
       {
 	 n=5;
       }
@@ -809,7 +808,7 @@ CHRONO("gif render_block begin");
       if (args>n) /* interlace and gce arguments */
       {
 	 if (args>n) {
-	    if (sp[n-args].type!=T_INT)
+	    if (TYPEOF(sp[n-args]) != T_INT)
 	       Pike_error("Image:GIF.render_block(): Illegal argument %d (expected int)\n",n+1);
 	    else
 	       delay=sp[n-args].u.integer;
@@ -818,7 +817,7 @@ CHRONO("gif render_block begin");
 	 if (!alpha)
 	 {
 	    if (args>n) {
-	       if (sp[n-args].type!=T_INT)
+	       if (TYPEOF(sp[n-args]) != T_INT)
 	          Pike_error("Image:GIF.render_block(): Illegal argument %d (expected int)\n",n+1);
 	       else
 	       {
@@ -837,21 +836,21 @@ CHRONO("gif render_block begin");
 	    n++;
 	 }
 	 if (args>n) {
-	    if (sp[n-args].type!=T_INT)
+	    if (TYPEOF(sp[n-args]) != T_INT)
 	       Pike_error("Image:GIF.render_block(): Illegal argument %d (expected int)\n",n+1);
 	    else
 	       interlace=!!sp[n-args].u.integer;
 	 }
 	 n++;
 	 if (args>n) {
-	    if (sp[n-args].type!=T_INT)
+	    if (TYPEOF(sp[n-args]) != T_INT)
 	       Pike_error("Image:GIF.render_block(): Illegal argument %d (expected int)\n",n+1);
 	    else
 	       user_input=!!sp[n-args].u.integer;
 	 }
 	 n++;
 	 if (args>n) {
-	    if (sp[n-args].type!=T_INT)
+	    if (TYPEOF(sp[n-args]) != T_INT)
 	       Pike_error("Image:GIF.render_block(): Illegal argument %d (expected int)\n",n+1);
 	    else
 	       disposal=sp[n-args].u.integer&7;
@@ -914,7 +913,7 @@ CHRONO("render_block index end");
 CHRONO("gif render_block write local colortable begin");
       ps=begin_shared_string((1<<bpp)*3);
       image_colortable_write_rgb(nct,(unsigned char *)ps->str);
-      MEMSET(ps->str+(numcolors+alphaentry)*3,0,((1<<bpp)-numcolors)*3);
+      memset(ps->str+(numcolors+alphaentry)*3,0,((1<<bpp)-numcolors)*3);
       if (alphaentry) 
       {
 	 ps->str[3*alphaidx+0]=alphacolor.r;
@@ -1032,15 +1031,15 @@ void _image_gif_encode(INT32 args,int fs)
    if (args<1)
       Pike_error("Image.GIF.encode(): Too few arguments\n");
 
-   if (sp[-args].type!=T_OBJECT ||
-       !(img=(struct image*)get_storage(imgobj=sp[-args].u.object,
+   if (TYPEOF(sp[-args]) != T_OBJECT ||
+       !(img=get_storage(imgobj=sp[-args].u.object,
 					image_program)))
       Pike_error("Image.GIF.encode(): Illegal argument 1 (expected image object)\n");
    add_ref(imgobj);
 
    
    if (args>=2) {
-      if (sp[1-args].type==T_INT)
+      if (TYPEOF(sp[1-args]) == T_INT)
       {
 	 if (args!=4)
 	 {
@@ -1050,18 +1049,16 @@ void _image_gif_encode(INT32 args,int fs)
 	    ref_push_object(imgobj);
 	    push_int(n);
 	    nctobj=clone_object(image_colortable_program,2);
-	    nct=(struct neo_colortable*)
-	       get_storage(nctobj,image_colortable_program);
+	    nct=get_storage(nctobj,image_colortable_program);
 	    if (!nct)
 	       Pike_error("Image.GIF.encode(): Internal error; colortable isn't colortable\n");
 	    arg=2;
 	 }
 	 else arg=1;
       }
-      else if (sp[1-args].type!=T_OBJECT)
+      else if (TYPEOF(sp[1-args]) != T_OBJECT)
 	 Pike_error("Image.GIF.encode(): Illegal argument 2 (expected image or colortable object or int)\n");
-      else if ((nct=(struct neo_colortable*)
-		get_storage(nctobj=sp[1-args].u.object,image_colortable_program)))
+      else if ((nct=get_storage(nctobj=sp[1-args].u.object,image_colortable_program)))
       {
 	add_ref(nctobj);
       }
@@ -1073,16 +1070,15 @@ void _image_gif_encode(INT32 args,int fs)
    }
    /* check transparency arguments */
    if (args-arg>0) {
-      if (sp[arg-args].type==T_OBJECT &&
-	  (alpha=(struct image*)
-	   get_storage(alphaobj=sp[arg-args].u.object,image_program)))
+      if (TYPEOF(sp[arg-args]) == T_OBJECT &&
+	  (alpha=get_storage(alphaobj=sp[arg-args].u.object,image_program)))
       {
 	add_ref(alphaobj);
 	 if (args-arg>1) {
 	    if (args-arg<4 ||
-		sp[1+arg-args].type!=T_INT ||
-		sp[2+arg-args].type!=T_INT ||
-		sp[3+arg-args].type!=T_INT)
+		TYPEOF(sp[1+arg-args]) != T_INT ||
+		TYPEOF(sp[2+arg-args]) != T_INT ||
+		TYPEOF(sp[3+arg-args]) != T_INT)
 	       Pike_error("Image.GIF.encode: Illegal arguments %d..%d (expected int)\n",arg+2,arg+4);
 	    else
 	    {
@@ -1097,12 +1093,12 @@ void _image_gif_encode(INT32 args,int fs)
 	 else tridx=image_colortable_size(nct);
       }
       else if (args-arg<3 ||
-	       sp[arg-args].type!=T_INT ||
-	       sp[1+arg-args].type!=T_INT ||
-	       sp[2+arg-args].type!=T_INT)
+	       TYPEOF(sp[arg-args]) != T_INT ||
+	       TYPEOF(sp[1+arg-args]) != T_INT ||
+	       TYPEOF(sp[2+arg-args]) != T_INT)
       {
 	 if (arg==2 &&
-	     sp[arg-args].type==T_INT)
+	     TYPEOF(sp[arg-args]) == T_INT)
 	    tridx=sp[arg-args].u.integer;
 	 else
 	    Pike_error("Image.GIF.encode(): Illegal argument %d or %d..%d\n",
@@ -1121,8 +1117,7 @@ void _image_gif_encode(INT32 args,int fs)
 	    push_object(imgobj);
 	    push_int(256);
 	    nctobj=clone_object(image_colortable_program,2);
-	    nct=(struct neo_colortable*)
-	       get_storage(nctobj,image_colortable_program);
+	    nct=get_storage(nctobj,image_colortable_program);
 	    if (!nct)
 	       Pike_error("Image.GIF.encode(): Internal error; colortable isn't colortable\n");
 	 }
@@ -1143,8 +1138,7 @@ void _image_gif_encode(INT32 args,int fs)
      ref_push_object(imgobj);
       if (alpha) push_int(255); else push_int(256);
       nctobj=clone_object(image_colortable_program,2);
-      nct=(struct neo_colortable*)
-	 get_storage(nctobj,image_colortable_program);
+      nct=get_storage(nctobj,image_colortable_program);
       if (!nct)
 	 Pike_error("Image.GIF.encode(): Internal error; colortable isn't colortable\n");
    }
@@ -1236,7 +1230,7 @@ void image_gif_netscape_loop_block(INT32 args)
    unsigned short loops=0;
    char buf[30];
    if (args) {
-      if (sp[-args].type!=T_INT) 
+      if (TYPEOF(sp[-args]) != T_INT)
 	 Pike_error("Image.GIF.netscape_loop_block: illegal argument (exected int)\n");
       else
 	/* Should check range here, but that wouldn't be strictly compatible. */
@@ -1432,8 +1426,8 @@ static void image_gif___decode(INT32 args)
    int n;
    ONERROR uwp;
 
-   if (args!=1 
-       || sp[-args].type!=T_STRING) 
+   if (args!=1
+       || TYPEOF(sp[-args]) != T_STRING)
       Pike_error("Image.GIF.__decode: illegal or illegal number of arguments\n");
 
    add_ref(str=sp[-args].u.string);
@@ -1670,7 +1664,7 @@ fprintf(stderr,"_gif_decode_lzw(%lx,%lu,%d,%lx,%lx,%lx,%lu,%d)\n",
 	s,len,obits,ncto,dest,alpha,dlen,tidx);
 #endif
 
-   nct=(struct neo_colortable*)get_storage(ncto,image_colortable_program);
+   nct=get_storage(ncto,image_colortable_program);
    if (!nct || nct->type!=NCT_FLAT) return; /* uh? */
 
    if (len<2) return;
@@ -1682,7 +1676,9 @@ fprintf(stderr,"_gif_decode_lzw(%lx,%lu,%d,%lx,%lx,%lx,%lu,%d)\n",
 
 #define MAX_GIF_CODE 4096
 
-   c=(struct lzwc*)xalloc(sizeof(struct lzwc)*MAX_GIF_CODE);
+   if (maxcode > MAX_GIF_CODE) return;
+
+   c=xalloc(sizeof(struct lzwc)*MAX_GIF_CODE);
 
    for (n=0; n<clearcode; n++)
       c[n].prev=0xffff,c[n].len=1,c[n].c=n;
@@ -1824,17 +1820,17 @@ static void gif_deinterlace(rgb_group *s,
    tmp=malloc(xsize*ysize*sizeof(rgb_group));
    if (!tmp) return;
 
-   MEMCPY(tmp,s,xsize*ysize*sizeof(rgb_group));
+   memcpy(tmp,s,xsize*ysize*sizeof(rgb_group));
 
    n=0;
    for (y=0; y<ysize; y+=8)
-      MEMCPY(s+y*xsize,tmp+n++*xsize,xsize*sizeof(rgb_group));
+      memcpy(s+y*xsize,tmp+n++*xsize,xsize*sizeof(rgb_group));
    for (y=4; y<ysize; y+=8)		  
-      MEMCPY(s+y*xsize,tmp+n++*xsize,xsize*sizeof(rgb_group));
+      memcpy(s+y*xsize,tmp+n++*xsize,xsize*sizeof(rgb_group));
    for (y=2; y<ysize; y+=4)		  
-      MEMCPY(s+y*xsize,tmp+n++*xsize,xsize*sizeof(rgb_group));
+      memcpy(s+y*xsize,tmp+n++*xsize,xsize*sizeof(rgb_group));
    for (y=1; y<ysize; y+=2)		  
-      MEMCPY(s+y*xsize,tmp+n++*xsize,xsize*sizeof(rgb_group));
+      memcpy(s+y*xsize,tmp+n++*xsize,xsize*sizeof(rgb_group));
    
    free(tmp);
 }
@@ -1852,12 +1848,12 @@ void image_gif__decode(INT32 args)
    if (!args)
       Pike_error("Image.GIF._decode: too few argument\n");
 
-   if (sp[-args].type==T_ARRAY)
+   if (TYPEOF(sp[-args]) == T_ARRAY)
       pop_n_elems(args-1);
    else
       image_gif___decode(args);
 
-   if (sp[-1].type!=T_ARRAY)
+   if (TYPEOF(sp[-1]) != T_ARRAY)
       Pike_error("Image.GIF._decode: internal error: "
 	    "illegal result from __decode\n");
    
@@ -1869,7 +1865,7 @@ void image_gif__decode(INT32 args)
    push_svalue(a->item+0); /* xsize */
    push_svalue(a->item+1); /* ysize */
 
-   if (a->item[3].type==T_STRING)
+   if (TYPEOF(a->item[3]) == T_STRING)
    {
       push_svalue(a->item+3);
       push_object(cto=clone_object(image_colortable_program,1));
@@ -1886,9 +1882,9 @@ void image_gif__decode(INT32 args)
 
    i=5;
    for (; i<a->size; i++)
-      if (a->item[i].type!=T_ARRAY ||
+      if (TYPEOF(a->item[i]) != T_ARRAY ||
 	  (b=a->item[i].u.array)->size<1 ||
-	  b->item[0].type!=T_INT)
+	  TYPEOF(b->item[0]) != T_INT)
 	 Pike_error("Image.GIF._decode: given (__decode'd) "
 	       "array has illegal contents (position %d)\n",i);
       else
@@ -1899,14 +1895,14 @@ void image_gif__decode(INT32 args)
 		  Pike_error("Image.GIF._decode: given (__decode'd) "
 			"array has illegal contents "
 			"(illegal size of block array in position %d)\n",i);
-	       if (b->item[0].type!=T_INT ||
-		   b->item[1].type!=T_INT ||
-		   b->item[2].type!=T_INT ||
-		   b->item[3].type!=T_INT ||
-		   b->item[4].type!=T_INT ||
-		   b->item[5].type!=T_INT ||
-		   b->item[7].type!=T_INT ||
-		   b->item[8].type!=T_STRING)
+	       if (TYPEOF(b->item[0]) != T_INT ||
+		   TYPEOF(b->item[1]) != T_INT ||
+		   TYPEOF(b->item[2]) != T_INT ||
+		   TYPEOF(b->item[3]) != T_INT ||
+		   TYPEOF(b->item[4]) != T_INT ||
+		   TYPEOF(b->item[5]) != T_INT ||
+		   TYPEOF(b->item[7]) != T_INT ||
+		   TYPEOF(b->item[8]) != T_STRING)
 		  Pike_error("Image.GIF._decode: given (__decode'd) "
 			"array has illegal contents "
 			"(illegal type(s) in block array in position %d)\n",i);
@@ -1918,28 +1914,37 @@ void image_gif__decode(INT32 args)
 
 	       interlace=b->item[5].u.integer;
 
-	       if (b->item[6].type==T_STRING)
+	       if (TYPEOF(b->item[6]) == T_STRING)
 	       {
 		  push_svalue(b->item+6);
 		  lcto=clone_object(image_colortable_program,1);
+                  push_object(lcto);
 	       }
 	       else
 	       {
 		  lcto=cto;
-		  if (lcto) add_ref(lcto);
+		  if (lcto) {
+                      ref_push_object(lcto);
+                  } else {
+                      push_int(0);
+                  }
 	       }
 
 	       push_int(b->item[3].u.integer);
 	       push_int(b->item[4].u.integer);
 	       o=clone_object(image_program,2);
-	       img=(struct image*)get_storage(o,image_program);
+	       img=get_storage(o,image_program);
 	       push_object(o);
+
+               /* Swap colortable and image program */
+               stack_swap();
+
 	       if (transparency)
 	       {
 		  push_int(b->item[3].u.integer);
 		  push_int(b->item[4].u.integer);
 		  o2=clone_object(image_program,2);
-		  aimg=(struct image*)get_storage(o2,image_program);
+		  aimg=get_storage(o2,image_program);
 		  push_object(o2);
 		  if (lcto)
 		     _gif_decode_lzw((unsigned char *)
@@ -1967,6 +1972,9 @@ void image_gif__decode(INT32 args)
 				     0);
 	       }
 
+               /* Swap colortable and image program */
+               stack_swap();
+
 	       if (interlace)
 	       {
 		  gif_deinterlace(img->img,img->xsize,img->ysize);
@@ -1974,9 +1982,6 @@ void image_gif__decode(INT32 args)
 		     gif_deinterlace(aimg->img,aimg->xsize,aimg->ysize);
 	       }
 
-	       if (lcto) push_object(lcto); 
-	       else push_int(0);
-	       
 	       push_int(interlace);
 	       push_int(transparency_index);
 	       push_int(delay);
@@ -1993,7 +1998,7 @@ void image_gif__decode(INT32 args)
 		  Pike_error("Image.GIF._decode: given (__decode'd) "
 			"array has illegal contents "
 			"(illegal size of block array in position %d)\n",i);
-	       if (b->item[2].type!=T_STRING)
+	       if (TYPEOF(b->item[2]) != T_STRING)
 		  Pike_error("Image.GIF._decode: given (__decode'd) "
 			"array has illegal contents "
 			"(no data string of block array in position %d)\n",i);
@@ -2009,6 +2014,9 @@ void image_gif__decode(INT32 args)
 			transparency_index=s[3];
 		     }
 		     break;
+	          case 0x01: /* Plain Text Extension. */
+	          case 0xfe: /* Comment Extension. */
+	          case 0xff: /* Application Extension. */
 		  default: /* unknown */
 		     push_svalue(a->item+i);
 		     n++;
@@ -2060,18 +2068,18 @@ void image_gif_decode(INT32 args)
    if (!args)
       Pike_error("Image.GIF._decode: too few argument\n");
 
-   if (sp[-args].type==T_ARRAY)
+   if (TYPEOF(sp[-args]) == T_ARRAY)
    {
       pop_n_elems(args-1);
       if (sp[-args].u.array->size<4)
 	 Pike_error("Image.GIF.decode: illegal argument\n");
-      if (sp[-args].u.array->item[3].type!=T_ARRAY)
+      if (TYPEOF(sp[-args].u.array->item[3]) != T_ARRAY)
 	 image_gif__decode(1);
    }
    else
       image_gif__decode(args);
 
-   if (sp[-1].type!=T_ARRAY)
+   if (TYPEOF(sp[-1]) != T_ARRAY)
       Pike_error("Image.GIF.decode: internal error: "
 	    "illegal result from _decode\n");
 
@@ -2085,16 +2093,16 @@ void image_gif_decode(INT32 args)
    o=clone_object(image_program,2);
    
    for (n=4; n<a->size; n++)
-      if (a->item[n].type==T_ARRAY
+      if (TYPEOF(a->item[n]) == T_ARRAY
 	  && (b=a->item[n].u.array)->size==11
-	  && b->item[0].type==T_INT 
+	  && TYPEOF(b->item[0]) == T_INT 
 	  && b->item[0].u.integer==GIF_RENDER
-	  && b->item[3].type==T_OBJECT
+          && TYPEOF(b->item[3]) == T_OBJECT
 	  && get_storage(b->item[3].u.object, image_program) )
       {
-	 if (b->item[4].type==T_OBJECT)
-	    alpha=(struct image*)get_storage(b->item[4].u.object,
-					     image_program);
+	 if (TYPEOF(b->item[4]) == T_OBJECT)
+	    alpha=get_storage(b->item[4].u.object,
+                              image_program);
 	 else
 	    alpha=NULL;
 	     
@@ -2150,18 +2158,18 @@ void image_gif_decode_layers(INT32 args)
    if (!args)
       Pike_error("Image.GIF.decode_layers: too few argument\n");
 
-   if (sp[-args].type==T_ARRAY)
+   if (TYPEOF(sp[-args]) == T_ARRAY)
    {
       pop_n_elems(args-1);
       if (sp[-args].u.array->size<4)
 	 Pike_error("Image.GIF.decode: illegal argument\n");
-      if (sp[-args].u.array->item[3].type!=T_ARRAY)
+      if (TYPEOF(sp[-args].u.array->item[3]) != T_ARRAY)
 	 image_gif__decode(1);
    }
    else
       image_gif__decode(args);
 
-   if (sp[-1].type!=T_ARRAY)
+   if (TYPEOF(sp[-1]) != T_ARRAY)
       Pike_error("Image.GIF.decode: internal error: "
 	    "illegal result from _decode\n");
 
@@ -2171,28 +2179,28 @@ void image_gif_decode_layers(INT32 args)
 	    "is too small\n");
 
    for (n=4; n<a->size; n++)
-      if (a->item[n].type==T_ARRAY
+      if (TYPEOF(a->item[n]) == T_ARRAY
 	  && (b=a->item[n].u.array)->size==11
-	  && b->item[0].type==T_INT 
+	  && TYPEOF(b->item[0]) == T_INT
 	  && b->item[0].u.integer==GIF_RENDER
-	  && b->item[3].type==T_OBJECT
+          && TYPEOF(b->item[3]) == T_OBJECT
 	  && get_storage(b->item[3].u.object, image_program) )
       {
-	 if (b->item[4].type==T_OBJECT)
-	    alpha=(struct image*)get_storage(b->item[4].u.object,
-					     image_program);
+	 if (TYPEOF(b->item[4]) == T_OBJECT)
+	    alpha=get_storage(b->item[4].u.object,
+                              image_program);
 	 else
 	    alpha=NULL;
 	     
 	 if (alpha) 
 	 {
-	    push_constant_text("image");
+	    push_text("image");
 	    push_svalue(b->item+3);
-	    push_constant_text("alpha");
+	    push_text("alpha");
 	    push_svalue(b->item+4);
-	    push_constant_text("xoffset");
+	    push_text("xoffset");
 	    push_svalue(b->item+1);
-	    push_constant_text("yoffset");
+	    push_text("yoffset");
 	    push_svalue(b->item+2);
 	    f_aggregate_mapping(8);
 	    push_object(clone_object(image_layer_program,1));
@@ -2200,11 +2208,11 @@ void image_gif_decode_layers(INT32 args)
 	 }
 	 else
 	 {
-	    push_constant_text("image");
+	    push_text("image");
 	    push_svalue(b->item+3);
-	    push_constant_text("xoffset");
+	    push_text("xoffset");
 	    push_svalue(b->item+1);
-	    push_constant_text("yoffset");
+	    push_text("yoffset");
 	    push_svalue(b->item+2);
 	    f_aggregate_mapping(6);
 	    push_object(clone_object(image_layer_program,1));
@@ -2250,10 +2258,10 @@ void image_gif_decode_map(INT32 args)
 {
    image_gif_decode_layer(args);
 
-   push_constant_text("image");
-   push_constant_text("alpha");
-   push_constant_text("xsize");
-   push_constant_text("ysize");
+   push_text("image");
+   push_text("alpha");
+   push_text("xsize");
+   push_text("ysize");
    f_aggregate(4);
 #define stack_swap_behind() do { struct svalue _=sp[-2]; sp[-2]=sp[-3]; sp[-3]=_; } while(0)
    stack_dup();
@@ -2261,8 +2269,8 @@ void image_gif_decode_map(INT32 args)
    f_rows(2);
    f_call_function(1);
    f_mkmapping(2);
-   push_constant_text("type");
-   push_constant_text("image/gif");
+   ref_push_string(literal_type_string);
+   push_text("image/gif");
    f_aggregate_mapping(2);
    f_add(2);
 }
@@ -2285,8 +2293,8 @@ void image_gif__encode_render(INT32 args)
    int localp;
 
    if (args<2 ||
-       sp[-args].type!=T_ARRAY ||
-       sp[1-args].type!=T_INT)
+       TYPEOF(sp[-args]) != T_ARRAY ||
+       TYPEOF(sp[1-args]) != T_INT)
       Pike_error("Image.GIF._encode_render: Illegal argument(s) (expected array, int)\n");
 
    localp=sp[1-args].u.integer;
@@ -2304,26 +2312,25 @@ void image_gif__encode_render(INT32 args)
    
    push_int(localp);
 
-   if (a->item[4].type==T_OBJECT)
+   if (TYPEOF(a->item[4]) == T_OBJECT)
    {
       struct neo_colortable *nct;
 
-      nct=(struct neo_colortable*)
-	 get_storage(a->item[4].u.object,image_colortable_program);
+      nct=get_storage(a->item[4].u.object,image_colortable_program);
       if (!nct)
       {
 	 free_array(a);
 	 Pike_error("Image.GIF._encode_render: Passed object is not colortable\n");
       }
-      
+
       if (nct->type!=NCT_FLAT)
       {
 	 free_array(a);
 	 Pike_error("Image.GIF._encode_render: Passed colortable is not flat (sorry9\n");
       }
       push_svalue(a->item+4);
-      if (a->item[7].type==T_INT 
-	  && a->item[7].u.integer>=0 
+      if (TYPEOF(a->item[7]) == T_INT
+	  && a->item[7].u.integer>=0
 	  && a->item[7].u.integer<nct->u.flat.numentries)
       {
 	 push_int(nct->u.flat.entries[a->item[7].u.integer].color.r);
@@ -2341,14 +2348,14 @@ void image_gif__encode_render(INT32 args)
 
    push_svalue(a->item+8); /* delay */
 
-   if (a->item[4].type!=T_OBJECT)
+   if (TYPEOF(a->item[4]) != T_OBJECT)
       push_int(-1);
 
    push_svalue(a->item+6); /* interlace */
    push_svalue(a->item+9); /* user_input */
    push_svalue(a->item+10); /* disposal */
 
-   image_gif_render_block( (a->item[4].type==T_OBJECT) ? 13 : 10 );
+   image_gif_render_block( (TYPEOF(a->item[4]) == T_OBJECT) ? 13 : 10 );
 
    free_array(a);
 }
@@ -2361,15 +2368,15 @@ void image_gif__encode_extension(INT32 args)
    struct pike_string *s,*d;
 
    if (args<1 ||
-       sp[-args].type!=T_ARRAY)
+       TYPEOF(sp[-args]) != T_ARRAY)
       Pike_error("Image.GIF._encode_extension: Illegal argument(s) (expected array)\n");
 
    a=sp[-args].u.array;
 
    if (a->size<3)
       Pike_error("Image.GIF._encode_extension: Illegal size of array\n");
-   if (a->item[1].type!=T_INT ||
-       a->item[2].type!=T_STRING)
+   if (TYPEOF(a->item[1]) != T_INT ||
+       TYPEOF(a->item[2]) != T_STRING)
       Pike_error("Image.GIF._encode_extension: Illegal type in indices 1 or 2\n");
 
    add_ref(a);
@@ -2390,7 +2397,7 @@ void image_gif__encode_extension(INT32 args)
       {
    	 d=begin_shared_string(256);
 	 *((unsigned char*)(d->str))=255;
-	 MEMCPY(d->str+1,s->str+i,255);
+	 memcpy(d->str+1,s->str+i,255);
 	 push_string(end_shared_string(d));
 	 n++;
 	 if (n>32) /* shrink stack */
@@ -2404,7 +2411,7 @@ void image_gif__encode_extension(INT32 args)
       {
 	 d=begin_shared_string(s->len-i+2);
 	 d->str[0] = DO_NOT_WARN(s->len - i);
-	 MEMCPY(d->str+1, s->str+i, d->len-i);
+	 memcpy(d->str+1, s->str+i, d->len-i);
 	 d->str[d->len-i+1]=0;
 	 push_string(end_shared_string(d));
 	 n++;
@@ -2423,7 +2430,7 @@ void image_gif__encode(INT32 args)
    int n;
 
    if (args<1 ||
-       sp[-args].type!=T_ARRAY)
+       TYPEOF(sp[-args]) != T_ARRAY)
       Pike_error("Image.GIF._encode: Illegal argument (expected array)\n");
 
    add_ref(a=sp[-args].u.array);
@@ -2438,7 +2445,7 @@ void image_gif__encode(INT32 args)
    push_svalue(a->item+1); /* ysize */
    push_svalue(a->item+2); /* colortable or void */
 
-   if (a->item[3].type!=T_ARRAY 
+   if (TYPEOF(a->item[3]) != T_ARRAY
       || a->item[3].u.array->size<3)
    {
       free_array(a);
@@ -2456,7 +2463,7 @@ void image_gif__encode(INT32 args)
 
    while (pos<a->size)
    {
-      if (a->item[pos].type!=T_ARRAY)
+      if (TYPEOF(a->item[pos]) != T_ARRAY)
       {
 	 free_array(a);
 	 Pike_error("Image.GIF._encode: Illegal type on array index %d (expected array)\n",pos);
@@ -2464,7 +2471,7 @@ void image_gif__encode(INT32 args)
       b=a->item[pos].u.array;
 
       if (b->size<1
-	  || b->item[0].type!=T_INT)
+	  || TYPEOF(b->item[0]) != T_INT)
       {
 	 free_array(a);
 	 Pike_error("Image.GIF._encode: Illegal array on array index %d\n",pos);
@@ -2500,7 +2507,7 @@ static void image_gif_lzw_encode(INT32 args)
 {
    struct gif_lzw lzw;
 
-   if (!args || sp[-args].type!=T_STRING)
+   if (!args || TYPEOF(sp[-args]) != T_STRING)
       Pike_error("Image.GIF.lzw_encode(): illegal argument\n");
 
    image_gif_lzw_init(&lzw,8);
@@ -2539,7 +2546,7 @@ static void image_gif_lzw_decode(INT32 args)
    signed long maxcode;
    int reversebits=0;
 
-   if (!args || sp[-args].type!=T_STRING)
+   if (!args || TYPEOF(sp[-args]) != T_STRING)
       Pike_error("Image.GIF.lzw_encode(): illegal argument\n");
 
    s=(unsigned char*)sp[-args].u.string->str;
@@ -2566,9 +2573,9 @@ static void image_gif_lzw_decode(INT32 args)
 
    last=clearcode;
 
-   c=(struct lzwc*)xalloc(sizeof(struct lzwc)*MAX_GIF_CODE);
+   c=xalloc(sizeof(struct lzwc)*MAX_GIF_CODE);
 
-   dest0=(unsigned char*)malloc(dlen0=len*4);
+   dest0=malloc(dlen0=len*4);
    if (!dest0)
    {
       free(c);
@@ -2749,11 +2756,11 @@ struct program *image_encoding_gif_program=NULL;
 
 PIKE_MODULE_INIT
 {
-#ifdef DYNAMIC_MODULE
+#ifndef FAKE_DYNAMIC_LOAD
   image_program = PIKE_MODULE_IMPORT(Image, image_program);
   image_colortable_program=PIKE_MODULE_IMPORT(Image, image_colortable_program);
   image_layer_program = PIKE_MODULE_IMPORT(Image, image_layer_program);
-#endif /* DYNAMIC_MODULE */
+#endif
 
   if (!image_program || !image_colortable_program || !image_layer_program) {
     yyerror("Could not load Image module.");
