@@ -2,21 +2,36 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: fd07d615670ee009693d884efd708815265438ec $
 */
 
 #include "global.h"
-
 #include "config.h"
 
 #if !defined(HAVE_LIBJPEG)
 #undef HAVE_JPEGLIB_H
 #endif
 
-#ifdef HAVE_JPEGLIB_H
+#include "pike_macros.h"
+#include "object.h"
+#include "constants.h"
+#include "interpret.h"
+#include "svalue.h"
+#include "threads.h"
+#include "array.h"
+#include "mapping.h"
+#include "pike_error.h"
+#include "stralloc.h"
+#include "threads.h"
+#include "builtin_functions.h"
+#include "module_support.h"
+#include "operators.h"
+#include "pike_types.h"
+
 
 #include <stdio.h>
 
+#ifdef HAVE_JPEGLIB_H
+
 /* NOTE: INT32 and INT16 are redefined by <jmorecfg.h>. */
 #define XMD_H /* Avoid INT16 / INT32 being redefined */
 
@@ -49,21 +64,6 @@
 #undef EXTERN
 #endif
 
-#include "pike_macros.h"
-#include "object.h"
-#include "constants.h"
-#include "interpret.h"
-#include "svalue.h"
-#include "threads.h"
-#include "array.h"
-#include "mapping.h"
-#include "pike_error.h"
-#include "stralloc.h"
-#include "threads.h"
-#include "builtin_functions.h"
-#include "module_support.h"
-#include "operators.h"
-
 #define sp Pike_sp
 
 #ifdef HAVE_JPEGLIB_H
@@ -119,7 +119,7 @@ static const int reverse_quality[101]=
  *!	Independent JPEG Group.
  */
 
-static void my_output_message(struct jpeg_common_struct *cinfo)
+static void my_output_message(struct jpeg_common_struct *UNUSED(cinfo))
 {
    /* no message */
    /* (this should not be called) */
@@ -134,7 +134,7 @@ static void my_error_exit(struct jpeg_common_struct *cinfo)
    Pike_error("Image.JPEG: fatal error in libjpeg; %s\n",buffer);
 }
 
-static void my_emit_message(struct jpeg_common_struct *cinfo,int msg_level)
+static void my_emit_message(struct jpeg_common_struct *UNUSED(cinfo), int UNUSED(msg_level))
 {
    /* no trace */
 }
@@ -199,7 +199,7 @@ static boolean my_jpeg_marker_parser(j_decompress_ptr cinfo)
    length-=2;
    length &= 0xffff;
 
-   mm=(struct my_marker*)xalloc(sizeof(struct my_marker)+length);
+   mm=xalloc(sizeof(struct my_marker)+length);
    mm->id=cinfo->unread_marker;
    mm->len=length;
    mm->next=mds->first_marker;
@@ -243,7 +243,7 @@ static boolean my_empty_output_buffer(struct jpeg_compress_struct *cinfo)
    char *new;
 
    pos=dm->len; /* foo! dm->len-dm->pub.free_in_buffer; */
-   new=(char*)realloc(dm->buf,dm->len+BUF_INCREMENT);
+   new=realloc(dm->buf,dm->len+BUF_INCREMENT);
    if (!new) return FALSE;
 
    dm->buf = new;
@@ -254,7 +254,7 @@ static boolean my_empty_output_buffer(struct jpeg_compress_struct *cinfo)
    return TRUE;
 }
 
-static void my_term_destination(struct jpeg_compress_struct *cinfo)
+static void my_term_destination(struct jpeg_compress_struct *UNUSED(cinfo))
 {
    /* don't do anything */
 }
@@ -281,7 +281,7 @@ static int parameter_int(struct svalue *map,struct pike_string *what,INT32 *p)
    struct svalue *v;
    v=low_mapping_string_lookup(map->u.mapping,what);
 
-   if (!v || v->type!=T_INT) return 0;
+   if (!v || TYPEOF(*v) != T_INT) return 0;
 
    *p=v->u.integer;
    return 1;
@@ -294,7 +294,7 @@ static int store_int_in_table(struct array *a,
    int i;
    int z=0;
    for (i=0; i<a->size && len; i++)
-      if (a->item[i].type==T_ARRAY)
+      if (TYPEOF(a->item[i]) == T_ARRAY)
       {
 	 int n;
 	 n=store_int_in_table(a->item[i].u.array,len,d);
@@ -302,7 +302,7 @@ static int store_int_in_table(struct array *a,
 	 len-=n;
 	 z+=n;
       }
-      else if (a->item[i].type==T_INT)
+      else if (TYPEOF(a->item[i]) == T_INT)
       {
 	 *(d++)=(unsigned int)(a->item[i].u.integer);
 	 len--;
@@ -323,14 +323,14 @@ static int parameter_qt(struct svalue *map,struct pike_string *what,
    v=low_mapping_string_lookup(map->u.mapping,what);
 
    if (!v) return 0;
-   else if (v->type!=T_MAPPING) 
+   else if (TYPEOF(*v) != T_MAPPING)
       Pike_error("Image.JPEG.encode: illegal value of option quant_table; expected mapping\n");
 
    md=v->u.mapping->data;
    NEW_MAPPING_LOOP(md)
       {
 	 int z;
-	 if (k->ind.type!=T_INT || k->val.type!=T_ARRAY)
+	 if (TYPEOF(k->ind) != T_INT || TYPEOF(k->val) != T_ARRAY)
 	    Pike_error("Image.JPEG.encode: illegal value of option quant_table; expected mapping(int:array)\n");
 
 	 if (k->ind.u.integer<0 || k->ind.u.integer>=NUM_QUANT_TBLS)
@@ -361,14 +361,14 @@ static int parameter_qt_d(struct svalue *map,struct pike_string *what,
    v=low_mapping_string_lookup(map->u.mapping,what);
 
    if (!v) return 0;
-   else if (v->type!=T_MAPPING) 
+   else if (TYPEOF(*v) != T_MAPPING)
       Pike_error("Image.JPEG.encode: illegal value of option quant_table; expected mapping\n");
 
    md=v->u.mapping->data;
    NEW_MAPPING_LOOP(md)
       {
 	 int z;
-	 if (k->ind.type!=T_INT || k->val.type!=T_ARRAY)
+	 if (TYPEOF(k->ind) != T_INT || TYPEOF(k->val) != T_ARRAY)
 	    Pike_error("Image.JPEG.encode: illegal value of option quant_table; expected mapping(int:array)\n");
 
 	 if (k->ind.u.integer<0 || k->ind.u.integer>=NUM_QUANT_TBLS)
@@ -419,19 +419,34 @@ static int parameter_marker(struct svalue *map,struct pike_string *what,
    v=low_mapping_string_lookup(map->u.mapping,what);
 
    if (!v) return 0;
-   else if (v->type!=T_MAPPING) 
+   else if (TYPEOF(*v) != T_MAPPING)
       Pike_error("Image.JPEG.encode: illegal value of option marker;"
 		 " expected mapping\n");
    md=v->u.mapping->data;
    NEW_MAPPING_LOOP(md)
       {
-	 if (k->ind.type!=T_INT || k->val.type!=T_STRING ||
-	     k->val.u.string->size_shift)
-	    Pike_error("Image.JPEG.encode: illegal value of option "
-		       "marker; expected mapping(int:8 bit string)\n");
-	 jpeg_write_marker(cinfo, k->ind.u.integer, 
-			   (const unsigned char *)k->val.u.string->str,
-			   k->val.u.string->len); 
+	 if (TYPEOF(k->ind) == T_INT) {
+	    if (TYPEOF(k->val) == T_STRING && !k->val.u.string->size_shift) {
+	       jpeg_write_marker(cinfo, k->ind.u.integer,
+				 (const unsigned char *)k->val.u.string->str,
+				 k->val.u.string->len);
+	       continue;
+	    } else if (TYPEOF(k->val) == T_ARRAY) {
+	       struct array *a = k->val.u.array;
+	       INT32 i;
+	       for (i = 0; i < a->size; i++) {
+		 struct pike_string *str = ITEM(a)[i].u.string;
+		 if (TYPEOF(ITEM(a)[i]) != T_STRING || str->size_shift)
+		   goto fail;
+		 jpeg_write_marker(cinfo, k->ind.u.integer,
+				   STR0(str), str->len);
+	       }
+	       continue;
+	    }
+	 }
+      fail:
+	 Pike_error("Image.JPEG.encode: illegal value of option marker; "
+		    "expected mapping(int:string(8bit)|array(string(8bit)))\n");
       }
 
    return 1;
@@ -445,7 +460,7 @@ static int parameter_comment(struct svalue *map,struct pike_string *what,
    v=low_mapping_string_lookup(map->u.mapping,what);
 
    if (!v) return 0;
-   else if (v->type!=T_STRING || v->u.string->size_shift) 
+   else if (TYPEOF(*v) != T_STRING || v->u.string->size_shift)
       Pike_error("Image.JPEG.encode: illegal value of option comment;"
 		 " expected 8 bit string\n");
 
@@ -494,7 +509,7 @@ static void my_skip_input_data(struct jpeg_decompress_struct *cinfo,
    sm->pub.bytes_in_buffer -= (size_t) num_bytes;
 }
 
-static void my_term_source(struct jpeg_decompress_struct *cinfo)
+static void my_term_source(struct jpeg_decompress_struct *UNUSED(cinfo))
 {
    /* nop */
 }
@@ -508,14 +523,14 @@ static int marker_exists_in_args(INT32 args, int which)
    struct keypair *k;
    v=low_mapping_string_lookup(map->u.mapping, param_comment);
    if (which==JPEG_COM && v) {
-     return (v->type==T_STRING && !v->u.string->size_shift);
+     return (TYPEOF(*v) == T_STRING && !v->u.string->size_shift);
    } else {
      v=low_mapping_string_lookup(map->u.mapping, param_marker);
-     if (v && v->type==T_MAPPING) {
+     if (v && TYPEOF(*v) == T_MAPPING) {
        md=v->u.mapping->data;
        NEW_MAPPING_LOOP(md)
        {
-	 if (k->ind.type==T_INT && k->val.type==T_STRING
+	 if (TYPEOF(k->ind) == T_INT && TYPEOF(k->val) == T_STRING
 	     && k->ind.u.integer==which && !k->val.u.string->size_shift) {
 	   return 1;
 	 }
@@ -656,7 +671,7 @@ void set_jpeg_transform_options(INT32 args, jpeg_transform_info *options)
  *!     Default is off for quality<25.
  *!   @member mapping(int:array(array(int))) "quant_tables"
  *!     Tune quantisation tables manually.
- *!   @member mapping(int:string) "marker"
+ *!   @member mapping(int:string(8bit)|array(string(8bit))) "marker"
  *!     Application and comment markers;
  *!     the integer should be one of @[Marker.COM], @[Marker.APP0],
  *!     @[Marker.APP1], ..., @[Marker.APP15]. The string is up to the application;
@@ -702,11 +717,11 @@ static void image_jpeg_encode(INT32 args)
    rgb_group *s;
    JSAMPROW row_pointer[8];
 
-   if (args<1 
-       || (sp[-args].type!=T_OBJECT && sp[-args].type!=T_STRING)
-       || (sp[-args].type==T_OBJECT &&
-	   !(img=(struct image*) get_storage(sp[-args].u.object,image_program)))
-       || (args>1 && sp[1-args].type!=T_MAPPING))
+   if (args<1
+       || (TYPEOF(sp[-args]) != T_OBJECT && TYPEOF(sp[-args]) != T_STRING)
+       || (TYPEOF(sp[-args]) == T_OBJECT &&
+	   !(img= get_storage(sp[-args].u.object,image_program)))
+       || (args>1 && TYPEOF(sp[1-args]) != T_MAPPING))
       Pike_error("Image.JPEG.encode: Illegal arguments\n");
 
    if (img) {
@@ -984,6 +999,8 @@ static void image_jpeg_encode(INT32 args)
  *!     JPEG quant tables.
  *!   @member int(0..100) "quality"
  *!     JPEG quality guess.
+ *!   @member mapping(int(0..255):string(8bit)|array(string(8bit))) "marker"
+ *!     Mapping from application and comment markers to their values.
  *! @endmapping
  */
 
@@ -1003,11 +1020,11 @@ static void img_jpeg_decode(INT32 args,int mode)
    rgb_group *d;
    JSAMPROW row_pointer[8];
 
-   int n=0,m;
+   int n=0;
 
-   if (args<1 
-       || sp[-args].type!=T_STRING
-       || (args>1 && sp[1-args].type!=T_MAPPING))
+   if (args<1
+       || TYPEOF(sp[-args]) != T_STRING
+       || (args>1 && TYPEOF(sp[1-args]) != T_MAPPING))
       Pike_error("Image.JPEG.decode: Illegal arguments\n");
 
    /* init jpeg library objects */
@@ -1055,9 +1072,12 @@ static void img_jpeg_decode(INT32 args,int mode)
 
    if (mode!=IMG_DECODE_IMAGE)
    {
+      struct mapping *markers;
+      int got_comment = 0;
+
       /* standard header info */
 
-      push_text("type"); n++;
+      ref_push_string(literal_type_string); n++;
       push_text("image/jpeg");
 
       push_text("xsize"); n++;
@@ -1115,31 +1135,64 @@ static void img_jpeg_decode(INT32 args,int mode)
       push_int(mds.cinfo.saw_Adobe_marker);
 
       ref_push_string(param_marker); n++;
-      m=0;
+      push_mapping(markers = allocate_mapping(mds.first_marker?10:0));
       while (mds.first_marker)
       {
 	 struct my_marker *mm=mds.first_marker;
+	 struct svalue *val;
 	 push_int(mm->id);
-	 push_string(make_shared_binary_string((char *)mm->data, mm->len));
-	 m++;
+	 if ((val = low_mapping_lookup(markers, Pike_sp-1))) {
+	    if (TYPEOF(*val) == T_STRING) {
+	       /* Both values are strings. */
+	       ref_push_string(val->u.string);
+	       push_string(make_shared_binary_string((char *)mm->data,
+						     mm->len));
+	       f_aggregate(2);
+	    } else {
+	       struct array *old = val->u.array;
+	       struct array *new;
+	       push_string(make_shared_binary_string((char *)mm->data,
+						     mm->len));
+
+	       /* Make sure not to free the old array in place
+		* on realloc, as that would invalidate the mapping.
+		*/
+	       add_ref(old);
+
+	       new = append_array(old, Pike_sp-1);
+	       pop_stack();
+	       if (new == old) {
+		 /* Array modified in place.
+		  * No need to reassign it.
+		  * Remove the extra reference from above.
+		  */
+		 free_array(old);
+		 goto next_marker;
+	       }
+	       /* The reference we added to old has migrated to new.
+		* Pass it along to the stack.
+		*/
+	       push_array(new);
+	    }
+	 } else {
+	    push_string(make_shared_binary_string((char *)mm->data, mm->len));
+	 }
+	 mapping_insert(markers, Pike_sp-2, Pike_sp-1);
+	 if ((mm->id == JPEG_COM) && !got_comment) {
+	    /* First comment - Add the comment to the main mapping too. */
+	    stack_swap();
+	    pop_stack();	/* Zap mm->id from the stack. */
+	    ref_push_string(param_comment);
+	    stack_swap();	/* And replace it with "comment". */
+	    n++;		/* One more element for the main mapping. */
+	    got_comment = 1;
+	 } else {
+	    pop_n_elems(2);
+	 }
+      next_marker:
 	 mds.first_marker=mm->next;
 	 free(mm);
       }
-      f_aggregate_mapping(m*2);
-
-      if (m)
-      {
-	 stack_dup();
-	 push_int(JPEG_COM);
-	 f_index(2);
-	 if (sp[-1].type==T_STRING) 
-	 {
-	    ref_push_string(param_comment); n++;
-	    stack_swap();
-	 }
-	 else
-	    pop_stack();
-      }
    }
 
    while (mds.first_marker)
@@ -1157,7 +1210,7 @@ static void img_jpeg_decode(INT32 args,int mode)
       bytes_per_pixel = mds.cinfo.output_components;
 
       o=clone_object(image_program,0);
-      img=(struct image*)get_storage(o,image_program);
+      img=get_storage(o,image_program);
       if (!img) Pike_error("image no image? foo?\n"); /* should never happen */
       img->img=malloc(bytes_per_pixel *
 		      mds.cinfo.output_width*mds.cinfo.output_height);
@@ -1341,7 +1394,7 @@ void image_jpeg_quant_tables(INT32 args)
    if (args)
    {
       INT_TYPE q;
-      get_all_args("Image.JPEG.quant_tables",args,"%i",&q);
+      get_all_args("quant_tables",args,"%i",&q);
       jpeg_set_quality(&cinfo,q,0);
    }
 
@@ -1502,7 +1555,7 @@ PIKE_MODULE_EXIT
 PIKE_MODULE_INIT
 {
 #ifdef HAVE_JPEGLIB_H
-#ifdef DYNAMIC_MODULE
+#ifndef FAKE_DYNAMIC_LOAD
     image_program = PIKE_MODULE_IMPORT(Image, image_program);
     if(!image_program) {
        yyerror("Could not load Image module.");
