@@ -18,7 +18,7 @@
 
 #define sp Pike_sp
 
-static void exit_blob_struct( );
+static void exit_blob_struct(struct object *o);
 
 /*
   +-----------+----------+---------+---------+---------+
@@ -273,13 +273,27 @@ static void _append_blob( struct blob_data *d, struct pike_string *s )
   {
     int docid = wf_buffer_rint( b );
     int nhits = wf_buffer_rbyte( b );
-    struct hash *h = find_hash( d, docid );
+    int remaining = b->size - b->rpos;
+    struct hash *h;
+    if (nhits > (remaining>>1)) {
+      fprintf(stderr, "Invalid blob entry for doc 0x%08x: %d hits of %d missing.\n",
+	      (unsigned INT32)docid, nhits - (remaining>>1), nhits);
+      nhits = remaining>>1;
+      remaining = -1;
+    }
+    if (!nhits) {
+      fprintf(stderr, "Invalid blob entry for document 0x%08x (0 hits!).\n",
+	      (unsigned INT32)docid);
+      break;
+    }
+    h = find_hash( d, docid );
     /* Make use of the fact that this dochash should be empty, and
      * assume that the incoming data is valid
      */
-    wf_buffer_rewind_r( b, 5 );
-    wf_buffer_rewind_w( h->data, -1 );
-    wf_buffer_memcpy( h->data, b, nhits*2+5 );    
+    wf_buffer_rewind_w( h->data, 1 );
+    wf_buffer_wbyte(h->data, nhits);
+    wf_buffer_memcpy( h->data, b, nhits*2 );
+    if (remaining < 0) break;
   }
   wf_buffer_free( b );
 }
@@ -518,18 +532,35 @@ static void f_blob__cast( INT32 args )
   /* 2: Sort in the blobs */
   for( i = 0; i<zp; i++ )
   {
-    int nh;
+    unsigned int nh;
 #ifdef PIKE_DEBUG
-    if( zipp[i].b->size < 7 )
-      fatal( "Expected at least 7 bytes! (1 hit)\n");
-    else
+    /* NB: As new_hash() returns a zero-length hash struct (ie 5 bytes),
+     *     requiring 7 bytes is a bit excessive.
+     */
+    if( zipp[i].b->size < 5 ) {
+      fprintf(stderr, "zipp[%d]: doc_id: %d, b: %p (%d bytes)\n",
+	      i, zipp[i].id, zipp[i].b->data, zipp[i].b->size);
+      Pike_fatal( "Expected at least 5 bytes! (0 hits)\n");
+    }
 #endif
     if((nh = zipp[i].b->data[4]) > 1 )
     {
 #ifdef HANDLES_UNALIGNED_READ
+#ifdef PIKE_DEBUG
+      if (nh<<1 != zipp[i].b->size-5) {
+	Pike_fatal("Unexpected blob size: %d != %d\n",
+		   nh<<1, zipp[i].b->size-5);
+      }
+#endif
       fsort( zipp[i].b->data+5, nh, 2, (void *)cmp_hit );
 #else
       short *data = malloc( nh * 2 );
+#ifdef PIKE_DEBUG
+      if (nh<<1 != zipp[i].b->size-5) {
+	Pike_fatal("Unexpected blob size: %d != %d\n",
+		   nh<<1, zipp[i].b->size-5);
+      }
+#endif
       memcpy( data,  zipp[i].b->data+5, nh * 2 );
       fsort( data, nh, 2, (void *)cmp_hit );
       memcpy( zipp[i].b->data+5, data, nh * 2 );
@@ -546,7 +577,7 @@ static void f_blob__cast( INT32 args )
 
   free( zipp );
 
-  exit_blob_struct(); /* Clear this buffer */
+  exit_blob_struct(NULL); /* Clear this buffer */
   pop_n_elems( args );
   push_string( make_shared_binary_string( (char *)res->data, res->size ) );
   wf_buffer_free( res );
@@ -563,14 +594,17 @@ static void init_blob_struct(struct object * UNUSED(o))
   memset( THIS, 0, sizeof( struct blob_data ) );
 }
 
-static void exit_blob_struct(struct object * o)
+/* NB: This function is called directly from f_blob__cast() above,
+ *     and thus needs to be prepared to be called again.
+ */
+static void exit_blob_struct(struct object * UNUSED(o))
 {
   int i;
   for( i = 0; i<HSIZE; i++ )
     if( THIS->hash[i] )
       free_hash( THIS->hash[i] );
-  /* why? */
-  init_blob_struct(o);
+  /* Prepare for next use. */
+  init_blob_struct(NULL);
 }
 
 void init_blob_program(void)
