@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: ff60005a17050c2b0592a52bfac72dd2e63c395b $
 */
 
 #ifndef PEEP_H
@@ -14,7 +13,7 @@ extern dynamic_buffer instrbuf;
 struct p_instr_s
 {
   short opcode;
-  short line;
+  INT_TYPE line;
   struct pike_string *file;
   INT32 arg;
   INT32 arg2;
@@ -28,13 +27,13 @@ void exit_bytecode(void);
 ptrdiff_t insert_opcode2(unsigned int f,
 			 INT32 a,
 			 INT32 b,
-			 INT32 current_line,
+			 INT_TYPE current_line,
 			 struct pike_string *current_file);
 ptrdiff_t insert_opcode1(unsigned int f,
 			 INT32 a,
-			 INT32 current_line,
+			 INT_TYPE current_line,
 			 struct pike_string *current_file);
-ptrdiff_t insert_opcode0(int f,int current_line,
+ptrdiff_t insert_opcode0(int f, INT_TYPE current_line,
 			 struct pike_string *current_file);
 void update_arg(int instr,INT32 arg);
 INT32 assemble(int store_linenumbers);
