@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: a38ec95039fad1bc7460c27e2d7907fe7e5b839d $
 */
 
 /* The basic IDEA transformation
@@ -171,7 +170,7 @@ idea_invert(unsigned INT16 *d,
   *--p = t2;
   *--p = t1;
   /* Copy and destroy temp copy */
-  MEMCPY(d, temp, sizeof(temp));
+  memcpy(d, temp, sizeof(temp));
   memset(temp, 0, sizeof(temp));
 } /* idea_invert */
 
